(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    320844,      10627]
NotebookOptionsPosition[    259236,       8880]
NotebookOutlinePosition[    281114,       9357]
CellTagsIndexPosition[    280073,       9320]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Number Theory and Cryptography", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 4}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes numerous functions for exploring number theory. In this chapter \
we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s computational abilities to compute and solve congruences, represent \
integers in bases other than ten, explore arithmetic algorithms in those \
bases, check whether or not a number is prime, and compute discrete \
logarithms. We will also see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help explore several of the applications described in the textbook, in \
particular, hashing functions, pseudorandom numbers, check digits, and, of \
course, cryptography. "
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.1 Divisibility and Modular Arithmetic", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to explore divisibility of integers and modular arithmetic. We will see \
how to compute quotients and remainders in integer division, how to test \
integers for the divisibility relationship, and how to perform computations \
in modular arithmetic. This section will conclude with an illustration of how \
to create infix addition and multiplication operators for modular arithmetic \
and a demonstration of how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to compute addition and multiplication tables. "
}], "Text"],

Cell[CellGroupData[{

Cell["Quotient, Remainder, and Divisibility", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s functions ",
 StyleBox[ButtonBox["Quotient",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Quotient"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " compute the quotient and remainder, respectively, obtained when you divide \
two integers. For example, consider 99 divided by 13."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Quotient", "[", 
  RowBox[{"99", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["7"], "Output",
 CellChangeTimes->{3.580648232304274*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"99", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["8"], "Output",
 CellChangeTimes->{3.580648232444497*^9}]
}, Open  ]],

Cell[TextData[{
 "These results indicate that 99 divided by 13 results in a quotient of 7 and \
a remainder of 8. That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"99", "=", 
    RowBox[{
     RowBox[{"13", "\[CenterDot]", "7"}], "+", "8"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Note that the textbook uses the notation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "mod", " ", "m"}], TraditionalForm]]],
 " to represent the remainder when ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is divided by ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", using ",
 Cell[BoxData[
  FormBox["mod", TraditionalForm]]],
 " like an operator. To enter this in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", you must use the functional notation ",
 Cell[BoxData[
  RowBox[{"Mod", "[", 
   StyleBox["a", "CodeVar"], ",", 
   StyleBox["m", "CodeVar"], "]"}]], "myCode"],
 "."
}], "Text"],

Cell[TextData[{
 "Another function, ",
 StyleBox[ButtonBox["QuotientRemainder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/QuotientRemainder"], "FunctionLink"],
 ", produces a list with first element the quotient and second element the \
remainder."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuotientRemainder", "[", 
  RowBox[{"99", ",", "13"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7", ",", "8"}], "}"}]], "Output",
 CellChangeTimes->{3.580648232488698*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Checking Divisibility", "Subsubsection"],

Cell[TextData[{
 "To test whether one integer divides another, you can, of course, check to \
see if the remainder is 0 or not. For example, the following shows that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "|", "132"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"132", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.580648232540496*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides the function ",
 StyleBox[ButtonBox["Divisible",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Divisible"], "FunctionLink"],
 " for testing divisibility.  The ",
 StyleBox[ButtonBox["Divisible",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Divisible"], "FunctionLink"],
 " function returns true if its first argument is divisible by the second."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Divisible", "[", 
  RowBox[{"132", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648232589855*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Divisible", "[", 
  RowBox[{"99", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.5806482326414547`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Offset", "Subsubsection"],

Cell[TextData[{
 "Recall from the division algorithm that the remainder must always be \
nonnegative, even when the dividend is negative. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " function respects that convention by default, with ",
 Cell[BoxData[
  RowBox[{"Mod", "[", 
   StyleBox["a", "CodeVar"], ",", 
   StyleBox["m", "CodeVar"], "]"}]], "myCode"],
 " returning a value between ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "-", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{
   RowBox[{"-", "27"}], ",", "5"}], "]"}]], "Input"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.580648232690185*^9}]
}, Open  ]],

Cell[TextData[{
 "However, there are times when it is more useful to allow negative values. \
For example, consider the following question: \[OpenCurlyDoubleQuote]It is \
now 11:00 AM. What time will it be 142 hours from \
now?\[CloseCurlyDoubleQuote] If we compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"142", " ", "mod", " ", "24"}], TraditionalForm]]],
 ", "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"142", ",", "24"}], "]"}]], "Input"],

Cell[BoxData["22"], "Output",
 CellChangeTimes->{3.580648232739316*^9}]
}, Open  ]],

Cell[TextData[{
 "We see that the time will be the same 142 hours from now as 22 hours from \
now. But it's also the case that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"142", "\[Congruent]", 
    RowBox[{
     RowBox[{"-", "2"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "24"}], ")"}]}]}], TraditionalForm]]],
 ", which means that the time 142 hours from now is the same as the time 2 \
hours earlier, i.e., 9:00 AM. You can see that this congruence is somewhat \
more convenient. "
}], "Text"],

Cell[TextData[{
 "To support this, the ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " function accepts an optional third argument, called the offset. The offset \
specifies a lower bound on the ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " function\[CloseCurlyQuote]s output. Specifically, for an integral offset \
",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  RowBox[{"Mod", "[", 
   StyleBox["a", "CodeVar"], ",", 
   StyleBox["m", "CodeVar"], ",", 
   StyleBox["d", "CodeVar"], "]"}]], "myCode"],
 " will return the value congruent to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", " ", "mod", " ", "m"}], TraditionalForm]]],
 " that lies between ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "+", "m", "-", "1"}], TraditionalForm]]],
 ". For example, by specifying an offset of 1, the result will be betwen 1 \
and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", so a computation that would ordinarily return 0 will return the value of \
the modulus instead."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"132", ",", "3", ",", "1"}], "]"}]], "Input"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.580648232793281*^9}]
}, Open  ]],

Cell[TextData[{
 "The value 1 is a common offset, as it ensures a positive result that can \
then be used as an index into a list. Another common offset is the fraction \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "m"}], "/", "2"}], TraditionalForm]]],
 ". This offset ensures that the output is the integer closest to 0 that is \
congruent to the first argument modulo the second. For example, returning to \
the time example above, to obtain the result ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "2"}], TraditionalForm]]],
 " we use the offset ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "24"}], "/", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"142", ",", "24", ",", 
   RowBox[{
    RowBox[{"-", "24"}], "/", "2"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "2"}]], "Output",
 CellChangeTimes->{3.580648232839851*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Congruences", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 "The first argument to ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " can be any algebraic expression. For example, you can compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", 
    RowBox[{
     RowBox[{"4", "\[CenterDot]", 
      SuperscriptBox["9", "2"]}], "mod", " ", "5"}]}], TraditionalForm]]],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{
   RowBox[{"3", "+", 
    RowBox[{"4", "*", 
     RowBox[{"9", "^", "2"}]}]}], ",", "5"}], "]"}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.580648232890594*^9}]
}, Open  ]],

Cell[TextData[{
 "To test a congruence, for example to confirm that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"428", "\[Congruent]", 
    RowBox[{"530", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "17"}], ")"}]}]}], TraditionalForm]]],
 ", you must apply the ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " function to both values and test them using the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") relation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Mod", "[", 
   RowBox[{"428", ",", "17"}], "]"}], "\[Equal]", 
  RowBox[{"Mod", "[", 
   RowBox[{"530", ",", "17"}], "]"}]}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648232938393*^9}]
}, Open  ]],

Cell[TextData[{
 "You may not include the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") relation within the argument to ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["Solving Congruences", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can solve congruences by using the ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function in conjunction with the ",
 StyleBox[ButtonBox["Modulus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Modulus"], "FunctionLink"],
 " option. To do so, give the congruence or list of simultaneous congruences \
as the first argument to ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " and identify the modulus using the ",
 StyleBox[ButtonBox["Modulus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Modulus"], "FunctionLink"],
 " option. As an example, consider Exercise 13a from Section 4.1 of the \
textbook. Under the assumption that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[Congruent]", 
    RowBox[{"4", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "13"}], ")"}]}]}], TraditionalForm]]],
 ", we are to solve ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[Congruent]", 
    RowBox[{"9", "a", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "13"}], ")"}]}]}], TraditionalForm]]],
 ". We solve this using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[Equal]", "4"}], ",", 
     RowBox[{"c", "\[Equal]", 
      RowBox[{"9", "*", "a"}]}]}], "}"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "13"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"a", "\[Rule]", "4"}], ",", 
    RowBox[{"c", "\[Rule]", "10"}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.580648232991681*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that the congruences are entered as a list. This tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that they must be simultaneously satisfied. Also observe that we must use \
the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") relation, not ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 "), when specifying the congruences."
}], "Text"],

Cell[TextData[{
 "If there are no solutions to the congruence, then ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " will return the empty list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"n", "^", "2"}], "\[Equal]", "3"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "4"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output",
 CellChangeTimes->{3.580648233028471*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
 "Arithmetic Modulo ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]]
}], "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "In this subsection we\[CloseCurlyQuote]ll define operators based on the \
definitions of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["+", "m"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[CenterDot]", "m"], TraditionalForm]]],
 " given in the text. Our goal will be to get as close as possible to being \
able to enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", 
    SubscriptBox["+", "11"], "9"}], TraditionalForm]]],
 " and have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " return 5."
}], "Text"],

Cell[TextData[{
 "The usual style of writing arithmetic operators in between the operands is \
referred to as infix notation. There are operators that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " recognizes as infix operators, but which do not have built-in definitions. \
We can take advantage of this to create our own infix operators by providing \
definitions for these undefined operators."
}], "Text"],

Cell[TextData[{
 "You can see all of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s operators in the table of operator precedence in the ",
 ButtonBox["Operator Input Forms",
  BaseStyle->"Link",
  ButtonData->"paclet:tutorial/OperatorInputForms"],
 " tutorial. Those operators with a triangular mark in the far right of the \
table have built-in definitions. Those without such a mark are available for \
your own use. Here, we will make use of the ",
 StyleBox[ButtonBox["CirclePlus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CirclePlus"], "FunctionLink"],
 " (",
 StyleBox["\[CirclePlus]", "Input"],
 ") and ",
 StyleBox[ButtonBox["CircleTimes",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CircleTimes"], "FunctionLink"],
 " (",
 StyleBox["\[CircleTimes]", "Input"],
 ") operators."
}], "Text"],

Cell["\<\
To enter these as infix operators, you need to use their aliases. To enter \
\[CirclePlus], type \[EscapeKey]c+\[EscapeKey], and to enter \[CircleTimes], \
type \[EscapeKey]c*\[EscapeKey].\
\>", "Text"],

Cell[TextData[{
 "Observe that if you enter an expression using one of the operators, and \
apply the ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " function, you can see that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " interprets the infix operator as an application of the named function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"2", "\[CirclePlus]", "3"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"CirclePlus", "[", 
    RowBox[{"2", ",", "3"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.580648233073674*^9}]
}, Open  ]],

Cell[TextData[{
 "Also note that while these functions are currently undefined, they do have \
a defined precedence. So, for example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "\[CirclePlus]", 
    RowBox[{"3", "\[CircleTimes]", "4"}]}], TraditionalForm]]],
 " is interpreted as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "\[CirclePlus]", 
    RowBox[{"(", 
     RowBox[{"3", "\[CircleTimes]", "4"}], ")"}]}], TraditionalForm]]],
 ", which can be verified by inspecting the functional expression obtained \
with ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"2", "\[CirclePlus]", 
   RowBox[{"3", "\[CircleTimes]", "4"}]}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"CirclePlus", "[", 
    RowBox[{"2", ",", 
     RowBox[{"CircleTimes", "[", 
      RowBox[{"3", ",", "4"}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.580648233124137*^9}]
}, Open  ]],

Cell[TextData[{
 "To define the operator, you can issue the definition using either the \
functional or infix form. Below, we define addition in functional form and \
multiplication in infix form. Regardless of the form you provide the \
definition, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will properly evaluate both forms."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CirclePlus", "[", 
   RowBox[{"a_", ",", "b_"}], "]"}], ":=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", "11"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"a_", "\[CircleTimes]", "b_"}], ":=", 
  RowBox[{"Mod", "[", 
   RowBox[{
    RowBox[{"a", "*", "b"}], ",", "11"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Now we can compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"7", 
    SubscriptBox["+", "11"], "9"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", 
    SubscriptBox["\[CenterDot]", "11"], "5"}], TraditionalForm]]],
 " using the \[CirclePlus] and \[CircleTimes] operators."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"7", "\[CirclePlus]", "9"}]], "Input"],

Cell[BoxData["5"], "Output",
 CellChangeTimes->{3.580648233203874*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"3", "\[CircleTimes]", "5"}]], "Input"],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.580648233240653*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Addition and Multiplication Tables", "Subsection",
 CellTags->"Section1d"],

Cell["\<\
We conclude this section by producing addition and multiplication tables.\
\>", "Text"],

Cell[TextData[{
 "We will create the tables, unsurprisingly, using the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function. The first argument will be the sum or product of two variables \
within the ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " repetition arguments will specify that the variables range from 0 to one \
less than the modulus. A call to ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 " will make the result readable."
}], "Text"],

Cell["Here is the addition table modulo 5.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"Mod", "[", 
     RowBox[{
      RowBox[{"a", "+", "b"}], ",", "5"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "0", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"b", ",", "0", ",", "4"}], "}"}]}], "]"}], "//", 
  "TableForm"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {"0", "1", "2", "3", "4"},
    {"1", "2", "3", "4", "0"},
    {"2", "3", "4", "0", "1"},
    {"3", "4", "0", "1", "2"},
    {"4", "0", "1", "2", "3"}
   },
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}},
   GridBoxSpacings->{"Columns" -> {
       Offset[0.27999999999999997`], {
        Offset[2.0999999999999996`]}, 
       Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
       Offset[0.2], {
        Offset[0.4]}, 
       Offset[0.2]}, "RowsIndexed" -> {}}],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.580648233279393*^9}]
}, Open  ]],

Cell[TextData[{
 "We can improve the table a bit more by adding column and row headings. This \
is done using the ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " option to ",
 StyleBox[ButtonBox["TableForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableForm"], "FunctionLink"],
 ". By setting the ",
 StyleBox[ButtonBox["TableHeadings",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TableHeadings"], "FunctionLink"],
 " option to a list consisting of two sublists corresponding to the desired \
labels for the rows and then the columns, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will display those labels as headings. The keywork ",
 StyleBox[ButtonBox["None",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/None"], "FunctionLink"],
 " can be used in place of one of the sublists so as to omit the \
corresponding set of labels."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       RowBox[{"a", "+", "b"}], ",", "5"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"a", ",", "0", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "0", ",", "4"}], "}"}]}], "]"}], ",", 
   RowBox[{"TableHeadings", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "2", ",", "3", ",", "4"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1", ",", "2", ",", "3", ",", "4"}], "}"}]}], 
     "}"}]}]}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      StyleBox["\[Null]",
       ShowStringCharacters->False], 
      TagBox["0",
       HoldForm], 
      TagBox["1",
       HoldForm], 
      TagBox["2",
       HoldForm], 
      TagBox["3",
       HoldForm], 
      TagBox["4",
       HoldForm]},
     {
      TagBox["0",
       HoldForm], "0", "1", "2", "3", "4"},
     {
      TagBox["1",
       HoldForm], "1", "2", "3", "4", "0"},
     {
      TagBox["2",
       HoldForm], "2", "3", "4", "0", "1"},
     {
      TagBox["3",
       HoldForm], "3", "4", "0", "1", "2"},
     {
      TagBox["4",
       HoldForm], "4", "0", "1", "2", "3"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxDividers->{
     "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
      "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   {OutputFormsDump`HeadedRows, OutputFormsDump`HeadedColumns}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {{0, 1, 2, 3, 4}, {0, 1, 2, 3, 4}}]]]], "Output",
 CellChangeTimes->{3.580648233323344*^9}]
}, Open  ]],

Cell["\<\
Using that example as a model, it is easy to create general functions that \
will accept a modulus and display the addition or multiplication table.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"AdditionTable", "[", "m_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"TableForm", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"a", "+", "b"}], ",", "m"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"a", ",", "0", ",", 
          RowBox[{"m", "-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b", ",", "0", ",", 
          RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"TableHeadings", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", 
           RowBox[{"m", "-", "1"}]}], "]"}], ",", 
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", 
           RowBox[{"m", "-", "1"}]}], "]"}]}], "}"}]}]}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"MultiplicationTable", "[", "m_", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"TableForm", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Mod", "[", 
         RowBox[{
          RowBox[{"a", "*", "b"}], ",", "m"}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"a", ",", "0", ",", 
          RowBox[{"m", "-", "1"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"b", ",", "0", ",", 
          RowBox[{"m", "-", "1"}]}], "}"}]}], "]"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"TableHeadings", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", 
           RowBox[{"m", "-", "1"}]}], "]"}], ",", 
         RowBox[{"Range", "[", 
          RowBox[{"0", ",", 
           RowBox[{"m", "-", "1"}]}], "]"}]}], "}"}]}]}], "]"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["Here is the multiplication table modulo 5.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplicationTable", "[", "5", "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      StyleBox["\[Null]",
       ShowStringCharacters->False], 
      TagBox["0",
       HoldForm], 
      TagBox["1",
       HoldForm], 
      TagBox["2",
       HoldForm], 
      TagBox["3",
       HoldForm], 
      TagBox["4",
       HoldForm]},
     {
      TagBox["0",
       HoldForm], "0", "0", "0", "0", "0"},
     {
      TagBox["1",
       HoldForm], "0", "1", "2", "3", "4"},
     {
      TagBox["2",
       HoldForm], "0", "2", "4", "1", "3"},
     {
      TagBox["3",
       HoldForm], "0", "3", "1", "4", "2"},
     {
      TagBox["4",
       HoldForm], "0", "4", "3", "2", "1"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxDividers->{
     "Columns" -> {False, True, {False}, False}, "ColumnsIndexed" -> {}, 
      "Rows" -> {False, True, {False}, False}, "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[2.0999999999999996`]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   {OutputFormsDump`HeadedRows, OutputFormsDump`HeadedColumns}],
  Function[BoxForm`e$, 
   TableForm[
   BoxForm`e$, 
    TableHeadings -> {{0, 1, 2, 3, 4}, {0, 1, 2, 3, 4}}]]]], "Output",
 CellChangeTimes->{3.580648233502535*^9}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.2 Integer Representations and Algorithms", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to explore representations of integers in various bases and to \
explore algorithms for computing with integers. We will begin by looking at \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s built-in functions for converting between bases. Then we'll focus our \
attention on binary representations of integers and how to implement \
algorithms for addition and multiplication on binary representations. We \
restrict our attention to positive integers throught this section. "
}], "Text"],

Cell[CellGroupData[{

Cell["Base Conversion", "Subsection",
 CellTags->"Section2a"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides support for converting from one base representation to another \
via the functions ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " function is used to convert a positive integer, expressed in base ten, \
into a list of the digits of the integer expressed in a specified base. The \
first argument to ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " is the base ten integer. If no other arguments are provided, the function \
returns the list of the digits of the integer. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", "1234", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output",
 CellChangeTimes->{3.580648233537799*^9}]
}, Open  ]],

Cell["\<\
Note that the most significant digit is first in the list and the least \
significant is last. That is, the \
\[OpenCurlyDoubleQuote]one\[CloseCurlyQuote]s digit\[CloseCurlyDoubleQuote] \
is the last element in the list.\
\>", "Text"],

Cell[TextData[{
 "By providing a base as a second argument, the ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " function will output the list of the digits in the representation of the \
integer in that base. The following indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{"(", "1234", ")"}], "10"], "=", 
    SubscriptBox[
     RowBox[{"(", "103102", ")"}], "4"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"1234", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "3", ",", "1", ",", "0", ",", "2"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648233575119*^9}]
}, Open  ]],

Cell[TextData[{
 "For bases larger than ten, ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " does not make use of letters for digits with values larger than 9. Rather, \
it uses the base ten representation of the value of such digits. For example, \
in Example 5 of the textbook, it is shown that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{"(", "177130", ")"}], "10"], "=", 
    SubscriptBox[
     RowBox[{"(", 
      RowBox[{"2", "B3EA"}], ")"}], "16"]}], TraditionalForm]]],
 ". The output of ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " reports 10, 11, and 14 rather than A, B, and E."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"177130", ",", "16"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "11", ",", "3", ",", "14", ",", "10"}], "}"}]], "Output",
 CellChangeTimes->{3.580648233624935*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " function also accepts a third argument to specify a minimum length of the \
output. If the representation of the integer in the given base has fewer \
digits than specified by the third argument, zeros will be added on the left. \
The following shows the representation of ",
 Cell[BoxData[
  FormBox["123", TraditionalForm]]],
 " in binary (base 2), with ten digits. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerDigits", "[", 
  RowBox[{"123", ",", "2", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "1", ",", "1", ",", "1", ",", "1", ",", "0", 
   ",", "1", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.58064823367507*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " function is very similar to ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 ", accepting the same arguments and having a very similar effect. The \
difference is that the output of ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " is a string rather than a list. This means that the output has a more \
typical appearance. Contrast the output below to the corresponding example \
above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerString", "[", 
  RowBox[{"1234", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["\<\"103102\"\>"], "Output",
 CellChangeTimes->{3.580648233760706*^9}]
}, Open  ]],

Cell["\<\
Note that, despite appearances, the output is in fact a string object, and \
not a numerical object.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Head", "[", "%", "]"}]], "Input"],

Cell[BoxData["String"], "Output",
 CellChangeTimes->{3.580648233806097*^9}]
}, Open  ]],

Cell[TextData[{
 "As a result, you cannot manipulate the output of ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " using numerical operators. The ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " function is much more useful if you wish to work with the output. However, \
",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " produces much more readable output. It also follows the usual convention \
of using letters for digits larger than 9."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerString", "[", 
  RowBox[{"177130", ",", "16"}], "]"}]], "Input"],

Cell[BoxData["\<\"2b3ea\"\>"], "Output",
 CellChangeTimes->{3.580648233859168*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " functions are both used to take a base ten representation of a positive \
integer and return a representation in another base. For the reverse, we use \
the ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 " function."
}], "Text"],

Cell[TextData[{
 "The first argument of ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 " can be either a list, like the output of ",
 StyleBox[ButtonBox["IntegerDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerDigits"], "FunctionLink"],
 ", or a string, like the output of ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 ". With no second argument, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will assume that the input is intended to be base ten and will convert the \
list or string into the corresponding integer."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], "]"}]], "Input"],

Cell[BoxData["1234"], "Output",
 CellChangeTimes->{3.5806482339109163`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", "\"\<1234\>\"", "]"}]], "Input"],

Cell[BoxData["1234"], "Output",
 CellChangeTimes->{3.5806482339575777`*^9}]
}, Open  ]],

Cell[TextData[{
 "The second argument specifies the base in which the first argument is \
given. For example, to convert ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", "103012", ")"}], "4"], TraditionalForm]]],
 " back to base ten, you enter either of the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "3", ",", "1", ",", "0", ",", "2"}], "}"}], 
   ",", "4"}], "]"}]], "Input"],

Cell[BoxData["1234"], "Output",
 CellChangeTimes->{3.580648233991579*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{"\"\<103102\>\"", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["1234"], "Output",
 CellChangeTimes->{3.5806482340466833`*^9}]
}, Open  ]],

Cell[TextData[{
 "For bases larger than ten, you can use letters to represent digits larger \
than nine when using a string as the first argument to ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 ". Note that either lower or upper case are acceptable."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{"\"\<2B3EA\>\"", ",", "16"}], "]"}]], "Input"],

Cell[BoxData["177130"], "Output",
 CellChangeTimes->{3.580648234093193*^9}]
}, Open  ]],

Cell[TextData[{
 "Finally, note that both ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 " can, in place of the base, accept the string ",
 StyleBox["\[OpenCurlyDoubleQuote]Roman\[CloseCurlyDoubleQuote]", "Input"],
 ", in which case the function will convert to or from a Roman numeral \
representation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerString", "[", 
  RowBox[{"2013", ",", "\"\<Roman\>\""}], "]"}]], "Input"],

Cell[BoxData["\<\"MMXIII\"\>"], "Output",
 CellChangeTimes->{3.5806482341451387`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{"\"\<MMXIII\>\"", ",", "\"\<Roman\>\""}], "]"}]], "Input"],

Cell[BoxData["2013"], "Output",
 CellChangeTimes->{3.580648234191319*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["BaseForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BaseForm"], "FunctionLink"],
 " function should also be mentioned. Unlike the functions above, ",
 StyleBox[ButtonBox["BaseForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BaseForm"], "FunctionLink"],
 " only affects how a number is displayed by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", not its type. That is, ",
 StyleBox[ButtonBox["BaseForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BaseForm"], "FunctionLink"],
 " does not change an integer into a list or string, it only displays it as \
if it were in a different base. Its arguments are a base ten integer and a \
base."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BaseForm", "[", 
  RowBox[{"177130", ",", "16"}], "]"}]], "Input"],

Cell[BoxData[
 TagBox[
  InterpretationBox[
   SubscriptBox["\<\"2b3ea\"\>", "\<\"16\"\>"],
   177130,
   Editable->False],
  BaseForm[#, 16]& ]], "Output",
 CellChangeTimes->{3.5806482342242947`*^9}]
}, Open  ]],

Cell[TextData[{
 "In the other direction, you can use the double-caret (",
 StyleBox["^^", "Input"],
 ") notation to enter a number in a specified base. Enter the base, followed \
by the representation of the number in that base. For example, to enter ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", "A3", ")"}], "16"], TraditionalForm]]],
 " you type the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["16^^a3"], "Input"],

Cell[BoxData["163"], "Output",
 CellChangeTimes->{3.580648234276291*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converting Between Two Non-ten Bases", "Subsubsection"],

Cell[TextData[{
 "Given a positive integer in a base other than ten, you can convert it to \
another by using base ten as an intermediary. For example, to convert ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", "123", ")"}], "5"], TraditionalForm]]],
 " to base 3, you proceed as follows. First convert to base ten using ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromDigits", "[", 
  RowBox[{"\"\<123\>\"", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["38"], "Output",
 CellChangeTimes->{3.5806482343276463`*^9}]
}, Open  ]],

Cell[TextData[{
 "Then use ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " to convert to base 3."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerString", "[", 
  RowBox[{"38", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["\<\"1102\"\>"], "Output",
 CellChangeTimes->{3.580648234412279*^9}]
}, Open  ]],

Cell[TextData[{
 "The result indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox[
     RowBox[{"(", "123", ")"}], "5"], "=", 
    SubscriptBox[
     RowBox[{"(", "1102", ")"}], "3"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We can combine these two steps into a single function that takes as its \
arguments a string representing the integer, the starting base, and the final \
base. The body of the function is simply a composition of ",
 StyleBox[ButtonBox["FromDigits",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FromDigits"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"convertString", "[", 
   RowBox[{"n_", ",", "b1_", ",", "b2_"}], "]"}], ":=", 
  RowBox[{"IntegerString", "[", 
   RowBox[{
    RowBox[{"FromDigits", "[", 
     RowBox[{"n", ",", "b1"}], "]"}], ",", "b2"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"convertString", "[", 
  RowBox[{"\"\<123\>\"", ",", "5", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["\<\"1102\"\>"], "Output",
 CellChangeTimes->{3.580648234590979*^9}]
}, Open  ]],

Cell["\<\
It is left to the reader to define a similar function that outputs a list of \
digits rather than a string.\
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Binary Addition", "Subsection",
 CellTags->"Section2b"],

Cell[TextData[{
 "In this subsection, we will implement Algorithm 2 from Section 4.2, \
addition of integers. Our function will accept two binary representations \
given as lists of 0s and 1s with the most significant digit first. The first \
task for our function will be to make sure that the binary representations \
are of the same length. To do this, we compute the maximum of the lengths of \
the two lists, which will be stored as ",
 StyleBox["n", "Input"],
 ", and then add as many 0s to the list as are necessary to make both lists \
that length. We also initialize a sum list ",
 StyleBox["S", "Input"],
 " to the list of all 0s of that same length."
}], "Text"],

Cell[TextData[{
 "To add 0s to the input list, we use the ",
 StyleBox[ButtonBox["PadLeft",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PadLeft"], "FunctionLink"],
 " function, which takes a list and a length and returns the list of the \
desired length obtained by adding 0s on the left side of the list. As you \
might suspect, there is also a ",
 StyleBox[ButtonBox["PadRight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PadRight"], "FunctionLink"],
 " function. To initialize ",
 StyleBox["S", "Input"],
 ", we use ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 ", which, given an expression and a length creates the list of the desired \
length all of whose elements are the given expression. We illustrate these \
functions below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PadLeft", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", "7"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "2", ",", "3"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648234626061*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConstantArray", "[", 
  RowBox[{"\"\<x\>\"", ",", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"x\"\>", ",", "\<\"x\"\>", ",", "\<\"x\"\>", ",", "\<\"x\"\>", 
   ",", "\<\"x\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.580648234678136*^9}]
}, Open  ]],

Cell[TextData[{
 "Once these initial tasks are completed, we follow Algorithm 2. Note that \
the indices used must be modified to match that used by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". The loop variable ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", as presented in the textbook ranges from ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " is the index of the least significant (the \[OpenCurlyDoubleQuote]one\
\[CloseCurlyQuote]s\[CloseCurlyDoubleQuote]) digit and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " is the index of the most significant digit. In this implementation, the \
least significant digit in the input values, ",
 StyleBox["A", "Input"],
 " and ",
 StyleBox["B", "Input"],
 ", will be in the last position, which has index equal to their length, ",
 StyleBox["n", "Input"],
 ". And the most significant digit will be in position 1. Consequently, our \
",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop will have variable ",
 StyleBox["j", "Input"],
 " ranging from ",
 StyleBox["n", "Input"],
 " to ",
 StyleBox["1", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The last difference between our implementation and Algorithm 2 is the use \
of the ",
 StyleBox[ButtonBox["PrependTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrependTo"], "FunctionLink"],
 " function to add a 1 at the beginning of the sum, in case a carry requires \
the result have an additional digit."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"addition", "[", 
   RowBox[{"a_List", ",", "b_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "n", ",", "A", ",", "B", ",", "S", ",", "c", ",", "j", ",", "d"}], "}"}],
     ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", "=", 
      RowBox[{"Max", "[", 
       RowBox[{
        RowBox[{"Length", "[", "a", "]"}], ",", 
        RowBox[{"Length", "[", "b", "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"A", "=", 
      RowBox[{"PadLeft", "[", 
       RowBox[{"a", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"B", "=", 
      RowBox[{"PadLeft", "[", 
       RowBox[{"b", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"S", "=", 
      RowBox[{"ConstantArray", "[", 
       RowBox[{"0", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"c", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"j", "=", "n"}], ",", 
       RowBox[{"j", "\[GreaterEqual]", "1"}], ",", 
       RowBox[{"j", "--"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"d", "=", 
         RowBox[{"Floor", "[", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"A", "[", 
              RowBox[{"[", "j", "]"}], "]"}], "+", 
             RowBox[{"B", "[", 
              RowBox[{"[", "j", "]"}], "]"}], "+", "c"}], ")"}], "/", "2"}], 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"S", "[", 
          RowBox[{"[", "j", "]"}], "]"}], "=", 
         RowBox[{
          RowBox[{"A", "[", 
           RowBox[{"[", "j", "]"}], "]"}], "+", 
          RowBox[{"B", "[", 
           RowBox[{"[", "j", "]"}], "]"}], "+", "c", "-", 
          RowBox[{"2", "*", "d"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"c", "=", "d"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"c", "\[Equal]", "1"}], ",", 
       RowBox[{"PrependTo", "[", 
        RowBox[{"S", ",", "1"}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
     "S"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"addition", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "0", ",", "1", ",", "0"}], "}"}]}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0", ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648234757051*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Binary Multiplication", "Subsection",
 CellTags->"Section2c"],

Cell["\<\
Finally, we will implement a multiplication algorithm, presented as Algorithm \
3 in Section 4.2. Once again, our function will accept the binary \
representations of positive integers as the inputs. This time, however, it is \
not necessary for them to have the same length. \
\>", "Text"],

Cell[TextData[{
 "The shift that occurs when ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["b", "j"], "=", "1"}], TraditionalForm]]],
 " will be accomplished as the following example illustrates. To shift the \
list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}], TraditionalForm]]],
 " by 5 places, we must add five 0s on the end of the list. We do this by \
using ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " to create the list of 5 0s and then ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " to combine the original list with this list of zeros."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"shiftExample", "=", 
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.58064811740582*^9, 3.580648234793254*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Join", "[", 
  RowBox[{"shiftExample", ",", 
   RowBox[{"ConstantArray", "[", 
    RowBox[{"0", ",", "5"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "1", ",", "1", ",", "1", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.580648234843955*^9}]
}, Open  ]],

Cell[TextData[{
 "We will store the partial products using an indexed variable. Recall from \
Section 2.3 of this manual that we can store an object in an indexed \
variable, ",
 StyleBox["c", "Input"],
 ", by making an assignment to the symbol ",
 StyleBox["c[i]", "Input"],
 " for index ",
 StyleBox["i", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The product ",
 StyleBox["p", "Input"],
 " will be initialized to ",
 StyleBox["{0}", "Input"],
 ", a binary representation of 0. The addition in the final loop will be \
performed by the ",
 StyleBox["addition", "Input"],
 " function we created above."
}], "Text"],

Cell[TextData[{
 "As noted above, there is a discrepancy between the way indices are used in \
the pseudocode in the text and the indices used in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". For this function, we will mirror the textbook with the loop variable ",
 StyleBox["j", "Input"],
 " ranging from ",
 StyleBox["0", "Input"],
 " to ",
 StyleBox["n-1", "Input"],
 ", where ",
 StyleBox["n", "Input"],
 " is the number of digits in the second number. We interpret ",
 StyleBox["j", "Input"],
 " as being the number of digits beyond the least significant digit, which \
has position ",
 StyleBox["n", "Input"],
 ". Within the body of the loop, we inspect location ",
 StyleBox["n-j", "Input"],
 "."
}], "Text"],

Cell["Here is our implementation of Algorithm 3.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"multiplication", "[", 
   RowBox[{"a_List", ",", "b_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "j", ",", "c", ",", "p"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"n", "=", 
      RowBox[{"Length", "[", "b", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"j", "=", "0"}], ",", 
       RowBox[{"j", "\[LessEqual]", 
        RowBox[{"n", "-", "1"}]}], ",", 
       RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"b", "[", 
           RowBox[{"[", 
            RowBox[{"n", "-", "j"}], "]"}], "]"}], "\[Equal]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"c", "[", "j", "]"}], "=", 
          RowBox[{"Join", "[", 
           RowBox[{"a", ",", 
            RowBox[{"ConstantArray", "[", 
             RowBox[{"0", ",", "j"}], "]"}]}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"c", "[", "j", "]"}], "=", 
          RowBox[{"{", "0", "}"}]}]}], "\[IndentingNewLine]", "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"p", "=", 
      RowBox[{"{", "0", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"j", "=", "0"}], ",", 
       RowBox[{"j", "\[LessEqual]", 
        RowBox[{"n", "-", "1"}]}], ",", 
       RowBox[{"j", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"p", "=", 
        RowBox[{"addition", "[", 
         RowBox[{"p", ",", 
          RowBox[{"c", "[", "j", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "p"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["We test our function using Example 10 from Section 4.2.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"multiplication", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.580648234940754*^9}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.3 Primes and Greatest Common Divisors", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 "In this section we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find primes, find prime factorizations, and compute greatest common \
divisors and least common multiples. We will also use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s capabilities to explore the distribution of primes. "
}], "Text"],

Cell[CellGroupData[{

Cell["Primes", "Subsection",
 CellTags->"Section3a"],

Cell[TextData[{
 "We will first introduce some of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s functions for testing whether a number is prime and for finding primes. \
"
}], "Text"],

Cell[CellGroupData[{

Cell["Testing for Primality", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " function accepts a single argument, an integer to be tested, and returns \
true or false. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrimeQ", "[", "5", "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5806482349781227`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrimeQ", "[", "10", "]"}]], "Input"],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.580648235029675*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrimeQ", "[", 
  RowBox[{
   RowBox[{"2", "^", "13"}], "-", "1"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648235078143*^9}]
}, Open  ]],

Cell[TextData[{
 "Unlike the trial division algorithm discussed in the book, which checks all \
possible divisors to see if a number is prime or composite, ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " uses a probabilistic primality test. This probabilistic test gains much \
faster performance at the cost of a small possibility that the command will \
return an incorrect result. There is no known example of an integer for which \
",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " is incorrect and any such example must be exceptionally large. So, despite \
there being a chance of error, ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " is in fact very reliable."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Listing Primes", "Subsubsection"],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["Prime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Prime"], "FunctionLink"],
 " accepts as input a positive integer ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and outputs the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th prime number."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prime", "[", "1", "]"}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.58064823512744*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prime", "[", "2", "]"}]], "Input"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.580648235181294*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Prime", "[", "i", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "20"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "3", ",", "5", ",", "7", ",", "11", ",", "13", ",", "17", ",", 
   "19", ",", "23", ",", "29", ",", "31", ",", "37", ",", "41", ",", "43", 
   ",", "47", ",", "53", ",", "59", ",", "61", ",", "67", ",", "71"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648235225873*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Prime", "[", "100000", "]"}]], "Input"],

Cell[BoxData["1299709"], "Output",
 CellChangeTimes->{3.580648235265177*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides the ",
 StyleBox[ButtonBox["NextPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NextPrime"], "FunctionLink"],
 " function, which returns the smallest prime larger than the input value. \
For example, to find the first prime number larger than 1000, enter the \
following. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NextPrime", "[", "1000", "]"}]], "Input"],

Cell[BoxData["1009"], "Output",
 CellChangeTimes->{3.5806482353112783`*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["NextPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/NextPrime"], "FunctionLink"],
 " function can also be given an optional second argument, which must be an \
integer. ",
 Cell[BoxData[
  RowBox[{"NextPrime", "[", 
   StyleBox["n", "CodeVar"], ",", 
   StyleBox["k", "CodeVar"], "]"}]], "myCode"],
 " returns the ",
 StyleBox["k", "CodeVar"],
 "th prime larger then ",
 StyleBox["n", "CodeVar"],
 ". If the second argument is negative, the function instead returns the \
prime smaller than ",
 StyleBox["n", "CodeVar"],
 ". For example, the following expressions find the third prime after 1000 \
and the prime before it."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NextPrime", "[", 
  RowBox[{"1000", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["1019"], "Output",
 CellChangeTimes->{3.580648235364977*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NextPrime", "[", 
  RowBox[{"1000", ",", 
   RowBox[{"-", "1"}]}], "]"}]], "Input"],

Cell[BoxData["997"], "Output",
 CellChangeTimes->{3.5806482354153547`*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Prime Factorization", "Subsection",
 CellTags->"Section3b"],

Cell[TextData[{
 "To compute the prime factorization of an integer, we can use the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 ". Several examples of using ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " follow. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", "100", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648235461158*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", "123456789", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3607", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3803", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648235592285*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", 
  RowBox[{"-", "987654321"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"17", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"379721", ",", "1"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648235650466*^9}]
}, Open  ]],

Cell[TextData[{
 "The output of ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " is a list of pairs. Each pair consists of a prime number and the \
multiplicity, or exponent, of that prime in the prime factorization. Note \
that in the last example, with negative input, one of the members of the list \
is the pair ",
 StyleBox["{-1,1}", "Input"],
 ", indicating that ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "1"], TraditionalForm]]],
 " is in the factorization. The output above indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"100", "=", 
    RowBox[{
     SuperscriptBox["2", "2"], "\[CenterDot]", 
     SuperscriptBox["5", "2"]}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"123456789", "=", 
    RowBox[{
     SuperscriptBox["3", "2"], "\[CenterDot]", 
     SuperscriptBox["3607", "1"], "\[CenterDot]", 
     SuperscriptBox["3803", "1"]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", "987654321"}], "=", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "1"], "\[CenterDot]", 
     SuperscriptBox["3", "2"], "\[CenterDot]", 
     SuperscriptBox["17", "2"], "\[CenterDot]", 
     SuperscriptBox["379721", "1"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Note that ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " can also accept an optional argument to limit the effort ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will exert in trying to factor. Entering ",
 StyleBox["Automatic", "Input"],
 " as the second argument limits the function to factors that it can find \
easily. You can also give a positive integer as the second argument, and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will then find at most that many distinct factors. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", "236914830635411777378758175934586404476822", 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"197", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"509", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"10459723", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"32129861", ",", "2"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648235698394*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", 
  RowBox[{"236914830635411777378758175934586404476822", ",", "3"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"394", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"509", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1181349070215370924270532326421800507", ",", "1"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648235745701*^9}]
}, Open  ]],

Cell[TextData[{
 "The first expression above factors the given integer into its complete \
prime factorization. The second is limited to finding only 3 distinct \
factors. After finding the first two prime factors, ",
 Cell[BoxData[
  FormBox["394", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["509", TraditionalForm]]],
 ", it reports the remainder as the last factor with exponent 1. This gives \
you a way to have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " only perform the easy parts of factorizations, which can help ensure that \
your functions run quickly during development. Then, when you are ready to \
let the function take all the time it needs, you can remove the limitation. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["The Distribution of Primes", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "The Prime Number Theorem (Theorem 4 in Section 4.3 of the text) tells us \
that the number of primes not exceeding ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is approximated by the function ",
 Cell[BoxData[
  FormBox[
   FractionBox["x", 
    RowBox[{"ln", "(", "x", ")"}]], TraditionalForm]]],
 ". In this subsection, we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s graphing capabilities to graph the number of primes not exceeding ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "Recall from Section 3.3 of this manual that we can graph a list of points \
by using the ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " function applied to a list of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " pairs. Our ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " values will be the integers from 2 to 1000 (we omit 1 since there are no \
primes less than or equal to 1)."
}], "Text"],

Cell[TextData[{
 "To find the number of primes not exceeding ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ", we use the function ",
 StyleBox[ButtonBox["PrimePi",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimePi"], "FunctionLink"],
 ". The function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Pi]", "(", "x", ")"}], TraditionalForm]]],
 " is the standard notation for the number of primes less than or equal to ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 ". The symbol ",
 StyleBox[ButtonBox["PrimePi",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimePi"], "FunctionLink"],
 " distinguishes this function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " from the mathematical constant which is given the symbol ",
 StyleBox[ButtonBox["Pi",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Pi"], "FunctionLink"],
 ". To calculate the number of primes less than or equal to 1000, for \
example, we enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrimePi", "[", "1000", "]"}]], "Input"],

Cell[BoxData["168"], "Output",
 CellChangeTimes->{3.580648235796813*^9}]
}, Open  ]],

Cell[TextData[{
 "We use the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function to produce the list of pairs ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"x", ",", 
     RowBox[{"\[Pi]", "(", "x", ")"}]}], "}"}], TraditionalForm]]],
 ", which we will graph."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"piList", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"PrimePi", "[", "x", "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "1000"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[TextData[{
 "As we did in the solution to Computer Project 9 of Chapter 3, we will \
graphically compare the values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Pi]", "(", "x", ")"}], TraditionalForm]]],
 " to the function ",
 Cell[BoxData[
  FormBox[
   FractionBox["x", 
    RowBox[{"ln", "(", "x", ")"}]], TraditionalForm]]],
 ". We will define two graphics objects and then combine them with the ",
 StyleBox[ButtonBox["Show",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Show"], "FunctionLink"],
 " function. Refer to Chapter 3 of this manual, particularly Section 3.3 and \
the solution to Computer Project 9, for detailed information about the \
commands ",
 StyleBox[ButtonBox["ListPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ListPlot"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 " that we use here."
}], "Text"],

Cell[TextData[{
 "We also use the legending functions ",
 StyleBox[ButtonBox["Legended",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Legended"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["LineLegend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LineLegend"], "FunctionLink"],
 ". For a single application of ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 ", you can use the ",
 StyleBox[ButtonBox["PlotLegends",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlotLegends"], "FunctionLink"],
 " option to very easily add a legend to the plot. That option is not \
available to ",
 StyleBox[ButtonBox["Show",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Show"], "FunctionLink"],
 ", however. Instead, you must use the more general function ",
 StyleBox[ButtonBox["Legended",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Legended"], "FunctionLink"],
 " and manually construct the legend with ",
 StyleBox[ButtonBox["LineLegend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LineLegend"], "FunctionLink"],
 " (or one of the related functions such as ",
 StyleBox[ButtonBox["PointLegend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PointLegend"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["SwatchLegend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LineLegend"], "FunctionLink"],
 "). In this context, ",
 StyleBox[ButtonBox["Legended",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Legended"], "FunctionLink"],
 " can be thought of as a wrapper containing a graphics object, or more \
precisely an expression that produces a graphics object, as the first \
argument, and a call to a function that creates a legend as the second \
argument. The ",
 StyleBox[ButtonBox["LineLegend",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LineLegend"], "FunctionLink"],
 " function generally takes two arguments: the first a list of colors and the \
second a list of labels for the color in the corresponding position."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"piPlot", "=", 
   RowBox[{"ListPlot", "[", 
    RowBox[{"piList", ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Blue"}]}], "]"}]}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"xlnxPlot", "=", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"x", "/", 
      RowBox[{"Log", "[", "x", "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "2", ",", "1000"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Red"}]}], "]"}]}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Legended", "[", 
  RowBox[{
   RowBox[{"Show", "[", 
    RowBox[{"piPlot", ",", "xlnxPlot"}], "]"}], ",", 
   RowBox[{"LineLegend", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Blue", ",", "Red"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<\[Pi](x)\>\"", ",", 
       "\"\<\!\(\*FractionBox[\(x\), \(ln \((x)\)\)]\)\>\""}], "}"}]}], 
    "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 TemplateBox[{GraphicsBox[{{{}, {
       RGBColor[0, 0, 1], 
       PointBox[CompressedData["
1:eJxd2znww0YVx/EdaFyqoHCRYskwjIdhGIXTnFlCEgwEEOEyt8iFIZCI2yQB
ltvcKl2qdKnS5ZYuVbrc0qVKlwzD+/2Z/brRfKzLkixp9723b33u1WdffJNz
7vZm5/47/d9nftyVn1B+vwjl9xX8llAut4QfwXoefhR+G/x27GcFvwP7fSf8
LriGH8PvfDf8Hvzu98Lvg98Pf8Cmj9p0DX8Q/hD8Yfgj8EdtqvP2Mfhxm+o8
htIRdh/HfNg9gfmyrssnSkfYPYn5sq7jU6WjrOv6dOkIu09ivqz/waZ0hN2n
MB92n8Z8Wf+rz5SOsHsG82H3WcyX9T/9XOko63/7+dIRdk3pAEc4yboPvlA6
wBFOsu6bZ0sH2e6jCCfYfbF0gCOcZN2XXyod4AgnWffxl0sHOMIJdl/B+rI9
FyKcYPfV0gGOcILdFuvL9hyKcILd10oH2Z5bEU6ynmNfLx3gCCdZz71vlA6y
PQcjnGD3zdJBtudmhBPsvlU6wBFOsPs21ocjnGD3HawPRzjJeg+0pT0c4Fa2
90SEBzjBGXbfLe3Neg8FuNXy9h6K8AAnOMPuudIeDnALR3gw6z2Z4Kzln7Dp
86U9HOAWjvBg1ns4wRl2L5T2cIBbs97rER7gBGdtT+2AF0t7OMAtHOHBrHZF
gjPsXirt4QC3ZrVTIjxoeWunJDjD7nulPRzgFo7wACc4mx/aUbvS3qx2VYBb
OMKDtmdOcNbyaod9v7SHA9zCER7gBGfY/QD7hwPcmtWOjPAAJzjD7uXSHg5w
C0d4gBOczQ/t3h+W9nCAW7PayREetLy1kxOcYfej0t6sdnaAWzjCA5zgrP2p
nf5KaW9WOz7ALRzhAU5wht2rpT0c4NasfkWEBzjBGXZd6cqsfoqHazjADdzC
nfZn/ZwI9/AAj3CCJ+3PnOFZy6sf9ePSFezhGg5wY1Y/rYU7OMK9tmf9uAEe
tbw5wROc4Rl2PyldwR6u4QA3ZvU7W7iDI9zDAzzCCZ7gDM+w+2npCvZwbVY/
OsAN3MKdtmf96Aj3ZvXDB3iEEzxpf+YMz7D7WekK9nANB7iBW7iDI9zDAzya
FYdI8ARneIbdz0tXZsU1PFzDAW7gFu7gCPfwAI9mxVkSPJkVd8nwDLtflK7M
itN4uIYD3MAt3JkV94lwDw/wCCd4gjM8mx/iTL8sXcEeruEAN2bFrVq4gyPc
wwM8mhUHS/AEZ3g2P8TN9qUr2MM1HODGrDhcC3dwhHt4gEc4wZNZcb8Mz7D7
VenKrDihh2s4wA3cwh0c4d6suOQAj3CCJzjDM+xeK13BHq7NipMGuDErbtrC
HRzhHh7gEU7wBGd4Nj/EcV8vXZkV1/VwDQe4gVu4MytOHOEeHuDRrLhygic4
wzPs3ihdmRWn9nANB7iBW7iDI9ybFRcf4BFO8GRWHD3Ds45XcfRfl65gD9dm
xeED3MAt3MER7uEBHuEET3CGZ/ND3uA3pSvYwzUc4AZu4c6sPEWEe3iAR7Py
Ggme4AzPsPtt6Qr2cG1WHiXADdzCnVl5lwj38ACPcIIns/I4GZ5hF0sv4Ape
wh5ewTW8hgO8MSvv1MBbs/JQLbyDO3gPR/gA9/ARHuATPMJnOMEXeIKvcIZv
ZuXlZvgOu9+VXsAVvDQrz+fhFVzDazjAG7iBt3AL78zKO3bwHo7wAe7ho1l5
zAE+wSN8hhN80fm2POcEX3W85gzf4Bm+w+73pRdm5V0reAl7eAXX8BoO8AZu
4K1ZeeAW3sEdvIcjfDArr9zDR3iAT/AIn83KUyf4YlbeeoKvcIZv8AzfzQ95
8D+UXsAVvIQ9vDIrr17DazjAG7Py8A28NSsv38I7uIP3cIQPcA8f4QE+wSN8
1vkyJ/gCT/AVzvANnuE77P5YemFWXUMFL82qc/DwCq7htVl1EQHewA28hVt4
p+Mxd/AejvAB7uGjzofVYQzwScdrHuEznOALPMFXOMM3eIbvsPtT6YVZdScV
vIQ9vDKrTqWG13CAN3ADb82qe2nhHdzBezjCB7iHjzpf5gE+wSN8hhN8gSf4
Cmf4puthnuE77P5cegFX8BL28MqsOqMaXsMB3sANvIVbeAd38N6suqcIH+Ae
PsIDfIJH+GxWnVWCL/AEX+EM38yq25rhO+z+UnoBV/DSrDowD6/gGl6bVTcW
4A3cwFu4hXdwB+/NqlOL8AHu4SM8wCez6t5G+Awn+GJWndwEX+EM3+AZvsPu
UHphVl1eBS9hD6/MquOr4TUc4A3cwFu4hXdwB+/hCB/gHj6aVZc4wCd4hM9w
gi/wBF/hDN/MqpOc4Tvs/lp6AVfwEvbwCq7hNRzgjVl1nQ28NavOs4V3cAfv
4Qgf4B4+wgN8MqvudITPZtWhJvgCT/DVrLrVDN/MqmOd4Tvs/lZ6AVfwEvbw
Cq7htVl1tQHewA28hVt4B3fwHo7wAe7hIzzAJ7PqhEf4DCf4YlZd8QRfzaoz
zvANnuG7OZnd30sv4Apewh5ewTW8hgO8gRt4C7fwzpzNHbyHI3wwqy67h49m
1WkP8Ake4bNZdd0JvsATfIUzfINn+A67f5RewBW8hD28gmt4DQd4Azfw1qw6
+BbewR28N6tuPsIHuIeP8ACf4BE+m1Wnn+ALPMFXOMM3eIbvsPtn6YVZ4wYq
eAl7eAXX8BoO8MascQoNvIVbeGfWuIYO3sMRPsA9fDRrnMQAn+ARPsMJvpg1
7mKCr3CGb/AM32H3r9ILuIKXsIdXcA2vzRpHEuAN3MBbs8adtPAO7uA9HOGD
WeNYevgID/AJHuGzWeNiEnyBJ/gKZ/gGz/DdrHE47t+lF3AFL2EPr8wa11PD
azj83/8BnnkG9A==
        "]]}, {}}, {{}, {}, {
       RGBColor[1, 0, 0], 
       LineBox[CompressedData["
1:eJwVkGk4FAgAhl0zhoZkZhzVOqNkBrGRcnyi5MgxRpKrGVcqnZSropgizcRo
tClHK1utaNgok1zFZqaclQ62NoZl8Wyi2nbZ9sf3vH/eH+/zGXL2M6MV5OTk
tn/b//Ssu35LVU4OjlIfAWjKULVJGpMdUoB4g20um0pEZ7W7cduIEp4Iy1iC
NgJy6NrhJdu/eUGl+QlSArxuyC6mSlQgrrgYepREBNm0ri/IiYzayaumI5FE
SK9mqX8vUsfB8UrFn/uIaOffXEHK14BGD0mg568MUx/NjUckS0DIeWtIGFAG
l5wSMaxEQcD4cHl+NAmyzndpTCcqnIdOcAc/kVDhLqq3EGnBy6VrZ6WJKkxd
/BijwmWY/teV2ZxPBneh3jOwezlONX4NDLdUg6xRf1ebih4IBhr3KFI1BFw5
2uPrZICtC83DjoTFoCx1J9NFRrguKbir36ABAUWWMSxcCSfzdK1DehR0/mNF
fy9cC7N0TY/5ZctR0GxOkcZtgnuam9NyqSkoNddzFVi+sGDFDNM9LKDtlrnf
jstCpaEcbaHEGuLnxezfJcFQKSUUu7rYQtllyj+1PhynGHt7OrLtkWhWkOVG
5KCYdoFYLu+IcxNxA9vLo5B60+6BtjmQrLtjdRQtFpazs16ZIhesKWp6ayaO
w8pVBq7fMV3ReMZZKLPZC8HaHYOJPW7Yb/m5xap0H46tJBeXum3GHJVy41Lp
ATw5sC+3qtcd+lvqD1x8dRDJ1GvHjTZ6gGEi3D1jcRhl7MG0DKknusLz9GnH
EnC6f8FfydsbIU91WTW/JOLX35bqVFVthXT0TUMl8SisvEkhfnRfrLJjlBlt
TEJKTeWjqc1+2G11995sYTLudxUkGBL84cyNOMcfTQF12jHjwgN//Luk4G25
QRp0b+5JztzNRJm7Yox2wjE0pBv69hoHwLJ2YTdBdByRn59Zv+wKgH+qjnnD
6AmU/OH7tfjot1/n7qjHXE7HRF8aauiBuOZZ0VFpn4G0R67kF/2ByJup1TUa
zkDF3Ezk2pxt6Oj0fZxy9iTiB4Nc4zcE4YNDzbq8lafwJrrfQf5tEDSUrF2e
d59Ccq55sDhvO4xjklRe7svECK/2y6xtME7MeVs2qWRB0HwzMmAgGE9fO031
ibLgQDRobePugGbop70VPlzc9s5g/rkqBB5rnmT+9JmLeNuiMdpACBLL4kJj
Ck8j1GXImZoaignngjySzRkU1Y1Uhy4NA8vmTcTL12fwJYs/JGgPQ03L+GRs
cjY++PWLpDHhULymRTNZkQOSne3ge0oErtR1//BIkoNxVaNq78YI0CP9h4x3
nQWdFD+2n7ETVVHT4ttLcnHIbtN0ycmdMJFnBX+tzgVr/uO26Z6dkEk2s8y8
zmEDo5Oaps/GpYltY1kz5/BuEZEae5gN1wqGQ+l5HmyDTlJOtrBBstcUhwl4
SG5IiRlqZUMqmbNfJuTBrqn6zvqHbLD+arITFvFA3NazZaadjSgHpk1uBQ97
mPMuUVI2MnuPrE66z8M6cmj6phdstM836fiN8ZAURrQiTLKRk1deqDbBQzA9
YhF7ig2fFdlakkke4sXy7+9Ps/Hcg0ndPMND9l2T0wkf2BgtGF7sMM9D3fGP
V99/+tZnrqK8isqHdZKyWYsCB9LGyawRLT76q+iPlytxcN6vV+lHXT4GaFej
kwgc6BwpUtDT58NL/YTAksSBWQtjnrKaD6dn5LYrahxMBWge66HzIRM/2PRF
nYMa2dxXniUfuxQfPmRpcLCe3PxFZS0fd7w66xdROJgvLk/qsOPjlrbUPJbK
Qeua7E+Z6/l48crociuNA+7D+CMbHfko/PspSU+bA88g5uyCMx+Ob7oPJutw
8B9JOGaY
        "]]}}}, AspectRatio -> 
    NCache[GoldenRatio^(-1), 0.6180339887498948], Axes -> True, 
    AxesLabel -> {None, None}, AxesOrigin -> {0, 0}, Method -> {}, 
    PlotRange -> {{0, 1000.}, {0, 168.}}, PlotRangeClipping -> True, 
    PlotRangePadding -> {{20., 20.}, {3.36, 3.36}}],
   TemplateBox[{
    "\"\[Pi](x)\"", 
     "\"\\!\\(\\*FractionBox[\\(x\\), \\(ln \\((x)\\)\\)]\\)\""}, 
    "LineLegend", DisplayFunction -> (StyleBox[
      StyleBox[
       PaneBox[
        TagBox[
         GridBox[{{
            TagBox[
             GridBox[{{
                GraphicsBox[{{
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0, 0, 1]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[0, 0, 1]], {}}}, AspectRatio -> Full, 
                 ImageSize -> {20, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #}, {
                GraphicsBox[{{
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[1, 0, 0]], {
                    LineBox[{{0, 10}, {20, 10}}]}}, {
                   Directive[
                    EdgeForm[{
                    Opacity[0.3], 
                    GrayLevel[0]}], 
                    RGBColor[1, 0, 0]], {}}}, AspectRatio -> Full, 
                 ImageSize -> {20, 10}, PlotRangePadding -> None, 
                 ImagePadding -> 1, 
                 BaselinePosition -> (Scaled[0.1] -> Baseline)], #2}}, 
              GridBoxAlignment -> {
               "Columns" -> {Center, Left}, "Rows" -> {{Baseline}}}, 
              AutoDelete -> False, 
              GridBoxDividers -> {
               "Columns" -> {{False}}, "Rows" -> {{False}}}, 
              GridBoxItemSize -> {"Columns" -> {{All}}, "Rows" -> {{All}}}, 
              GridBoxSpacings -> {"Columns" -> {{0.5}}, "Rows" -> {{0.8}}}], 
             "Grid"]}}, 
          GridBoxAlignment -> {"Columns" -> {{Left}}, "Rows" -> {{Top}}}, 
          AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          GridBoxSpacings -> {"Columns" -> {{1}}, "Rows" -> {{0}}}], "Grid"], 
        Alignment -> Left, AppearanceElements -> None, 
        ImageMargins -> {{5, 5}, {5, 5}}, ImageSizeAction -> "ResizeToFit"], 
       LineIndent -> 0, StripOnInput -> False], {FontFamily -> "Times"}, 
      Background -> Automatic, StripOnInput -> False]& ), Editable -> True, 
    InterpretationFunction :> (RowBox[{"LineLegend", "[", 
       RowBox[{
         RowBox[{"{", 
           RowBox[{
             RowBox[{"RGBColor", "[", 
               RowBox[{"0", ",", "0", ",", "1"}], "]"}], ",", 
             RowBox[{"RGBColor", "[", 
               RowBox[{"1", ",", "0", ",", "0"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
           RowBox[{#, ",", #2}], "}"}]}], "]"}]& )]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.580648235974576*^9}]
}, Open  ]],

Cell[TextData[{
 "Notice that while the blue line representing ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Pi]", "(", "x", ")"}], TraditionalForm]]],
 " seems to remain above the red line representing ",
 Cell[BoxData[
  FormBox[
   FractionBox["x", 
    RowBox[{"ln", "(", "x", ")"}]], TraditionalForm]]],
 ", it is fairly clear from the graph that they grow at the same rate."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Greatest Common Divisor and Least Common Multiple", "Subsection",
 CellTags->"Section3d"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the functions ",
 StyleBox[ButtonBox["GCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GCD"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["LCM",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LCM"], "FunctionLink"],
 " for computing the greatest common divisor and the least common multiple of \
integers. To compute the greatest common divisor of two integers, you apply \
the ",
 StyleBox[ButtonBox["GCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GCD"], "FunctionLink"],
 " function to them."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GCD", "[", 
  RowBox[{"6", ",", "9"}], "]"}]], "Input"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.5806482360296707`*^9}]
}, Open  ]],

Cell["\<\
You can also compute the greatest common divisor of more than two integers. \
For more than 2 integers, the greatest common divisor is defined to be the \
largest integer that is a divisor of all of them. For example, 3 divides 6, \
9, and 12, so:\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GCD", "[", 
  RowBox[{"6", ",", "9", ",", "12"}], "]"}]], "Input"],

Cell[BoxData["3"], "Output",
 CellChangeTimes->{3.5806482360996532`*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["LCM",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/LCM"], "FunctionLink"],
 " command finds the least common multiple of two or more integers. For \
example, "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LCM", "[", 
  RowBox[{"6", ",", "9"}], "]"}]], "Input"],

Cell[BoxData["18"], "Output",
 CellChangeTimes->{3.580648236149501*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LCM", "[", 
  RowBox[{"12", ",", "18", ",", "33"}], "]"}]], "Input"],

Cell[BoxData["396"], "Output",
 CellChangeTimes->{3.580648236182721*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Relatively Prime", "Subsubsection"],

Cell["\<\
Recall from the text that two numbers are said to be relatively prime if \
their greatest common divisor is 1. For example, consider 10 and 21. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GCD", "[", 
  RowBox[{"10", ",", "21"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.580648236229411*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 StyleBox[ButtonBox["GCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GCD"], "FunctionLink"],
 " returned 1, we conclude that 10 and 21 are relatively prime. "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["CoprimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CoprimeQ"], "FunctionLink"],
 ", applied to two integers, returns true if they are relatively prime and \
false if not."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoprimeQ", "[", 
  RowBox[{"3", ",", "6"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.580648236278335*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoprimeQ", "[", 
  RowBox[{"22", ",", "15"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648236346287*^9}]
}, Open  ]],

Cell[TextData[{
 "Also recall that a list of integers ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "n"]}], TraditionalForm]]],
 " are said to be pairwise relatively prime if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"gcd", "(", 
     RowBox[{
      SubscriptBox["a", "i"], ",", 
      SubscriptBox["a", "j"]}], ")"}], "=", "1"}], TraditionalForm]]],
 " whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "\[LessEqual]", "i", "<", "j", "\[LessEqual]", "n"}], 
   TraditionalForm]]],
 ". That is, when every pair is relatively prime. The ",
 StyleBox[ButtonBox["CoprimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CoprimeQ"], "FunctionLink"],
 " function, applied to more than two integers, tests this property."
}], "Text"],

Cell["Observe that 14, 39, and 55 are pairwise relatively prime.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoprimeQ", "[", 
  RowBox[{"14", ",", "39", ",", "55"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648236396287*^9}]
}, Open  ]],

Cell["\<\
However, 42, 165, and 182 are not pairwise relatively prime, since 42 and 182 \
are both even, even though the common GCD of all three integers is 1.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GCD", "[", 
  RowBox[{"42", ",", "165", ",", "182"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.580648236446587*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CoprimeQ", "[", 
  RowBox[{"42", ",", "165", ",", "182"}], "]"}]], "Input"],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.580648236487752*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Extended Euclidean Algorithm", "Subsubsection"],

Cell[TextData[{
 "While ",
 StyleBox[ButtonBox["GCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GCD"], "FunctionLink"],
 " is useful for calculating the greatest common divisor of integers, it is \
sometimes desirable to be able to express the greatest common divisor as an \
integral combination of the integers. Specifically, given integers ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ", we may wish to express ",
 Cell[BoxData[
  FormBox[
   RowBox[{"gcd", "(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"s", "\[CenterDot]", "a"}], "+", 
    RowBox[{"t", "\[CenterDot]", "b"}]}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["t", TraditionalForm]]],
 " are integers. The fact that such integers always exist is known as B\
\[EAcute]zout's Theorem, given in the text as Theorem 6 of Section 4.3. In \
the preamble to Exercise 41 in the text, the extended Euclidean algorithm is \
described, which produces not only the greatest common divisor but also the \
integers ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["t", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", the function ",
 StyleBox[ButtonBox["ExtendedGCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ExtendedGCD"], "FunctionLink"],
 " is an implementation of the extended Euclidean algorithm. This function \
accepts two or more integers as arguments. It returns a list whose first \
element is the greatest common divisor of the arguments and whose second \
element is a sublist whose members are the coefficients required to obtain \
the GCD. As an example, consider 252 and 198, the values used in Example 17 \
of the textbook."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExtendedGCD", "[", 
  RowBox[{"252", ",", "198"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"18", ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", 
     RowBox[{"-", "5"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648236627523*^9}]
}, Open  ]],

Cell[TextData[{
 "The results above indicate that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"gcd", "(", 
     RowBox[{"252", ",", "198"}], ")"}], "=", 
    RowBox[{"18", "=", 
     RowBox[{
      RowBox[{"4", "\[CenterDot]", "252"}], "-", 
      RowBox[{"5", "\[CenterDot]", "198"}]}]}]}], TraditionalForm]]],
 ". Note that the order of the coefficients is the same as the order of the \
arguments to the function. "
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.4 Solving Congruences", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section, we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to solve congruences. We will begin the section by looking at \
how to find inverses and solve linear congruences. We will then consider the \
Chinese Remainder Theorem. Next, we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find pseudoprimes, and we conclude with an exploration of primitive \
roots and discrete logarithms. "
}], "Text"],

Cell[CellGroupData[{

Cell["Modular Inverses", "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "Example 1 of Section 4.4 of the text demonstrates how B\[EAcute]zout \
coefficients can be used to find the inverse of an integer modulo a number. \
In the previous section of this manual, we saw that the ",
 StyleBox[ButtonBox["ExtendedGCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ExtendedGCD"], "FunctionLink"],
 " function can be used to obtain the B\[EAcute]zout coefficients. "
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Finding Inverses with ",
 StyleBox["ExtendedGCD", "Input"]
}], "Subsubsection"],

Cell[TextData[{
 "For example, to find the inverse of ",
 Cell[BoxData[
  FormBox["264", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["3185", TraditionalForm]]],
 ", we need to find ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " so that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"s", "\[CenterDot]", "264"}], "+", 
     RowBox[{"t", "\[CenterDot]", "3185"}]}], "=", "1"}], TraditionalForm]]],
 ", provided that ",
 Cell[BoxData[
  FormBox["264", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["3185", TraditionalForm]]],
 " are relatively prime. "
}], "Text"],

Cell[TextData[{
 "Recall that ",
 StyleBox[ButtonBox["ExtendedGCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ExtendedGCD"], "FunctionLink"],
 " applied to two integers returns a structure of the form ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["gcd", "CodeVar"], ",", 
    RowBox[{"{", 
     RowBox[{
      StyleBox["s", "CodeVar"], ",", 
      RowBox[{
       StyleBox["t", "CodeVar"], 
       RowBox[{"}", "}"}]}]}]}]}]}]], "myCode"],
 " where ",
 StyleBox["gcd", "CodeVar"],
 " is the greatest common divisor of the two integers, and ",
 StyleBox["s", "CodeVar"],
 " and ",
 StyleBox["t", "CodeVar"],
 " are the B\[EAcute]zout coefficients. Knowing that this is always the form \
of the output, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " allows us to assign the result of ",
 StyleBox[ButtonBox["ExtendedGCD",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ExtendedGCD"], "FunctionLink"],
 " to a structure of that form. This has the effect of assigning the symbols \
to the corresponding numbers in the output."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"gBezout", ",", 
    RowBox[{"{", 
     RowBox[{"sBezout", ",", "tBezout"}], "}"}]}], "}"}], "=", 
  RowBox[{"ExtendedGCD", "[", 
   RowBox[{"264", ",", "3185"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   RowBox[{"{", 
    RowBox[{"374", ",", 
     RowBox[{"-", "31"}]}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648117447412*^9, 3.580648236679001*^9}]
}, Open  ]],

Cell[TextData[{
 "Since the first element is 1, we know that ",
 Cell[BoxData[
  FormBox["264", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["3185", TraditionalForm]]],
 " are relatively prime. Also, the assignment has caused the B\[EAcute]zout \
coefficients to be stored in the symbols."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["sBezout"], "Input"],

Cell[BoxData["374"], "Output",
 CellChangeTimes->{3.580648236751088*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["tBezout"], "Input"],

Cell[BoxData[
 RowBox[{"-", "31"}]], "Output",
 CellChangeTimes->{3.580648236797214*^9}]
}, Open  ]],

Cell[TextData[{
 "This indicates that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "=", 
    RowBox[{
     RowBox[{"374", "\[CenterDot]", "264"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "31"}], ")"}], "\[CenterDot]", "3185"}]}]}], 
   TraditionalForm]]],
 ". And thus ",
 Cell[BoxData[
  FormBox["374", TraditionalForm]]],
 " is the inverse of ",
 Cell[BoxData[
  FormBox["264", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["3185", TraditionalForm]]],
 ". We can confirm this by computing the product modulo ",
 Cell[BoxData[
  FormBox["3185", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{
   RowBox[{"374", "*", "264"}], ",", "3185"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.580648236864813*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Finding Inverses with ",
 StyleBox["PowerMod", "Input"]
}], "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a simpler way to compute the modular inverse. The textbook uses \
the notation ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["a", "_"], TraditionalForm]]],
 " to indicate the modular inverse of an integer. An alternate notation is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", 
    RowBox[{"-", "1"}]], TraditionalForm]]],
 ", which calls to mind the notation used in algebra for reciprocals, as in \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", 
     RowBox[{"-", "1"}]], "=", 
    FractionBox["1", "3"]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " function computes powers of integers in modular arithmetic. It takes three \
arguments: the base integer, the exponent, and the modulus. For example, the \
following computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["3", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "5"}], ")"}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"3", ",", "4", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.580648236917966*^9}]
}, Open  ]],

Cell[TextData[{
 "For positive exponents, ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " is more efficient than applying ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " with the exponent computed in the argument. Moreover, ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " accepts negative (and even rational) exponents. In particular, applying ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " with second argument -1 computes the modular inverse."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"264", ",", 
   RowBox[{"-", "1"}], ",", "3185"}], "]"}]], "Input"],

Cell[BoxData["374"], "Output",
 CellChangeTimes->{3.5806482369560957`*^9}]
}, Open  ]],

Cell["\<\
Note that if the integer and the modulus are not relatively prime, no inverse \
exists and an error is generated.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"4", ",", 
   RowBox[{"-", "1"}], ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"PowerMod", "::", "ninv"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(4\\)\[NoBreak] is not invertible modulo \
\[NoBreak]\\!\\(10\\)\[NoBreak]. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/PowerMod/ninv\\\", ButtonNote -> \
\\\"PowerMod::ninv\\\"]\\)\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5806482370263863`*^9}],

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"4", ",", 
   RowBox[{"-", "1"}], ",", "10"}], "]"}]], "Output",
 CellChangeTimes->{3.580648237035522*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Solving Congruences", "Subsubsection"],

Cell[TextData[{
 "We saw in Section 4.1 of this manual that the ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " function with ",
 StyleBox[ButtonBox["Modulus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Modulus"], "FunctionLink"],
 " option can be used for solving congruences. We can use this command to \
solve linear congruences such as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "x"}], "\[Congruent]", 
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "11"}], ")"}]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"4", "*", "x"}], "\[Equal]", "3"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "11"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", "9"}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.580648237085019*^9}]
}, Open  ]],

Cell[TextData[{
 "The first argument to ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " is the congruence expressed with an ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") symbol. Following the equation is a rule setting the ",
 StyleBox[ButtonBox["Modulus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Modulus"], "FunctionLink"],
 " option to the modulus. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " returns a list whose elements express the solutions to the congruence. If \
there is no solution, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " returns an empty list."
}], "Text"],

Cell[TextData[{
 "The following attempts to solve ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "x"}], "\[Congruent]", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "10"}], ")"}]}]}], TraditionalForm]]],
 ", which is the same as finding an inverse for 4 modulo 10 and has no \
solution. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"4", "*", "x"}], "\[Equal]", "1"}], ",", 
   RowBox[{"Modulus", "->", "10"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output",
 CellChangeTimes->{3.580648237150257*^9}]
}, Open  ]],

Cell[TextData[{
 "It is also possible to have multiple solutions. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"3", "x"}], "\[Congruent]", 
    RowBox[{"9", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "12"}], ")"}]}]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", "*", "x"}], "\[Equal]", "9"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "12"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"3", "+", 
     RowBox[{"4", " ", 
      RowBox[{"C", "[", "1", "]"}]}]}]}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.580648237217963*^9}]
}, Open  ]],

Cell[TextData[{
 "The symbol ",
 StyleBox["C[1]", "Input"],
 " is used by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to stand for an arbitrary integer. This output indicates that any value of \
",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", 
    RowBox[{"4", "\[CenterDot]", "C"}]}], TraditionalForm]]],
 " will solve the congruence. You can obtain a specific solution by \
substituting a particular integer for the symbol ",
 StyleBox["C[1]", "Input"],
 ", using ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ")."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Solve", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"3", "*", "x"}], "\[Equal]", "9"}], ",", 
    RowBox[{"Modulus", "\[Rule]", "12"}]}], "]"}], "/.", 
  RowBox[{
   RowBox[{"C", "[", "1", "]"}], "\[Rule]", "1"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", "7"}], "}"}], "}"}]], "Output",
 CellChangeTimes->{3.580648237264443*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["The Chinese Remainder Theorem", "Subsection",
 CellTags->"Section4b"],

Cell["\<\
The text describes two approaches to solving systems of congruences of the \
form\
\>", "Text"],

Cell[BoxData[{
 FormBox[
  RowBox[{"x", "\[Congruent]", 
   SubscriptBox["a", "1"], " ", "(", "mod", " ", 
   SubscriptBox["m", "1"], ")"}], TraditionalForm], "\n", 
 FormBox[
  RowBox[{"x", "\[Congruent]", 
   RowBox[{
    SubscriptBox["a", "2"], " ", "(", 
    RowBox[{"mod", " ", 
     SubscriptBox["m", "2"]}], ")"}]}], TraditionalForm], "\n", 
 FormBox[
  RowBox[{"             ", "\[VerticalEllipsis]"}], TraditionalForm], "\n", 
 FormBox[
  RowBox[{"x", "\[Congruent]", 
   RowBox[{
    SubscriptBox["a", "n"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", 
      SubscriptBox["m", "n"]}], ")"}]}]}], TraditionalForm]}], "Equation"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides an implementation of the Chinese remainder theorem as the \
function ",
 StyleBox[ButtonBox["ChineseRemainder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ChineseRemainder"], "FunctionLink"],
 ". The function takes two arguments: a list of the values ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["a", "1"], ",", 
     SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["a", "n"]}], "}"}], TraditionalForm]]],
 " first and a list of the moduli ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["m", "1"], ",", 
     SubscriptBox["m", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["m", "n"]}], "}"}], TraditionalForm]]],
 " second. The result is the smallest positive integer that satisfies all of \
the congruences. ",
 "As an example, we solve the congruences"
}], "Text"],

Cell[BoxData[{
 FormBox[
  RowBox[{"x", "\[Congruent]", "2", " ", "(", "mod", " ", "3", ")"}], 
  TraditionalForm], "\n", 
 FormBox[
  RowBox[{"x", "\[Congruent]", "4", " ", "(", "mod", " ", "5", ")"}], 
  TraditionalForm], "\n", 
 FormBox[
  RowBox[{"x", "\[Congruent]", "6", " ", "(", "mod", " ", "7", ")"}], 
  TraditionalForm], "\n", 
 FormBox[
  RowBox[{"x", "\[Congruent]", "10", " ", "(", "mod", " ", "11", ")"}], 
  TraditionalForm]}], "Equation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChineseRemainder", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "6", ",", "10"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "7", ",", "11"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["1154"], "Output",
 CellChangeTimes->{3.5806482373127604`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating a Function", "Subsubsection"],

Cell[TextData[{
 "We will create a function for solving systems of congruences. This \
implementation will be based on the construction given in the proof of the \
Chinese remainder theorem. While this will be less efficient than ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s built-in ",
 StyleBox[ButtonBox["ChineseRemainder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ChineseRemainder"], "FunctionLink"],
 " function, implementing the algorithm can help you to better understand the \
proof of the theorem. "
}], "Text"],

Cell[TextData[{
 "Our function, which we call ",
 StyleBox["crTheorem", "Input"],
 ", will accept the same arguments as ",
 StyleBox[ButtonBox["ChineseRemainder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ChineseRemainder"], "FunctionLink"],
 ": two lists, ",
 StyleBox["a", "Input"],
 " and ",
 StyleBox["m", "Input"],
 ", representing the values and the moduli of the congruences. It will begin \
with two tests to check that the lists are the same length and that the \
moduli are in fact pairwise relatively prime, as is required by the \
assumptions of the theorem. We use ",
 StyleBox[ButtonBox["CoprimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CoprimeQ"], "FunctionLink"],
 " from Section 4.3 of this manual to check that the moduli are pairwise \
relatively prime. If these tests fail, the following messages are generated."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"crTheorem", "::", "argsize"}], "=", 
   "\"\<Arguments must be lists of the same size.\>\""}], ";"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"crTheorem", "::", "argcp"}], "=", 
   "\"\<Moduli must be pairwise relatively prime.\>\""}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We will embed the tests inside a small function in order to make the ",
 StyleBox["crTheorem", "Input"],
 " function easier to read."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"crTestArgs", "[", 
   RowBox[{"a_", ",", "m_"}], "]"}], ":=", 
  RowBox[{"Check", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Length", "[", "a", "]"}], "\[NotEqual]", 
        RowBox[{"Length", "[", "m", "]"}]}], ",", 
       RowBox[{"Message", "[", 
        RowBox[{"crTheorem", "::", "argsize"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"Not", "[", 
        RowBox[{"Apply", "[", 
         RowBox[{"CoprimeQ", ",", "m"}], "]"}], "]"}], ",", 
       RowBox[{"Message", "[", 
        RowBox[{"crTheorem", "::", "argcp"}], "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "True"}], ",", "\[IndentingNewLine]", "False"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Most of the work of ",
 StyleBox["crTestArgs", "Input"],
 " is done in the two ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statements. The first compares the lengths of ",
 StyleBox["a", "Input"],
 " and ",
 StyleBox["m", "Input"],
 " and the second checks whether the moduli are relatively prime. If the \
lists are of different lengths or the moduli are not relatively prime, the \
appropriate message is issued. Note that the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " function is used in the second ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement in order to apply the function ",
 StyleBox[ButtonBox["CoprimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CoprimeQ"], "FunctionLink"],
 ", which expects integer arguments, to the list ",
 StyleBox["m", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " function is used to \[OpenCurlyDoubleQuote]listen\[CloseCurlyDoubleQuote] \
for messages. It evaluates its first argument, which in this case is the \
three lines ending with the expression ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". If no messages are generated, then the result of the ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " is the outcome of that evaluation. In this case, if the two ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statements do not raise messages, then the outcome of the ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " is ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". However, if any messages are raised while evaluating the first argument \
to ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 ", then ",
 StyleBox[ButtonBox["Check",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Check"], "FunctionLink"],
 " returns its second argument instead. In this case, this means that if any \
messages are raised, then the result of the function will be ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "As you can see below, with valid input, ",
 StyleBox["crTestArgs", "Input"],
 " results in ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ", but it also is able to produce both error messages and return ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " for arguments that violate the rules."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"crTestArgs", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "7", ",", "11"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.580648237494919*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"crTestArgs", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "6"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"crTheorem", "::", "argsize"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Arguments must be lists of the same size.\"\>"}]], \
"Message", "MSG",
 CellChangeTimes->{3.58064823761158*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"crTheorem", "::", "argcp"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Moduli must be pairwise relatively prime.\"\>"}]], \
"Message", "MSG",
 CellChangeTimes->{3.580648237720035*^9}],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.580648237725225*^9}]
}, Open  ]],

Cell[TextData[{
 "When we define the ",
 StyleBox["crTheorem", "Input"],
 " function, we will use patterns in the arguments to ensure that the \
arguments are lists of integers, and we will give ",
 StyleBox["crTestArgs", "Input"],
 " as a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 "), as shown below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"crTheorem", "[", 
    RowBox[{
     RowBox[{"a", ":", 
      RowBox[{"{", "__Integer", "}"}]}], ",", 
     RowBox[{"m", ":", 
      RowBox[{"{", "__Integer", "}"}]}]}], "]"}], "/;", 
   RowBox[{"crTestArgs", "[", 
    RowBox[{"a", ",", "m"}], "]"}]}], ":=", "..."}]], "myCode"],

Cell[TextData[{
 "The syntax ",
 StyleBox["a:{__Integer}", "Input"],
 ", and likewise for ",
 StyleBox["m", "Input"],
 ", names the argument and imposes the pattern that it be a sequence of \
integers enclosed in braces, that is, a list of integers. On the right hand \
side of the ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") operator, we apply ",
 StyleBox["crTestArgs", "Input"],
 " to the arguments before the ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Input"],
 ") operator. With a function definition of this form, when you enter a call \
to ",
 StyleBox["crTheorem", "Input"],
 ", ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will first check to see that the arguments match the specified patterns. \
If not, for instance if you provide a different number of arguments or \
attempt to use anything other than two lists of integers, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will simply return the expression unevaluated. Assuming that you enter two \
lists of integers, then ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will apply ",
 StyleBox["crTestArgs", "Input"],
 ". If this function returns ",
 StyleBox["False", "Input"],
 ", then ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will return the ",
 StyleBox["crTheorem", "Input"],
 " call unevaluated. Moreover, in this case, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will not attempt to evaluate the body of the function. Only after the \
arguments have matched the pattern and ",
 StyleBox["crTestArgs", "Input"],
 " has returned ",
 StyleBox["True", "Input"],
 ", will ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " evaluate the body of the function definition."
}], "Text"],

Cell[TextData[{
 "While the above is a bit more complicated seeming than placing the tests in \
the body of the function and causing them to terminate execution with a ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " or an ",
 StyleBox[ButtonBox["Abort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Abort"], "FunctionLink"],
 ", it is a much more elegant way to ensure that the function is robust."
}], "Text"],

Cell[TextData[{
 "Turning now to the main work of ",
 StyleBox["crTheorem", "Input"],
 ", it begins by setting ",
 StyleBox["p", "Input"],
 " equal to the product of the moduli. (Note that ",
 StyleBox["p", "Input"],
 " corresponds to ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " in the statement of the theorem in the text. This is the only notational \
difference between our function and the text.) Note that we use the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") operator with the ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " (",
 StyleBox["*", "Input"],
 ") function in order to compute this product. ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " (",
 StyleBox["*", "Input"],
 ") is the functional version of the multiplication operator, and combining \
it with ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") results in the product of the elements of the list ",
 StyleBox["m", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "The function then needs to compute ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "k"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "k"], TraditionalForm]]],
 ". We use the indexed variables ",
 StyleBox["M", "Input"],
 " and ",
 StyleBox["y", "Input"],
 " for this. Note that this creates a subtle syntactic point to pay attention \
to. Specifically, the third element of the list ",
 StyleBox["m", "Input"],
 " is accessed via ",
 StyleBox["m[[3]]", "Input"],
 ", while ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "3"], TraditionalForm]]],
 " is referred to by ",
 StyleBox["M[3]", "Input"],
 ". The values are computed within a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop. The values for ",
 StyleBox["M", "Input"],
 " are calculated by the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["M", "k"], "=", 
    FractionBox["P", 
     SubscriptBox["m", "k"]]}], TraditionalForm]]],
 ". For ",
 StyleBox["y", "Input"],
 ", we use the fact that the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "k"], TraditionalForm]]],
 " are the inverses of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["M", "k"], TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "k"], TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["y", "k"], "\[Congruent]", 
    RowBox[{
     SuperscriptBox[
      SubscriptBox["M", "k"], 
      RowBox[{"-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", 
       SubscriptBox["m", "k"]}], ")"}]}]}], TraditionalForm]]],
 ". Finally, we compute the result ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "1"], 
      SubscriptBox["M", "1"], 
      SubscriptBox["y", "1"]}], "+", 
     RowBox[{
      SubscriptBox["a", "2"], 
      SubscriptBox["M", "2"], 
      SubscriptBox["y", "2"]}], "+", "\[CenterEllipsis]", "+", 
     RowBox[{
      SubscriptBox["a", "n"], 
      SubscriptBox["M", "n"], 
      SubscriptBox["y", "n"]}]}]}], TraditionalForm]]],
 " using the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function and return ",
 Cell[BoxData[
  FormBox[
   RowBox[{"x", " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "p"}], ")"}]}], TraditionalForm]]],
 ". Here is the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"crTheorem", "[", 
    RowBox[{
     RowBox[{"a", ":", 
      RowBox[{"{", "__Integer", "}"}]}], ",", 
     RowBox[{"m", ":", 
      RowBox[{"{", "__Integer", "}"}]}]}], "]"}], "/;", 
   RowBox[{"crTestArgs", "[", 
    RowBox[{"a", ",", "m"}], "]"}]}], ":=", "\[IndentingNewLine]", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "M", ",", "y", ",", "i", ",", "x"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"p", "=", 
      RowBox[{"Times", "@@", "m"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "a", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", "i", "]"}], "=", 
         RowBox[{"p", "/", 
          RowBox[{"m", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"y", "[", "i", "]"}], "=", 
         RowBox[{"PowerMod", "[", 
          RowBox[{
           RowBox[{"M", "[", "i", "]"}], ",", 
           RowBox[{"-", "1"}], ",", 
           RowBox[{"m", "[", 
            RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"x", "=", 
      RowBox[{"Sum", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "*", 
         RowBox[{"M", "[", "i", "]"}], "*", 
         RowBox[{"y", "[", "i", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", 
          RowBox[{"Length", "[", "a", "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Mod", "[", 
      RowBox[{"x", ",", "p"}], "]"}]}]}], " ", "\[IndentingNewLine]", 
   "]"}]}]], "Input"],

Cell[TextData[{
 "Note that our function produces the same result as ",
 StyleBox[ButtonBox["ChineseRemainder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ChineseRemainder"], "FunctionLink"],
 " did above."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"crTheorem", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "6", ",", "10"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "7", ",", "11"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["1154"], "Output",
 CellChangeTimes->{3.58064823792874*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pseudoprimes", "Subsection",
 CellTags->"Section4c"],

Cell[TextData[{
 "Recall from the text that a pseudoprime to the base ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " is a composite number ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", 
     RowBox[{"n", "-", "1"}]], "\[Congruent]", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ". We will write a function to find pseudoprimes. Our function will accept \
two arguments, the base ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " and a maximum value for ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and will return a list of the pseudoprimes that it identifies. "
}], "Text"],

Cell[TextData[{
 "The algorithm is fairly straightforward. We will use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop beginning at 3, ending with the specified maximum and increasing by 2 \
each time (so as to skip even integers). Within the loop, we test whether the \
congruence holds and whether the number is composite, using ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 ". Note that if the congruence fails, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will not bother testing primality. If it is composite, then ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " is invoked. The ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 " surrounding the loop collects the pseudoprimes into a list. As we have \
done in the past, we use ",
 StyleBox["[[2,1]]", "Input"],
 " to access the list of pseudoprimes without the additional information \
produced by ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findPseudoprimes", "[", 
   RowBox[{"b_Integer", ",", "max_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "n", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"n", "=", "3"}], ",", 
        RowBox[{"n", "\[LessEqual]", "max"}], ",", 
        RowBox[{"n", "=", 
         RowBox[{"n", "+", "2"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"PowerMod", "[", 
             RowBox[{"b", ",", 
              RowBox[{"n", "-", "1"}], ",", "n"}], "]"}], "\[Equal]", "1"}], "&&", 
           RowBox[{"Not", "[", 
            RowBox[{"PrimeQ", "[", "n", "]"}], "]"}]}], ",", 
          RowBox[{"Sow", "[", "n", "]"}]}], "]"}]}], "\[IndentingNewLine]", 
       "]"}], "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that we used the ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " function rather than the ",
 StyleBox[ButtonBox["Power",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Power"], "FunctionLink"],
 " (",
 StyleBox["^", "Input"],
 ") operator. The ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " function performs modular exponentiation intelligently, using techniques \
such as those discussed in Section 4.2 of the text for performing efficient \
modular exponentiation."
}], "Text"],

Cell[TextData[{
 "Here are the pseudoprimes to the base 2 up to ",
 Cell[BoxData[
  FormBox["100000", TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findPseudoprimes", "[", 
  RowBox[{"2", ",", "100000"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "341", ",", "561", ",", "645", ",", "1105", ",", "1387", ",", "1729", ",", 
   "1905", ",", "2047", ",", "2465", ",", "2701", ",", "2821", ",", "3277", 
   ",", "4033", ",", "4369", ",", "4371", ",", "4681", ",", "5461", ",", 
   "6601", ",", "7957", ",", "8321", ",", "8481", ",", "8911", ",", "10261", 
   ",", "10585", ",", "11305", ",", "12801", ",", "13741", ",", "13747", ",", 
   "13981", ",", "14491", ",", "15709", ",", "15841", ",", "16705", ",", 
   "18705", ",", "18721", ",", "19951", ",", "23001", ",", "23377", ",", 
   "25761", ",", "29341", ",", "30121", ",", "30889", ",", "31417", ",", 
   "31609", ",", "31621", ",", "33153", ",", "34945", ",", "35333", ",", 
   "39865", ",", "41041", ",", "41665", ",", "42799", ",", "46657", ",", 
   "49141", ",", "49981", ",", "52633", ",", "55245", ",", "57421", ",", 
   "60701", ",", "60787", ",", "62745", ",", "63973", ",", "65077", ",", 
   "65281", ",", "68101", ",", "72885", ",", "74665", ",", "75361", ",", 
   "80581", ",", "83333", ",", "83665", ",", "85489", ",", "87249", ",", 
   "88357", ",", "88561", ",", "90751", ",", "91001", ",", "93961"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648238088437*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Primitive Roots and Discrete Logarithms", "Subsection",
 CellTags->"Section4d"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes several functions for computing primitive roots and discrete \
logarithms."
}], "Text"],

Cell[CellGroupData[{

Cell["Primitive Roots", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a function, ",
 StyleBox[ButtonBox["PrimitiveRoot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimitiveRoot"], "FunctionLink"],
 ", that computes primitive roots. It takes a single argument, the modulus, \
and returns the smallest positive primitive root for that modulus. For \
example, the smallest positive primitive root of 13 is 2."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PrimitiveRoot", "[", "13", "]"}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.580648238118719*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s ",
 StyleBox[ButtonBox["PrimitiveRoot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimitiveRoot"], "FunctionLink"],
 " function applies to some non-prime moduli as well. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " uses a definition of primitive root that is more general than the \
definition in the text. Specifically, an integer ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is a primitive root modulo an integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " if every positive integer that is both less than ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and relatively prime to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " can be obtained as a power of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "To obtain a list of all of the primitive roots of a prime, not just the \
first, we will make use of the ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " function. The multiplicative order of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is defined to be the smallest positive integer ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "m"], "\[Congruent]", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 ". Equivalently, one can say that the multiplicative order of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is the number of distinct powers of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", that is, the number of different values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "k"], " ", "mod", " ", "p"}], TraditionalForm]]],
 ". We leave it to the reader to prove the equivalence."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " function accepts the element and modulus as arguments and returns the \
multiplicative order. For example, to compute the multiplicative order of 8 \
modulo 13, you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplicativeOrder", "[", 
  RowBox[{"8", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["4"], "Output",
 CellChangeTimes->{3.5806482381673727`*^9}]
}, Open  ]],

Cell[TextData[{
 "We conclude from this that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "1"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "13"}], ")"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "13"}], ")"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "3"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "13"}], ")"}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "13"}], ")"}]}], TraditionalForm]]],
 " are all distinct, with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "4"], "\[Congruent]", 
    RowBox[{"1", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "13"}], ")"}]}]}], TraditionalForm]]],
 ", but that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["8", "5"], "\[Congruent]", 
    RowBox[{
     SuperscriptBox["8", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "13"}], ")"}]}]}], TraditionalForm]]],
 ". We can verify this by computing the values."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"PowerMod", "[", 
    RowBox[{"8", ",", "k", ",", "13"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "4"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"8", ",", "12", ",", "5", ",", "1"}], "}"}]], "Output",
 CellChangeTimes->{3.580648238204658*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"8", ",", "5", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["8"], "Output",
 CellChangeTimes->{3.580648238254376*^9}]
}, Open  ]],

Cell[TextData[{
 "Recall from Definition 3 of the textbook that a primitive root modulo a \
prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is an integer ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " such that every nonzero element of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    StyleBox["Z",
     FontWeight->"Bold"], "p"], TraditionalForm]]],
 " is a power of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". Since there are ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " elements of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    StyleBox["Z",
     FontWeight->"Bold"], "p"], TraditionalForm]]],
 ", there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "1"}], TraditionalForm]]],
 " nonzero elements. Consequently, being a primitive root modulo a prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is identical to having multiplicative order ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "1"}], TraditionalForm]]],
 ". The fact that, as we calculated above, the multiplicative order of ",
 Cell[BoxData[
  FormBox["8", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["13", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 ", not ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"13", "-", "1"}], "=", "12"}], TraditionalForm]]],
 " implies that ",
 Cell[BoxData[
  FormBox["8", TraditionalForm]]],
 " is not a primitive root modulo 13. However, 6 has multiplicative order 12 \
modulo 13."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplicativeOrder", "[", 
  RowBox[{"6", ",", "13"}], "]"}]], "Input"],

Cell[BoxData["12"], "Output",
 CellChangeTimes->{3.58064823831666*^9}]
}, Open  ]],

Cell["Consequently, 6 is a primitive root modulo 13.", "Text"],

Cell[TextData[{
 "To summarize, for ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " to be a primitive root modulo a prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", it is necessary and sufficient that the multiplicative order of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " be equal to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "1"}], TraditionalForm]]],
 ". This observation provides us with a convenient way to list all of the \
primitive roots for a given prime. We just consider each possible ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " from 2 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "1"}], TraditionalForm]]],
 " and calculate their multiplicative order with ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 ". Those whose order is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "-", "1"}], TraditionalForm]]],
 " are included in the list. Here is the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allPrimitiveRoots", "[", 
   RowBox[{"p_", "?", "PrimeQ"}], "]"}], ":=", 
  RowBox[{"Select", "[", 
   RowBox[{
    RowBox[{"Range", "[", 
     RowBox[{"2", ",", 
      RowBox[{"p", "-", "1"}]}], "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"MultiplicativeOrder", "[", 
       RowBox[{"#", ",", "p"}], "]"}], "\[Equal]", 
      RowBox[{"p", "-", "1"}]}], "&"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We make two comments about this function. First, we ensure that the \
argument is prime by using the ",
 ButtonBox["PatternTest",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PatternTest"],
 " syntax: the pattern, in this case ",
 StyleBox[ButtonBox["Blank",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Blank"], "FunctionLink"],
 " (",
 StyleBox["_", "Input"],
 ") is followed by a question mark and the name of a function, ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 ", that performs a test."
}], "Text"],

Cell[TextData[{
 "Second, the ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " function is used to obtain, given a list and a condition, the sublist of \
those elements meeting the condition. ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " requires two arguments. First, the initial list. Second, a function in one \
argument that returns ",
 StyleBox["True", "Input"],
 " for the desired elements. For the second argument, you can either provide \
the name of a function that accepts a single argument, or, as in the above, \
the function can be given as a pure ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 "). Recall that a pure function is terminated with an ampersand (",
 StyleBox["&", "Input"],
 ") and uses a ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ") for its argument."
}], "Text"],

Cell[TextData[{
 "Applying the ",
 StyleBox["allPrimitiveRoots", "Input"],
 " function to 13 produces the list of all primitive roots modulo 13."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"allPrimitiveRoots", "[", "13", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "6", ",", "7", ",", "11"}], "}"}]], "Output",
 CellChangeTimes->{3.58064823848053*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Discrete Logarithms", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " function can also be used to find discrete logarithms. Recall that the \
discrete logarithm of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " modulo a prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " to the base ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " is a number ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "e"], "\[Congruent]", 
    RowBox[{"a", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 ". The multiplicative order of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ", as we defined it above, is the smallest positive ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["r", "m"], "\[Congruent]", 
    RowBox[{"1", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 ". Notice that the congruences defining these two concepts are similar. \
Indeed, the discrete logarithm problem is simply more general than the \
multiplicative order problem, replacing the specific 1 with an arbitrary ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " function accepts a third argument which generalizes it to include the \
computation of discrete logarithms. Recall that the first argument of ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " is the value ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", the second is the prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". By providing a third argument, ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", ",
 StyleBox[ButtonBox["MultiplicativeOrder",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MultiplicativeOrder"], "FunctionLink"],
 " will compute the discrete logarithm of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " to the base ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". That is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["log", "r"], "a"}], TraditionalForm]]],
 " is computed by ",
 Cell[BoxData[
  RowBox[{"MultiplicativeOrder", "[", 
   StyleBox["r", "CodeVar"], ",", 
   StyleBox["p", "CodeVar"], ",", 
   StyleBox["a", "CodeVar"], "]"}]], "myCode"],
 "."
}], "Text"],

Cell["\<\
To compute the discrete logarithm of 3 modulo 11 to the base 2, you enter the \
following.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MultiplicativeOrder", "[", 
  RowBox[{"2", ",", "11", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["8"], "Output",
 CellChangeTimes->{3.580648238516068*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.5 Applications of Congruences", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to further explore the applications of congruences discussed \
in the text. In particular, we will see how to use a hashing function to \
store student information in a list. We will create a pseudorandom number \
generator. And we will write a function that will check the validity of an \
ISBN. "
}], "Text"],

Cell[CellGroupData[{

Cell["Hashing Functions", "Subsection",
 CellTags->"Section5a"],

Cell["\<\
The first application we will explore is the hashing function. Suppose that a \
small school wants to store information about its students. In particular, \
each student has a unique four digit identification number and a GPA, which \
is a real number between 0 and 4. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Initial Examples", "Subsubsection"],

Cell["\<\
Each student record will be stored as a list with first element the student \
ID and the second element the student\[CloseCurlyQuote]s GPA. Here are three \
example students.\
\>", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"student1", "=", 
   RowBox[{"{", 
    RowBox[{"7319", ",", "3.21"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"student2", "=", 
   RowBox[{"{", 
    RowBox[{"2908", ",", "2.89"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"student3", "=", 
   RowBox[{"{", 
    RowBox[{"6578", ",", "3.42"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True],

Cell["\<\
Our student records are going to be stored in a list. Because the school is \
small, it will suffice to allocate space for 57 records in the school's \
database and so we create a list with 57 entries all initialized to 0. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"studentRecords", "=", 
  RowBox[{"Table", "[", 
   RowBox[{"0", ",", 
    RowBox[{"{", "57", "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117526647*^9, 3.580648238615551*^9}]
}, Open  ]],

Cell[TextData[{
 "In order to store a student record in the list (which represents the \
school's database), we need to apply a hashing function to the unique student \
ID. The hashing function we'll use is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "(", "k", ")"}], "=", 
    RowBox[{
     RowBox[{"k", " ", "mod", " ", "57"}], "+", "1"}]}], TraditionalForm]]],
 ". Note that the addition of 1 is to occur after the computation of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "mod", " ", "57"}], TraditionalForm]]],
 ". It is included in our function because the indices in our ",
 StyleBox["studentRecords", "Input"],
 " list run from 1 to 57 while the values of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", " ", "mod", " ", "57"}], TraditionalForm]]],
 " range from 0 to 56. "
}], "Text"],

Cell["\<\
The following function accepts a student ID as input and returns the result \
of applying the hashing function to the ID number. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"calculateHash", "[", "id_Integer", "]"}], ":=", 
  RowBox[{
   RowBox[{"Mod", "[", 
    RowBox[{"id", ",", "57"}], "]"}], "+", "1"}]}]], "Input",
 InitializationCell->True],

Cell["For example,", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"calculateHash", "[", 
  RowBox[{"student1", "[", 
   RowBox[{"[", "1", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.580648238713966*^9}]
}, Open  ]],

Cell[TextData[{
 "This indicates that ",
 StyleBox["student1", "Input"],
 "'s record should be stored in location 24. We store a record in a \
particular location in the usual way."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studentRecords", "[", 
   RowBox[{"[", "24", "]"}], "]"}], "=", "student1"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7319", ",", "3.21`"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117551697*^9, 3.5806482387719727`*^9}]
}, Open  ]],

Cell["\<\
Note that accessing location 24 returns the list containing the student\
\[CloseCurlyQuote]s data.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"studentRecords", "[", 
  RowBox[{"[", "24", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7319", ",", "3.21`"}], "}"}]], "Output",
 CellChangeTimes->{3.5806482389173307`*^9}]
}, Open  ]],

Cell["\<\
To access the ID and GPA of the student stored in location 24, we can use a \
second pair of double brackets with 1 or 2 to access the ID or GPA.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studentRecords", "[", 
   RowBox[{"[", "24", "]"}], "]"}], "[", 
  RowBox[{"[", "1", "]"}], "]"}]], "Input"],

Cell[BoxData["7319"], "Output",
 CellChangeTimes->{3.580648238969925*^9}]
}, Open  ]],

Cell[TextData[{
 "Or we can include the record number and the index of the particular piece \
of data in a single ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operation."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"studentRecords", "[", 
  RowBox[{"[", 
   RowBox[{"24", ",", "1"}], "]"}], "]"}]], "Input"],

Cell[BoxData["7319"], "Output",
 CellChangeTimes->{3.580648239003587*^9}]
}, Open  ]],

Cell[TextData[{
 "We can store ",
 StyleBox["student2", "Input"],
 "\[CloseCurlyQuote]s information in the same way."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"calculateHash", "[", 
  RowBox[{"student2", "[", 
   RowBox[{"[", "1", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.580648239051979*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studentRecords", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "=", "student2"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2908", ",", "2.89`"}], "}"}]], "Output",
 CellChangeTimes->{3.5806481175795803`*^9, 3.58064823908365*^9}]
}, Open  ]],

Cell[TextData[{
 "If we try to store ",
 StyleBox["student3", "Input"],
 "\[CloseCurlyQuote]s data, we find that a collision occurs."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"calculateHash", "[", 
  RowBox[{"student3", "[", 
   RowBox[{"[", "1", "]"}], "]"}], "]"}]], "Input"],

Cell[BoxData["24"], "Output",
 CellChangeTimes->{3.580648239137444*^9}]
}, Open  ]],

Cell[TextData[{
 "Since ",
 StyleBox["student3", "Input"],
 " has the same hash value as ",
 StyleBox["student1", "Input"],
 " did, we look for the next free location. Check location 25."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studentRecords", "[", 
   RowBox[{"[", "25", "]"}], "]"}], "\[Equal]", "0"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.58064823918694*^9}]
}, Open  ]],

Cell[TextData[{
 "Since location 25 is still equal to 0, we know that it has not been used \
and we store ",
 StyleBox["student3", "Input"],
 "\[CloseCurlyQuote]s record in location 25."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"studentRecords", "[", 
   RowBox[{"[", "25", "]"}], "]"}], "=", "student3"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"6578", ",", "3.42`"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117612639*^9, 3.5806482392374*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Printing Records", "Subsubsection"],

Cell[TextData[{
 "Before going any further, take a look at the current state of ",
 StyleBox["studentRecords", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData["studentRecords"], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"2908", ",", "2.89`"}], "}"}], ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", 
   RowBox[{"{", 
    RowBox[{"7319", ",", "3.21`"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6578", ",", "3.42`"}], "}"}], ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", 
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
   ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648239284141*^9}]
}, Open  ]],

Cell[TextData[{
 "This is not very easy to read. We need to write a function to print out the \
data in a more useful format. To do this, we loop through the elements of the \
list and, for those that are non-zero, print the index and the data stored in \
that position. Note that because we are potentially comparing numbers to \
lists, we must use ",
 StyleBox[ButtonBox["UnsameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UnsameQ"], "FunctionLink"],
 " (",
 StyleBox["=!=", "Input"],
 ") rather than ",
 StyleBox[ButtonBox["Unequal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Unequal"], "FunctionLink"],
 " (",
 StyleBox["!=", "Input"],
 ")."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"printRecords", "[", "database_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"For", "[", 
     RowBox[{
      RowBox[{"i", "=", "1"}], ",", 
      RowBox[{"i", "\[LessEqual]", 
       RowBox[{"Length", "[", "database", "]"}]}], ",", 
      RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"database", "[", 
          RowBox[{"[", "i", "]"}], "]"}], "=!=", "0"}], ",", 
        RowBox[{"Print", "[", 
         RowBox[{"i", ",", "\"\< \>\"", ",", 
          RowBox[{"database", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "]"}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printRecords", "[", "studentRecords", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2908", ",", "2.89`"}], "}"}]}],
  SequenceForm[2, " ", {2908, 2.89}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648239365917*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"24", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"7319", ",", "3.21`"}], "}"}]}],
  SequenceForm[24, " ", {7319, 3.21}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648239371318*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"25", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"6578", ",", "3.42`"}], "}"}]}],
  SequenceForm[25, " ", {6578, 3.42}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648239376546*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Function for Storing New Records", "Subsubsection"],

Cell[TextData[{
 "Now we'll write a function ",
 StyleBox["storeRecord", "Input"],
 " to automate the process of adding records to the database. ",
 StyleBox["storeRecord", "Input"],
 " will accept two arguments, the ID and GPA of a student, and will add that \
student's record to the ",
 StyleBox["studentRecords", "Input"],
 " list (the database). "
}], "Text"],

Cell[TextData[{
 "The first step in implementing ",
 StyleBox["storeRecord", "Input"],
 " will be to assign to a local variable, which we'll call ",
 StyleBox["record", "Input"],
 ", the list representing the student record. Then ",
 StyleBox["storeRecord", "Input"],
 " needs to determine the location in the ",
 StyleBox["studentRecords", "Input"],
 " list in which the record will be stored. In particular, it will need to \
avoid collision. To do this, we'll use something similar to the linear \
probing function defined in the text. Beginning with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "=", "0"}], TraditionalForm]]],
 ", we'll calculate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"h", "(", 
     RowBox[{"k", "+", "i"}], ")"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"k", "+", "i"}], ")"}], " ", "mod", " ", "57"}], "+", "1"}]}], 
   TraditionalForm]]],
 ". We will store that value in the local name ",
 StyleBox["hash", "Input"],
 " and check to see if ",
 Cell["studentRecords[[hash]]", "myCode"],
 " is 0. If so, then we know the list does not already have a record stored \
in that location and we can stop our search for an open position. The ",
 StyleBox[ButtonBox["Break",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Break"], "FunctionLink"],
 " function causes the loop in which it is contained to terminate. If the \
location is not empty, we increment ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and continue looking. Once we have found an open position, we only need to \
assign our record to that position. We give ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " as the final expression so that the function does not display anything \
when the record is successfully stored. "
}], "Text"],

Cell[TextData[{
 "Note that for this function we chose not to include the database as a \
parameter, but instead we've described the function in relation to the ",
 StyleBox["studentRecords", "Input"],
 " list that we began above. This can result in a significant improvement in \
performance, especially when the list of records is long, because the \
database does not have to be passed as an argument to the function and then \
returned from it each time a new record is to be stored. The disadvantage, of \
course, is that in order to use a different name for the database, we have to \
revise the ",
 StyleBox["storeRecord", "Input"],
 " function. "
}], "Text"],

Cell[TextData[{
 "Here is the completed ",
 StyleBox["storeRecord", "Input"],
 " function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"storeRecord", "[", 
   RowBox[{"id_Integer", ",", "gpa_Real"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"record", ",", "hash", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"record", "=", 
      RowBox[{"{", 
       RowBox[{"id", ",", "gpa"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "0"}], ",", 
       RowBox[{"i", "\[LessEqual]", "56"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hash", "=", 
         RowBox[{"calculateHash", "[", 
          RowBox[{"id", "+", "i"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"studentRecords", "[", 
            RowBox[{"[", "hash", "]"}], "]"}], "===", "0"}], ",", 
          RowBox[{"Break", "[", "]"}]}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"studentRecords", "[", 
       RowBox[{"[", "hash", "]"}], "]"}], "=", "record"}], ";", 
     "\[IndentingNewLine]", "Null"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["Now we add a few records.", "Text"],

Cell[BoxData[
 RowBox[{"storeRecord", "[", 
  RowBox[{"2216", ",", "1.98"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"storeRecord", "[", 
  RowBox[{"1325", ",", "3.14"}], "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"storeRecord", "[", 
  RowBox[{"7061", ",", "3.51"}], "]"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Look again at ",
 StyleBox["studentRecords", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printRecords", "[", "studentRecords", "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2908", ",", "2.89`"}], "}"}]}],
  SequenceForm[2, " ", {2908, 2.89}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482395511503`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"15", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1325", ",", "3.14`"}], "}"}]}],
  SequenceForm[15, " ", {1325, 3.14}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482395582647`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"24", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"7319", ",", "3.21`"}], "}"}]}],
  SequenceForm[24, " ", {7319, 3.21}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648239563141*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"25", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"6578", ",", "3.42`"}], "}"}]}],
  SequenceForm[25, " ", {6578, 3.42}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648239567931*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"51", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2216", ",", "1.98`"}], "}"}]}],
  SequenceForm[51, " ", {2216, 1.98}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482395730133`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"52", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"7061", ",", "3.51`"}], "}"}]}],
  SequenceForm[52, " ", {7061, 3.51}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482395777397`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Retrieving Records", "Subsubsection"],

Cell["\<\
We now have functions for storing a student record in our database and for \
printing all of the records. But we also need a way to retrieve the record \
for a particular student. Indeed, one of the benefits of hash functions is \
that they provide an efficient way to look up records \[LongDash] given the \
unique key, we need only apply the hash function to determine the memory \
location in which the record is stored (subject to collision, of course). \
\>", "Text"],

Cell[TextData[{
 "Our ",
 StyleBox["retrieveRecord", "Input"],
 " function will accept a student ID number as its input and return the list \
storing the student\[CloseCurlyQuote]s record. Most of the work will take \
place within the same ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop as was in the ",
 StyleBox["storeRecord", "Input"],
 " function. This time, we enclose the loop in a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 ", as the means of short-circuiting the loop and passing the result, either ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " or the record, out of the function. We test to make sure the location \
we're looking in is non-zero. If the location is 0, that tells us that the \
entry does not exist and the procedure will display the following message and \
return ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"retrieveRecord", "::", "missing"}], "=", 
   "\"\<Desired record does not exist.\>\""}], ";"}]], "Input",
 InitializationCell->True],

Cell["\<\
Assuming the location is not 0, we check to see if the ID of the record in \
that position is the ID we're looking for. If so, we return the student data. \
If the ID is not the one we're searching for, it must have been the case that \
our record was pushed down the line because of a collision and we continue \
the loop. \
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"retrieveRecord", "[", "id_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"hash", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "0"}], ",", 
       RowBox[{"i", "\[LessEqual]", "56"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"hash", "=", 
         RowBox[{"calculateHash", "[", 
          RowBox[{"id", "+", "i"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"studentRecords", "[", 
            RowBox[{"[", "hash", "]"}], "]"}], "===", "0"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Message", "[", 
            RowBox[{"retrieveRecord", "::", "missing"}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Throw", "[", "Null", "]"}]}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"studentRecords", "[", 
             RowBox[{"[", "hash", "]"}], "]"}], "[", 
            RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", "id"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"Throw", "[", 
           RowBox[{"studentRecords", "[", 
            RowBox[{"[", "hash", "]"}], "]"}], "]"}]}], "\[IndentingNewLine]",
          "]"}]}]}], "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", 
     "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"retrieveRecord", "[", "1325", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1325", ",", "3.14`"}], "}"}]], "Output",
 CellChangeTimes->{3.580648239715802*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"retrieveRecord", "[", "7061", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7061", ",", "3.51`"}], "}"}]], "Output",
 CellChangeTimes->{3.5806482397579536`*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pseudorandom Numbers", "Subsection",
 CellTags->"Section5b"],

Cell["\<\
Many applications require sequences of random numbers, which are important in \
cryptography and in generating data for computer simulations. It is \
impossible to produce a truly random stream of numbers using software only, \
since software employs algorithms. Anything that can be generated by an \
algorithm is, by definition, not random. Fortunately, for most applications, \
it is sufficient to generate a stream of pseudorandom numbers. This is a \
stream of numbers that, while not truly random, exhibits some of the same \
properties of a random number stream. Effective algorithms for generating \
pseudorandom numbers can be based on modular arithmetic. We will implement a \
linear congruential method, as described in the text. \
\>", "Text"],

Cell[TextData[{
 "We must choose four integers: the modulus ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", the multiplier ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "\[LessEqual]", "a", "<", "m"}], TraditionalForm]]],
 ", the increment ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "\[LessEqual]", "c", "<", "m"}], TraditionalForm]]],
 ", and the seed ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "0"], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "\[LessEqual]", 
    SubscriptBox["x", "0"], "<", "m"}], TraditionalForm]]],
 ". Then we can create a sequence of pseudorandom numbers using the recursive \
formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", 
     RowBox[{"n", "+", "1"}]], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", 
        SubscriptBox["x", "n"]}], "+", "c"}], ")"}], " ", "mod", " ", "m"}]}],
    TraditionalForm]]],
 ". It is common to have the seed chosen based on some physical property \
accessible by the computer, for instance the time. Alternately, the seed can \
be based on some truly random physical process, such as radioactive decay. \
For this example, we will generate a seed by multiplying by 1000 the result \
of the ",
 StyleBox[ButtonBox["SessionTime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SessionTime"], "FunctionLink"],
 " function, which gives the total number of seconds since the beginning of \
the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session. We apply ",
 StyleBox[ButtonBox["Floor",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Floor"], "FunctionLink"],
 " to be certain that we obtain an integer. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Floor", "[", 
  RowBox[{"1000", "*", 
   RowBox[{"SessionTime", "[", "]"}]}], "]"}]], "Input"],

Cell[BoxData["25247"], "Output",
 CellChangeTimes->{3.580648239803074*^9}]
}, Open  ]],

Cell[TextData[{
 "We will write two functions that generate random student IDs and GPAs that \
we can use to add some random records to our ",
 StyleBox["studentRecords", "Input"],
 " from above. We first write the function ",
 StyleBox["randomIDs", "Input"],
 ", which will accept a positive integer as input to control the number of \
IDs to generate. It will return a sequence of that number of random student \
IDs. "
}], "Text"],

Cell[TextData[{
 "Recall that a student ID, in the context described above, is a four-digit \
number. So our random numbers must be between 1000 and 9999. We can obtain \
such numbers by generating random integers between 0 and 8999 and adding \
1000. So our modulus will be 8999. We will choose a multiplier of 57 and an \
increment of 328. (These values were chosen for no particular reason, but in \
practice the choice of ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " can be an important consideration. See the references in the textbook for \
more information.) The seed will be determined from ",
 StyleBox[ButtonBox["SessionTime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SessionTime"], "FunctionLink"],
 " as described above. "
}], "Text"],

Cell["The function is straightforward.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"randomIDs", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m", "=", "8999"}], ",", 
      RowBox[{"a", "=", "57"}], ",", 
      RowBox[{"c", "=", "328"}], ",", "x", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"x", "=", 
      RowBox[{"Mod", "[", 
       RowBox[{
        RowBox[{"Floor", "[", 
         RowBox[{"1000", "*", 
          RowBox[{"SessionTime", "[", "]"}]}], "]"}], ",", "m"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Reap", "[", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "1"}], ",", 
         RowBox[{"i", "\[LessEqual]", "n"}], ",", 
         RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Sow", "[", 
          RowBox[{"x", "=", 
           RowBox[{"Mod", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"a", "*", "x"}], "+", "c"}], ",", "m"}], "]"}]}], 
          "]"}]}], "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", "]"}],
       "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["We generate 10 random IDs by applying the procedure to 10. ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"someIDs", "=", 
  RowBox[{"randomIDs", "[", "10", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "3108", ",", "6503", ",", "2040", ",", "8620", ",", "5722", ",", "2518", 
   ",", "8869", ",", "1917", ",", "1609", ",", "2051"}], "}"}]], "Output",
 CellChangeTimes->{3.5806481177000313`*^9, 3.580648240018166*^9}]
}, Open  ]],

Cell[TextData[{
 "To generate GPAs, the approach will be essentially the same. We use the \
pure multiplicative generator mentioned in the text with modulus ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "31"], "-", "1"}], TraditionalForm]]],
 ", multiplier ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["7", "5"], TraditionalForm]]],
 ", and increment 0. This will produce integers between ",
 Cell[BoxData[
  FormBox["0", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "31"], "-", "2"}], TraditionalForm]]],
 ". To obtain numbers between 0 and 4, we'll divide the random integer by ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "31"], "-", "2"}], TraditionalForm]]],
 " and multiply by ",
 Cell[BoxData[
  FormBox["4", TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"randomGPAs", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"m", "=", 
       RowBox[{
        RowBox[{"2", "^", "31"}], "-", "1"}]}], ",", 
      RowBox[{"a", "=", 
       RowBox[{"7", "^", "5"}]}], ",", "x", ",", "i"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"x", "=", 
      RowBox[{"Mod", "[", 
       RowBox[{
        RowBox[{"Floor", "[", 
         RowBox[{"1000", "*", 
          RowBox[{"SessionTime", "[", "]"}]}], "]"}], ",", "m"}], "]"}]}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Reap", "[", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "1"}], ",", 
         RowBox[{"i", "\[LessEqual]", "n"}], ",", 
         RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"x", "=", 
           RowBox[{"Mod", "[", 
            RowBox[{
             RowBox[{"a", "*", "x"}], ",", "m"}], "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"Sow", "[", 
           RowBox[{"Round", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{"x", "/", 
                RowBox[{"(", 
                 RowBox[{"m", "-", "1"}], ")"}]}], ")"}], "*", "4"}], ",", 
             "0.01"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
       "\[IndentingNewLine]", "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"someGPAs", "=", 
  RowBox[{"randomGPAs", "[", "10", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.8`", ",", "1.37`", ",", "1.62`", ",", "1.86`", ",", "1.37`", ",", 
   "3.99`", ",", "1.8900000000000001`", ",", "2.1`", ",", "2.2`", ",", 
   "0.49`"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117746439*^9, 3.5806482400828953`*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that we use ",
 StyleBox[ButtonBox["Round",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Round"], "FunctionLink"],
 " to round the random number to the nearest hundredth, so that out output \
has at most two digits after the decimal place. The second argument to ",
 StyleBox[ButtonBox["Round",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Round"], "FunctionLink"],
 " being ",
 Cell[BoxData[
  FormBox["0.01", TraditionalForm]]],
 " means that the result will be rounded to the nearest multiple of ",
 Cell[BoxData[
  FormBox["0.01", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Now we add the random students to ",
 StyleBox["studentRecords", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{"For", "[", 
  RowBox[{
   RowBox[{"i", "=", "1"}], ",", 
   RowBox[{"i", "\[LessEqual]", "10"}], ",", 
   RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
   RowBox[{"storeRecord", "[", 
    RowBox[{
     RowBox[{"someIDs", "[", 
      RowBox[{"[", "i", "]"}], "]"}], ",", 
     RowBox[{"someGPAs", "[", 
      RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"printRecords", "[", "studentRecords", "]"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"2", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2908", ",", "2.89`"}], "}"}]}],
  SequenceForm[2, " ", {2908, 2.89}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240152521*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"6", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"6503", ",", "1.37`"}], "}"}]}],
  SequenceForm[6, " ", {6503, 1.37}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240158243*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"11", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2518", ",", "3.99`"}], "}"}]}],
  SequenceForm[11, " ", {2518, 3.99}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482401629467`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"14", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"8620", ",", "1.86`"}], "}"}]}],
  SequenceForm[14, " ", {8620, 1.86}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240167472*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"15", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1325", ",", "3.14`"}], "}"}]}],
  SequenceForm[15, " ", {1325, 3.14}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240172002*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"16", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1609", ",", "2.2`"}], "}"}]}],
  SequenceForm[16, " ", {1609, 2.2}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240176565*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"23", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"5722", ",", "1.37`"}], "}"}]}],
  SequenceForm[23, " ", {5722, 1.37}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240181011*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"24", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"7319", ",", "3.21`"}], "}"}]}],
  SequenceForm[24, " ", {7319, 3.21}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240185564*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"25", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"6578", ",", "3.42`"}], "}"}]}],
  SequenceForm[25, " ", {6578, 3.42}],
  Editable->False]], "Print",
 CellChangeTimes->{3.58064824019*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"31", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"3108", ",", "0.8`"}], "}"}]}],
  SequenceForm[31, " ", {3108, 0.8}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240194462*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"35", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"8869", ",", "1.8900000000000001`"}], "}"}]}],
  SequenceForm[35, " ", {8869, 1.8900000000000001`}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240199028*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"37", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"1917", ",", "2.1`"}], "}"}]}],
  SequenceForm[37, " ", {1917, 2.1}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240203602*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"46", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2040", ",", "1.62`"}], "}"}]}],
  SequenceForm[46, " ", {2040, 1.62}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240208189*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"51", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2216", ",", "1.98`"}], "}"}]}],
  SequenceForm[51, " ", {2216, 1.98}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482402127457`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"52", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"7061", ",", "3.51`"}], "}"}]}],
  SequenceForm[52, " ", {7061, 3.51}],
  Editable->False]], "Print",
 CellChangeTimes->{3.5806482402172537`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"57", "\[InvisibleSpace]", "\<\" \"\>", "\[InvisibleSpace]", 
   RowBox[{"{", 
    RowBox[{"2051", ",", "0.49`"}], "}"}]}],
  SequenceForm[57, " ", {2051, 0.49}],
  Editable->False]], "Print",
 CellChangeTimes->{3.580648240221747*^9}]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Check Digits", "Subsection",
 CellTags->"Section5c"],

Cell["\<\
We conclude this section with a function to check the validity of an ISBN. \
Recall that the ISBN-10 code consists of 10 digits, the last of which is \
computed by the formula \
\>", "Text"],

Cell[BoxData[
 FormBox[Cell[TextData[{
   Cell[BoxData[
    FormBox[
     RowBox[{
      SubscriptBox["x", "10"], "="}], TraditionalForm]]],
   Cell[BoxData[
    FormBox[
     RowBox[{
      UnderoverscriptBox[
       StyleBox["\[Sum]",
        FontSize->16], 
       StyleBox[
        RowBox[{"i", "=", "1"}], "TI"], "9",
       LimitsPositioning->False], 
      RowBox[{
       RowBox[{"i", "\[CenterDot]", 
        SubscriptBox["x", "i"]}], " ", 
       RowBox[{"(", 
        RowBox[{"mod", " ", "11"}], ")"}]}]}], TraditionalForm]], "InlineMath"]
  }]], TraditionalForm]], "Equation"],

Cell[TextData[{
 "The symbol X is used in case ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "10"], "=", "10"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Our ",
 StyleBox["checkISBN", "Input"],
 " function will accept the ISBN as a string. It is necessary that we use \
strings in case the ISBN contains X as the check digit. Consider the ISBN \
below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"isbnExample", "=", "\"\<0073383090\>\""}]], "Input",
 InitializationCell->True],

Cell[BoxData["\<\"0073383090\"\>"], "Output",
 CellChangeTimes->{3.580648117829256*^9, 3.5806482402717657`*^9}]
}, Open  ]],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", in order to access a character within a string, you use the ",
 StyleBox[ButtonBox["StringTake",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringTake"], "FunctionLink"],
 " function. The first argument to ",
 StyleBox[ButtonBox["StringTake",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringTake"], "FunctionLink"],
 " is the string. In order to obtain a single character, you provide as the \
second argument a list with the position of the character as the sole \
element. For example, the third character of our example is obtained as \
follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringTake", "[", 
  RowBox[{"isbnExample", ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"7\"\>"], "Output",
 CellChangeTimes->{3.5806482403385878`*^9}]
}, Open  ]],

Cell["Note that the output is still a string, however.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"StringTake", "[", 
   RowBox[{"isbnExample", ",", 
    RowBox[{"{", "3", "}"}]}], "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox["\"\<7\>\"",
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.5806482403872004`*^9}]
}, Open  ]],

Cell[TextData[{
 "In order to perform arithmetic, we need to turn the character into an \
integer. To do this, we can use the ",
 StyleBox[ButtonBox["ToExpression",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToExpression"], "FunctionLink"],
 " function. When ",
 StyleBox[ButtonBox["ToExpression",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToExpression"], "FunctionLink"],
 " is applied to a string, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " interprets the string as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " input. In this example, the string \[OpenCurlyDoubleQuote]7\
\[CloseCurlyDoubleQuote] will be interpreted as if we had entered 7 on an \
input line. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ToExpression", "[", 
   RowBox[{"StringTake", "[", 
    RowBox[{"isbnExample", ",", 
     RowBox[{"{", "3", "}"}]}], "]"}], "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox["7",
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.580648240420834*^9}]
}, Open  ]],

Cell[TextData[{
 "Conversely, the function ",
 StyleBox[ButtonBox["ToString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToString"], "FunctionLink"],
 " will convert an expression into a string. The following converts the \
number 7 into the string \[OpenCurlyDoubleQuote]7\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ToString", "[", "7", "]"}], "//", "FullForm"}]], "Input"],

Cell[BoxData[
 TagBox[
  StyleBox["\"\<7\>\"",
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellChangeTimes->{3.580648240472189*^9}]
}, Open  ]],

Cell[TextData[{
 "Our function will compute the sum indicated by the formula above using ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 ". Recall that the first argument to ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " is an expression in terms of an index variable and the second argument is \
the range for the variable. Once the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "10"], TraditionalForm]]],
 " is determined, we compare it to the check digit. This is only slightly \
complicated by the fact that a check digit of 10 corresponds to the symbol X. \
Note that we must compare the check digit with the value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "10"], TraditionalForm]]],
 " as strings in both the case that the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "10"], "=", "10"}], TraditionalForm]]],
 " and when ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "10"], "<", "10"}], TraditionalForm]]],
 ". This is because the last digit may be X, whether it should be or not, and \
applying the ",
 StyleBox[ButtonBox["ToExpression",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToExpression"], "FunctionLink"],
 " function to the string \[OpenCurlyDoubleQuote]X\[CloseCurlyDoubleQuote] \
will result in the symbol ",
 StyleBox["X", "Input"],
 ", which may in fact be assigned to an expression."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkISBN", "[", "isbn_String", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "check"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"check", "=", 
      RowBox[{"Mod", "[", 
       RowBox[{
        RowBox[{"Sum", "[", 
         RowBox[{
          RowBox[{"i", "*", 
           RowBox[{"ToExpression", "[", 
            RowBox[{"StringTake", "[", 
             RowBox[{"isbn", ",", 
              RowBox[{"{", "i", "}"}]}], "]"}], "]"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "9"}], "}"}]}], "]"}], ",", "11"}], "]"}]}], ";",
      "\[IndentingNewLine]", 
     RowBox[{"Which", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"check", "<", "10"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"ToString", "[", "check", "]"}], "\[Equal]", 
        RowBox[{"StringTake", "[", 
         RowBox[{"isbn", ",", 
          RowBox[{"{", "10", "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"check", "\[Equal]", "10"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"StringTake", "[", 
         RowBox[{"isbn", ",", 
          RowBox[{"{", "10", "}"}]}], "]"}], "\[Equal]", "\"\<X\>\""}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Recall that a ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 " statement evaluates the first argument and if it is true, returns the \
value of the second argument. If the first argument is false, then it moves \
to the third argument and evaluates that test, and so on."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkISBN", "[", "isbnExample", "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5806482405503273`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkISBN", "[", "\"\<084930149X\>\"", "]"}]], "Input"],

Cell[BoxData["False"], "Output",
 CellChangeTimes->{3.5806482405877743`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkISBN", "[", "\"\<232150031X\>\"", "]"}]], "Input"],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5806482406394377`*^9}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["4.6 Cryptography", "Section",
 CellTags->"Section6"],

Cell[TextData[{
 "In this, the final section of Chapter 4, we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to encode and decode strings using two of the approaches \
described in the textbook. Specifically, we will see how to implement a \
classical affine cypher and the RSA system. "
}], "Text"],

Cell[CellGroupData[{

Cell["Encoding Strings", "Subsection",
 CellTags->"Section6a"],

Cell["\<\
Before we can implement the encryption algorithms, we need to encode strings \
as numbers. In this manual, we will deviate slightly from the convention used \
in the textbook. Instead of assigning the letter A to 0, B to 1, and so on \
with Z assigned to 25, we will assign the space character to 0, A to 1, B to \
2, and so on with Z set to 26. We will then work modulo 27 instead of 26. \
\>", "Text"],

Cell[CellGroupData[{

Cell["Some Functions for Working with Strings", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " contains a variety of functions for working with strings. We have already \
seen, in the previous section, the ",
 StyleBox[ButtonBox["StringTake",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringTake"], "FunctionLink"],
 " function. This function is used to obtain a substring of a given string. \
Its first argument is the initial string, and the second argument specifies \
what part of the string to return. "
}], "Text"],

Cell[TextData[{
 "If the second argument to ",
 StyleBox[ButtonBox["StringTake",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringTake"], "FunctionLink"],
 " is a positive integer, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the result will be the first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " characters. For example, the following produces the first 5 characters in \
the string \[OpenCurlyDoubleQuote]The quick brown fox\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringTake", "[", 
  RowBox[{"\"\<The quick brown fox\>\"", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["\<\"The q\"\>"], "Output",
 CellChangeTimes->{3.580648240687872*^9}]
}, Open  ]],

Cell[TextData[{
 "If the second argument is a negative integer, say ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "n"}], TraditionalForm]]],
 ", the result is the final ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " characters."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringTake", "[", 
  RowBox[{"\"\<The quick brown fox\>\"", ",", 
   RowBox[{"-", "5"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"n fox\"\>"], "Output",
 CellChangeTimes->{3.580648240737556*^9}]
}, Open  ]],

Cell["\<\
To obtain the character in a particular location, as was done in Section 4.5, \
the second argument is given as a list containing the desired position. For \
example, to obtain the character in position 8, enter the following.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringTake", "[", 
  RowBox[{"\"\<The quick brown fox\>\"", ",", 
   RowBox[{"{", "8", "}"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"c\"\>"], "Output",
 CellChangeTimes->{3.580648240770713*^9}]
}, Open  ]],

Cell[TextData[{
 "With a pair of integers in a list given as the second argument, ",
 StyleBox[ButtonBox["StringTake",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringTake"], "FunctionLink"],
 " returns the substring between the given two positions. The following \
produces the substring consisting of characters 5 through 8."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringTake", "[", 
  RowBox[{"\"\<The quick brown fox\>\"", ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "8"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["\<\"quic\"\>"], "Output",
 CellChangeTimes->{3.580648240821969*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ToUpperCase",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToUpperCase"], "FunctionLink"],
 " function makes all of the letters in its input string upper case."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToUpperCase", "[", "\"\<The quick brown fox\>\"", "]"}]], "Input"],

Cell[BoxData["\<\"THE QUICK BROWN FOX\"\>"], "Output",
 CellChangeTimes->{3.580648241001553*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ToUpperCase",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToUpperCase"], "FunctionLink"],
 " command is useful in this context because it means we only have to work \
with the 26 uppercase letters and the space character instead of the full 53 \
characters, including both upper and lower case letters and space."
}], "Text"],

Cell[TextData[{
 "The next function we will need is the ",
 StyleBox[ButtonBox["Characters",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Characters"], "FunctionLink"],
 " function and its inverse ",
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " (",
 StyleBox["<>", "Input"],
 "). The ",
 StyleBox[ButtonBox["Characters",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Characters"], "FunctionLink"],
 " function takes a string and returns a list of characters."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Characters", "[", "\"\<THE QUICK BROWN FOX\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"T\"\>", ",", "\<\"H\"\>", ",", "\<\"E\"\>", ",", "\<\" \"\>", 
   ",", "\<\"Q\"\>", ",", "\<\"U\"\>", ",", "\<\"I\"\>", ",", "\<\"C\"\>", 
   ",", "\<\"K\"\>", ",", "\<\" \"\>", ",", "\<\"B\"\>", ",", "\<\"R\"\>", 
   ",", "\<\"O\"\>", ",", "\<\"W\"\>", ",", "\<\"N\"\>", ",", "\<\" \"\>", 
   ",", "\<\"F\"\>", ",", "\<\"O\"\>", ",", "\<\"X\"\>"}], "}"}]], "Output",
 CellChangeTimes->{3.5806482410592127`*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " function does the opposite. Given two or more strings as arguments, it \
joins them into one string."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringJoin", "[", 
  RowBox[{"\"\<The quick\>\"", ",", "\"\< \>\"", ",", "\"\<brown fox\>\""}], 
  "]"}]], "Input"],

Cell[BoxData["\<\"The quick brown fox\"\>"], "Output",
 CellChangeTimes->{3.580648241092537*^9}]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " also has an operator form, ",
 StyleBox["<>", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\"\<The quick\>\"", " ", "<>", " ", "\"\< \>\"", " ", "<>", " ", 
  "\"\<brown fox\>\""}]], "Input"],

Cell[BoxData["\<\"The quick brown fox\"\>"], "Output",
 CellChangeTimes->{3.580648241153698*^9}]
}, Open  ]],

Cell["\<\
It can also accept a list of strings as its arguments and will return the \
string formed by joining the members of the list. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StringJoin", "[", 
  RowBox[{"{", 
   RowBox[{
   "\"\<T\>\"", ",", "\"\<H\>\"", ",", "\"\<E\>\"", ",", "\"\< \>\"", ",", 
    "\"\<Q\>\"", ",", "\"\<U\>\"", ",", "\"\<I\>\"", ",", "\"\<C\>\"", ",", 
    "\"\<K\>\"", ",", "\"\< \>\"", ",", "\"\<B\>\"", ",", "\"\<R\>\"", ",", 
    "\"\<O\>\"", ",", "\"\<W\>\"", ",", "\"\<N\>\"", ",", "\"\< \>\"", ",", 
    "\"\<F\>\"", ",", "\"\<O\>\"", ",", "\"\<X\>\""}], "}"}], "]"}]], "Input"],

Cell[BoxData["\<\"THE QUICK BROWN FOX\"\>"], "Output",
 CellChangeTimes->{3.5806482411918592`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mapping Characters to Integers", "Subsubsection"],

Cell[TextData[{
 "To represent the function that maps characters to integers, and its \
inverse, we will use two indexed variables, ",
 StyleBox["charToNum", "Input"],
 " and ",
 StyleBox["numToChar", "Input"],
 ". In the ",
 StyleBox["charToNum", "Input"],
 " variable, the space character and capital letters will serve as the \
indices with the corresponding integers the entries. The ",
 StyleBox["numToChar", "Input"],
 " variable will be the reverse. "
}], "Text"],

Cell[TextData[{
 "To define these two indexed variables, rather than entering each individual \
assignment manually, we will apply the ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " function. Recall that ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is used to apply a function to lists of arguments. The first argument to \
",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is a function. In this case, we will create a pure function that sets a \
value of the indexed variable. The second argument to ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is a list of lists, with the inner lists containing the arguments. The \
elements of the first sublist are values of the first argument of the \
function, the elements of the second sublist values of the second argument \
and so forth. The result of ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is that the function is evaluated on corresponding pairs of elements from \
the lists."
}], "Text"],

Cell[TextData[{
 "Here, ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " is used to define ",
 StyleBox["charToNum", "Input"],
 ". Note that the parentheses are needed in the pure function, since ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (",
 StyleBox["&", "Input"],
 ") has greater precedence than ",
 StyleBox[ButtonBox["Set",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Set"], "FunctionLink"],
 " (",
 StyleBox["=", "Input"],
 ")."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MapThread", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"charToNum", "[", "#1", "]"}], "=", "#2"}], ")"}], "&"}], ",", 
    
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Characters", "[", "\"\< ABCDEFGHIJKLMNOPQRSTUVWXYZ\>\"", "]"}],
       ",", 
      RowBox[{"Range", "[", 
       RowBox[{"0", ",", "26"}], "]"}]}], "}"}]}], "]"}], ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "After executing this ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " expression, ",
 StyleBox["charToNum", "Input"],
 ", issued with a character as an index, returns the appropriate value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"charToNum", "[", "\"\<F\>\"", "]"}]], "Input"],

Cell[BoxData["6"], "Output",
 CellChangeTimes->{3.580648241300215*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["numToChar", "Input"],
 " indexed variable is created in the same way, but reversed."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"MapThread", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"numToChar", "[", "#1", "]"}], "=", "#2"}], ")"}], "&"}], ",", 
    
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Range", "[", 
       RowBox[{"0", ",", "26"}], "]"}], ",", 
      RowBox[{
      "Characters", "[", "\"\< ABCDEFGHIJKLMNOPQRSTUVWXYZ\>\"", "]"}]}], 
     "}"}]}], "]"}], ";"}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"numToChar", "[", "6", "]"}]], "Input"],

Cell[BoxData["\<\"F\"\>"], "Output",
 CellChangeTimes->{3.580648241368133*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Converting Between a String and a Numerical Representation", \
"Subsubsection"],

Cell["\<\
We now have the tools needed to encode a string as a list of numbers and a \
decode the numerical representation as a string. \
\>", "Text"],

Cell[TextData[{
 "In the ",
 StyleBox["stringToNums", "Input"],
 " function, we will first apply ",
 StyleBox[ButtonBox["ToUpperCase",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToUpperCase"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Characters",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Characters"], "FunctionLink"],
 " to produce a list of uppercase characters. Then we will use the ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") function to apply the ",
 StyleBox["charToNum", "Input"],
 " table to each character. When ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") is applied to a function (in this case a function represented by an \
indexed variable) and a list, it returns the list obtained by applying the \
function to each element of the list. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"stringToNums", "[", "s_String", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "charList", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"charList", "=", 
      RowBox[{"Characters", "[", 
       RowBox[{"ToUpperCase", "[", "s", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Map", "[", 
      RowBox[{"charToNum", ",", "charList"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"stringToNums", "[", "\"\<The quick brown fox\>\"", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "20", ",", "8", ",", "5", ",", "0", ",", "17", ",", "21", ",", "9", ",", 
   "3", ",", "11", ",", "0", ",", "2", ",", "18", ",", "15", ",", "23", ",", 
   "14", ",", "0", ",", "6", ",", "15", ",", "24"}], "}"}]], "Output",
 CellChangeTimes->{3.58064824143512*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox["numsToString", "Input"],
 " function begins with a list of integers and returns the string."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"numsToString", "[", 
   RowBox[{"numList", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "charList", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"charList", "=", 
      RowBox[{"Map", "[", 
       RowBox[{"numToChar", ",", "numList"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"StringJoin", "[", "charList", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"numsToString", "[", 
  RowBox[{"{", 
   RowBox[{
   "8", ",", "5", ",", "12", ",", "12", ",", "15", ",", "0", ",", "23", ",", 
    "15", ",", "18", ",", "12", ",", "4"}], "}"}], "]"}]], "Input"],

Cell[BoxData["\<\"HELLO WORLD\"\>"], "Output",
 CellChangeTimes->{3.5806482415158463`*^9}]
}, Open  ]],

Cell["\<\
Now that we have the ability to convert strings into a numerical \
representation and back again, we are ready to implement our encryption \
algorithms. \
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Classical Cryptography", "Subsection",
 PageBreakWithin->False,
 CellTags->"Section6b"],

Cell[TextData[{
 "We will now implement an affine cipher in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". Recall from the text that a general affine cipher has the form"
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "p", ")"}], "=", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", "\[CenterDot]", "p"}], "+", "b"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "27"}], ")"}], Cell[""]}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " is an integer corresponding to a character that is to be encrypted. We \
will refer to the pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 " as the key to the cipher. For decryption to be feasible, the key must be \
chosen so that ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " is a bijection. This amounts to choosing an ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " that is relatively prime to 27. (Note that the text uses a modulus of ",
 Cell[BoxData[
  FormBox["26", TraditionalForm]]],
 " where we use ",
 Cell[BoxData[
  FormBox["27", TraditionalForm]]],
 " because we are considering space to be an encodable character.) "
}], "Text"],

Cell[TextData[{
 "Encrypting a string requires three simple steps. First, the string is \
transformed into its numerical representation via ",
 StyleBox["stringToNums", "Input"],
 ". Second, the function ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " is applied to each number. And third, the ",
 StyleBox["numsToString", "Input"],
 " function transforms the result back into a string. Our ",
 StyleBox["affineCipher", "Input"],
 " function accepts as input a string and values of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "We ensure that the argument ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is relatively prime to 27 by imposing a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") and creating a message if it is not. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"affineCipher", "::", "arga"}], "=", 
   "\"\<Second argument must be relatively prime to 27.\>\""}], 
  ";"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Recall that following the name and arguments of a function definition with \
the condition operator ",
 StyleBox["/;", "Input"],
 " and an expression that results in true or false allows you to create \
functions that will not execute on invalid arguments."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"affineCipher", "[", 
    RowBox[{"s_String", ",", "a_Integer", ",", "b_Integer"}], "]"}], "/;", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"CoprimeQ", "[", 
      RowBox[{"a", ",", "27"}], "]"}], ",", "True", ",", 
     RowBox[{
      RowBox[{"Message", "[", 
       RowBox[{"affineCipher", "::", "arga"}], "]"}], ";", "False"}]}], 
    "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"S", ",", "T"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"S", "=", 
      RowBox[{"stringToNums", "[", "s", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"T", "=", 
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"a", "*", "#"}], "+", "b"}], ",", "27"}], "]"}], "&"}], 
        ",", "S"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"numsToString", "[", "T", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note the use of ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Input"],
 ") to apply the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", "(", "p", ")"}], TraditionalForm]]],
 ", defined as a pure function, to each character."
}], "Text"],

Cell[TextData[{
 "We now use the cipher to encrypt \[OpenCurlyDoubleQuote]The quick brown fox\
\[CloseCurlyDoubleQuote] with the key ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"5", ",", "3"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"affineCipher", "[", 
  RowBox[{"\"\<The quick brown fox\>\"", ",", "5", ",", "3"}], "]"}]], "Input"],

Cell[BoxData["\<\"VPACG URDCMLXJSCFXO\"\>"], "Output",
 CellChangeTimes->{3.58064824162071*^9}]
}, Open  ]],

Cell[TextData[{
 "To decrypt the message, we use the same function. The discussion following \
Example 4 in Section 4.5 of the text indicates that decrypting amounts to \
solving ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "\[Congruent]", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", "p"}], "+", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "27"}], ")"}]}]}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". As the text shows, we obtain ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "\[Congruent]", 
    RowBox[{
     RowBox[{
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]], "(", 
      RowBox[{"c", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "27"}], ")"}]}], "\[Congruent]", 
    RowBox[{
     RowBox[{
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]], "c"}], "-", 
     RowBox[{
      SuperscriptBox["a", 
       RowBox[{"-", "1"}]], "b", " ", 
      RowBox[{"(", 
       RowBox[{"mod", " ", "27"}], ")"}]}]}]}], TraditionalForm]]],
 ". In other words, to decrypt a message encrypted using the key ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"a", ",", "b"}], ")"}], TraditionalForm]]],
 ", we use the same procedure but with key ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]], ",", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["a", 
        RowBox[{"-", "1"}]]}], "b"}]}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "First, compute the inverse of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "=", "5"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerMod", "[", 
  RowBox[{"5", ",", 
   RowBox[{"-", "1"}], ",", "27"}], "]"}]], "Input"],

Cell[BoxData["11"], "Output",
 CellChangeTimes->{3.5806482416537848`*^9}]
}, Open  ]],

Cell[TextData[{
 "And then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox["a", 
      RowBox[{"-", "1"}]]}], "b"}], TraditionalForm]]],
 ", being sure to include the negative."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", "11"}], "*", "3"}], ",", "27"}], "]"}]], "Input"],

Cell[BoxData["21"], "Output",
 CellChangeTimes->{3.580648241691242*^9}]
}, Open  ]],

Cell[TextData[{
 "Thus the decryption key is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"11", ",", "21"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"affineCipher", "[", 
  RowBox[{"\"\<VPACG URDCMLXJSCFXO\>\"", ",", "11", ",", "21"}], 
  "]"}]], "Input"],

Cell[BoxData["\<\"THE QUICK BROWN FOX\"\>"], "Output",
 CellChangeTimes->{3.580648241720047*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["RSA Encryption", "Subsection",
 CellTags->"Section6c"],

Cell[TextData[{
 "We will now see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to implement the RSA cryptosystem. Implementing the RSA system involves \
two steps: key generation and the encryption algorithm. "
}], "Text"],

Cell["\<\
To construct keys in the RSA system, we need to find pairs of large primes, \
say with 200 digits each. Since messages can be decrypted by anyone who can \
factor the product of these primes, the two primes must be large enough so \
that their product is extremely difficult to factor. A 400 digit integer fits \
the bill since factoring requires an extremely large amount of computer time. \
\>", "Text"],

Cell[TextData[{
 "Because the use of very large prime numbers would make our examples \
impractical as examples, we shall illustrate the RSA system using smaller \
primes. We will discuss at the end of this section how you can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate large prime numbers. "
}], "Text"],

Cell[CellGroupData[{

Cell["Key Generation", "Subsubsection"],

Cell[TextData[{
 "The first step in key generation is to choose two distinct large prime \
numbers, ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ". From these, we produce the public key, which consists of the public \
modulus ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{"p", "\[CenterDot]", "q"}]}], TraditionalForm]]],
 " and the public exponent ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " which is relatively prime to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"p", "-", "1"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"q", "-", "1"}], ")"}]}]}], TraditionalForm]]],
 ". We also produce the private key, consisting of the public modulus ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and the inverse of ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " modulo ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"p", "-", "1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"q", "-", "1"}], ")"}]}], TraditionalForm]]],
 ". Since ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " is unrelated to the primes ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", it can be generated in a number of ways. For our implementation below, we \
will take ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 " to be ",
 Cell[BoxData[
  FormBox["13", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "Here is a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to handle key generation. The ",
 StyleBox["generateKeys", "Input"],
 " function accepts as input two prime numbers. It returns a list of two \
lists where the sublists are the public and private keys. That is, it returns \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"n", ",", "e"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", 
       SuperscriptBox["e", 
        RowBox[{"-", "1"}]]}], "}"}]}], "}"}], TraditionalForm]]],
 ". Given the primes ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", the procedure computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{"p", "\[CenterDot]", "q"}]}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Phi]", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"p", "-", "1"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"q", "-", "1"}], ")"}]}]}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"d", "=", 
    RowBox[{
     SuperscriptBox["e", 
      RowBox[{"-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", 
       RowBox[{"\[Phi]", "(", "n", ")"}]}], ")"}]}]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"generateKeys", "[", 
   RowBox[{
    RowBox[{"p_", "?", "PrimeQ"}], ",", 
    RowBox[{"q_", "?", "PrimeQ"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n", ",", "phin", ",", "e", ",", "d"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"e", "=", "13"}], ";", "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"p", "*", "q"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"phin", "=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"p", "-", "1"}], ")"}], "*", 
       RowBox[{"(", 
        RowBox[{"q", "-", "1"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"d", "=", 
      RowBox[{"PowerMod", "[", 
       RowBox[{"e", ",", 
        RowBox[{"-", "1"}], ",", "phin"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"n", ",", "e"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"n", ",", "d"}], "}"}]}], "}"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "In a practical RSA implementation, we would likely use some of the \
techniques discussed at the end of this section to incorporate into our ",
 StyleBox["generateKeys", "Input"],
 " procedure the generation of the primes ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["q", TraditionalForm]]],
 ", rather than passing them as arguments."
}], "Text"],

Cell[TextData[{
 "We generate keys using the prime numbers ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", "=", "59"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"q", "=", "71"}], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"keys", "=", 
  RowBox[{"generateKeys", "[", 
   RowBox[{"59", ",", "71"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"4189", ",", "13"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4189", ",", "937"}], "}"}]}], "}"}]], "Output",
 CellChangeTimes->{3.580648117922452*^9, 3.580648241802704*^9}]
}, Open  ]],

Cell["The public and private keys are:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"publickey", "=", 
  RowBox[{"keys", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4189", ",", "13"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117947547*^9, 3.580648241862368*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"privatekey", "=", 
  RowBox[{"keys", "[", 
   RowBox[{"[", "2", "]"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4189", ",", "937"}], "}"}]], "Output",
 CellChangeTimes->{3.580648117979858*^9, 3.580648241972598*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Encoding", "Subsubsection"],

Cell[TextData[{
 "Now that we have the keys, we turn to encoding the message. As described in \
the text, we encode the message in much the same way as for affine ciphers, \
except that we block groups of characters into single integers. The block \
length must be chosen so that, after conversion, the largest integer produced \
is less than the modulus ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Here, we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "4198"}], TraditionalForm]]],
 " and the largest block that can be produced is 2626 for \
\[OpenCurlyDoubleQuote]ZZ\[CloseCurlyDoubleQuote]. "
}], "Text"],

Cell["\<\
We need to ensure that this part of the process is reversible. Consider the \
string \[OpenCurlyDoubleQuote]VA\[CloseCurlyDoubleQuote]. This comprises one \
block. Since \[OpenCurlyDoubleQuote]V\[CloseCurlyDoubleQuote] has code 22 and \
\[OpenCurlyDoubleQuote]A\[CloseCurlyDoubleQuote] has code 1, it is tempting \
to code \[OpenCurlyDoubleQuote]VA\[CloseCurlyDoubleQuote] as 221. But when \
you go to convert this back to a string, it is impossible to tell if it was \
22 and 1 indicating \[OpenCurlyDoubleQuote]VA\[CloseCurlyDoubleQuote] or if \
it was 2 and 21, which represents \[OpenCurlyDoubleQuote]BU\
\[CloseCurlyDoubleQuote]. To avoid this, we code \[OpenCurlyDoubleQuote]A\
\[CloseCurlyDoubleQuote] as 01. Or, what amounts to the same thing, when we \
compose the block, we multiply the value of the first character by 100. \
\>", "Text"],

Cell[TextData[{
 "For a specific example, consider the message \[OpenCurlyDoubleQuote]SECRET \
MESSAGE\[CloseCurlyDoubleQuote]. We can use our ",
 StyleBox["stringToNums", "Input"],
 " function from above to get the numeric representation of each character. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"messageString", "=", 
  RowBox[{"stringToNums", "[", "\"\<SECRET MESSAGE\>\"", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "19", ",", "5", ",", "3", ",", "18", ",", "5", ",", "20", ",", "0", ",", 
   "13", ",", "5", ",", "19", ",", "19", ",", "1", ",", "7", ",", "5"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.580648118013386*^9, 3.580648242010542*^9}]
}, Open  ]],

Cell["\<\
You can see that the first pair should be encoded as 1905, the second as \
0318, and so on. Note that the extra 0 is unnecessary in second block, since \
0318 and 318 are numerically equal. We can obtain the desired results by \
multiplying the first number in each pair by 100 as follows. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"messageCode", "=", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"messageString", "[", 
       RowBox[{"[", 
        RowBox[{"i", "-", "1"}], "]"}], "]"}], "*", "100"}], "+", 
     RowBox[{"messageString", "[", 
      RowBox[{"[", "i", "]"}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "2", ",", 
      RowBox[{"Length", "[", "messageString", "]"}], ",", "2"}], "}"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1905", ",", "318", ",", "520", ",", "13", ",", "519", ",", "1901", ",", 
   "705"}], "}"}]], "Output",
 CellChangeTimes->{3.5806481180475283`*^9, 3.5806482420399427`*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that the final 2 in the list describing the table variable ",
 StyleBox["i", "Input"],
 " indicates that the variable should be increased by 2 each iteration."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Encryption", "Subsubsection"],

Cell[TextData[{
 "The encryption algorithm will take as input this list of integers and the \
public key. Each message block ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["m", "i"], TraditionalForm]]],
 " is transformed into a ciphertext block ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["c", "i"], TraditionalForm]]],
 " with the function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "\[Congruent]", 
    RowBox[{
     SuperscriptBox["M", "e"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"RSA", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"n_Integer", ",", "e_Integer"}], "}"}], ",", 
    RowBox[{"msg", ":", 
     RowBox[{"{", "__Integer", "}"}]}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "c", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"c", "=", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"PowerMod", "[", 
         RowBox[{"#", ",", "e", ",", "n"}], "]"}], "&"}], ",", "msg"}], 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Observe how the arguments for this function were defined. The function ",
 StyleBox["RSA", "Input"],
 " accepts a list consisting of two integers named ",
 StyleBox["n", "Input"],
 " and ",
 StyleBox["e", "Input"],
 " and a list of integers called ",
 StyleBox["msg", "Input"],
 "."
}], "Text"],

Cell["\<\
Our \[OpenCurlyDoubleQuote]SECRET MESSAGE\[CloseCurlyDoubleQuote] is \
encrypted as\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cipherText", "=", 
  RowBox[{"RSA", "[", 
   RowBox[{"publickey", ",", "messageCode"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "723", ",", "3360", ",", "2306", ",", "1979", ",", "2695", ",", "917", ",", 
   "1863"}], "}"}]], "Output",
 CellChangeTimes->{3.580648118081271*^9, 3.580648242123251*^9}]
}, Open  ]],

Cell["\<\
Decryption is accomplished by applying the same algorithm with the private \
decryption key. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RSA", "[", 
  RowBox[{"privatekey", ",", "cipherText"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1905", ",", "318", ",", "520", ",", "13", ",", "519", ",", "1901", ",", 
   "705"}], "}"}]], "Output",
 CellChangeTimes->{3.580648242177787*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that the result is identical to ",
 StyleBox["messageCode", "Input"],
 " and it can be decoded into the message \[OpenCurlyDoubleQuote]SECRET \
MESSAGE\[CloseCurlyDoubleQuote]."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generating Large Primes", "Subsubsection"],

Cell[TextData[{
 "If you were to use small primes, as we did in the example, there would be \
no real security. Anyone could factor ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the product of the primes, and then could compute the decrypting key ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " from the encrypting key ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "Using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s computational abilities, we can generate fairly large prime numbers for \
use in an RSA key. Remember that what is needed is a pair of prime numbers, \
each of about 200 digits. Moreover, they should be selected in an \
unpredictable fashion. To do this in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", we can use the ",
 StyleBox[ButtonBox["RandomPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomPrime"], "FunctionLink"],
 " function."
}], "Text"],

Cell[TextData[{
 "The first argument to ",
 StyleBox[ButtonBox["RandomPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomPrime"], "FunctionLink"],
 " is either an integer, in which case the function returns a prime up to \
that value, or a list of two integers, in which case the prime generated will \
be between them. The function can also accept a second argument in order to \
produce more than one at a time. For example, with second argument 2, the \
function will return a list of two primes."
}], "Text"],

Cell[TextData[{
 "Of course, the primes generated by ",
 StyleBox[ButtonBox["RandomPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomPrime"], "FunctionLink"],
 " are in fact pseudorandom, not truly random."
}], "Text"],

Cell[TextData[{
 "To produce two random primes with between 200 and 300 digits, we call ",
 StyleBox[ButtonBox["RandomPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomPrime"], "FunctionLink"],
 " as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomPrime", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"10", "^", "200"}], ",", 
     RowBox[{"10", "^", "300"}]}], "}"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "203811282225624478078515012702649786138281496527439371700973612663525005733\
829808284691085575889803229552341487758547691682881193084510110939501440934750\
105320461906995334938382416772382434499456075519109653526335197080906586381573\
712759700848157253301804288748583902501712285961052463578025183343447", ",", 
   "59365546738754022436340641450864883587732512591348579073759119314307629192\
350532985495885848595791014292128024112807121899383404220166868674425680721962\
309072738952130066966126911805826586497758986222947673932573148571101045565016\
262106458575853970966783858288253877707789607325765303266287865624013"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5806482423232927`*^9}]
}, Open  ]],

Cell[TextData[{
 "It is left to the reader to incorporate these ideas in improved versions of \
the ",
 StyleBox["generateKeys", "Input"],
 " and ",
 StyleBox["RSA", "Input"],
 " functions."
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 PageBreakWithin->False,
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 3", "Subsection",
 PageBreakAbove->False,
 PageBreakWithin->False,
 CellTags->"SolutionsCP3"],

Cell["\<\
Given a positive integer, find the Cantor expansion of this integer (see the \
preamble to Exercise 48 of Section 4.2).\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " Recall the definition of the Cantor expansion. Given an integer ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", the Cantor expansion of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " is"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{"a", "=", 
   RowBox[{
    RowBox[{
     SubscriptBox["a", "n"], 
     RowBox[{"n", "!"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]], "(", 
      RowBox[{"n", "-", "1"}], ")"}], "!"}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["a", "2"], 
     RowBox[{"2", "!"}]}], "+", 
    RowBox[{
     SubscriptBox["a", "1"], 
     RowBox[{"1", "!"}]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Observe that every term except for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], 
    RowBox[{"1", "!"}]}], TraditionalForm]]],
 " is divisible by 2. That is, "
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["a", "n"], 
     RowBox[{"n", "!"}]}], "+", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]], "(", 
      RowBox[{"n", "-", "1"}], ")"}], "!"}], "+", "\[CenterEllipsis]", "+", 
    RowBox[{
     SubscriptBox["a", "2"], 
     RowBox[{"2", "!"}]}], "+", 
    RowBox[{
     SubscriptBox["a", "1"], 
     RowBox[{"1", "!"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "2"}], ")"}]}]}], "=", 
   SubscriptBox["a", "1"]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "So set ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "=", 
    RowBox[{"a", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "2"}], ")"}]}]}], TraditionalForm]]],
 ". And let ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["y", "1"], TraditionalForm]]],
 " be the remainder with the 2 divided out. In other words, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["y", "1"], "=", 
    FractionBox[
     RowBox[{"a", "-", 
      SubscriptBox["a", "1"]}], "2"]}], TraditionalForm]]],
 ", or"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["y", "1"], "=", 
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{
       SubscriptBox["a", "n"], 
       RowBox[{"n", "!"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["a", 
         RowBox[{"n", "-", "1"}]], "(", 
        RowBox[{"n", "-", "1"}], ")"}], "!"}], "+", "\[CenterEllipsis]", "+", 
      
      RowBox[{
       SubscriptBox["a", "2"], 
       RowBox[{"2", "!"}]}]}], "2"], "=", 
    RowBox[{
     RowBox[{
      SubscriptBox["a", "n"], 
      FractionBox[
       RowBox[{"n", "!"}], "2"]}], "+", 
     RowBox[{
      SubscriptBox["a", 
       RowBox[{"n", "-", "1"}]], 
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"n", "-", "1"}], ")"}], "!"}], "2"]}], "+", 
     "\[CenterEllipsis]", "+", 
     RowBox[{
      SubscriptBox["a", "3"], "3"}], "+", 
     SubscriptBox["a", "2"]}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Now every term other than the last contains a factor of ",
 Cell[BoxData[
  FormBox["3", TraditionalForm]]],
 ", so set ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "2"], "=", 
    RowBox[{
     SubscriptBox["y", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "3"}], ")"}]}]}], TraditionalForm]]],
 " and let ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["y", "2"], "=", 
    FractionBox[
     RowBox[{
      SubscriptBox["y", "1"], "-", 
      SubscriptBox["a", "2"]}], "3"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "In general, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "k"], "=", 
    RowBox[{
     SubscriptBox["y", 
      RowBox[{"k", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"mod", " ", "k"}], "+", "1"}], ")"}]}]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["y", "k"], "=", 
    FractionBox[
     RowBox[{
      SubscriptBox["y", 
       RowBox[{"k", "-", "1"}]], "-", 
      SubscriptBox["a", "k"]}], 
     RowBox[{"k", "+", "1"}]]}], TraditionalForm]]],
 ". It is left to the reader to verify that this process produces the Cantor \
expansion of ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The algorithm described above leads to the function below which accepts a \
positive integer as input and returns a list of integers ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["a", "1"], ",", 
     SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["a", "n"]}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"cantorExpansion", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", 
      RowBox[{"k", "=", "1"}], ",", 
      RowBox[{"y", "=", "n"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"While", "[", 
       RowBox[{
        RowBox[{"y", "\[NotEqual]", "0"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"a", "=", 
          RowBox[{"Mod", "[", 
           RowBox[{"y", ",", 
            RowBox[{"k", "+", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"Sow", "[", "a", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"y", "=", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"y", "-", "a"}], ")"}], "/", 
           RowBox[{"(", 
            RowBox[{"k", "+", "1"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"k", "++"}]}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"cantorExpansion", "[", "471", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "1", ",", "2", ",", "4", ",", "3"}], "}"}]], "Output",
 CellChangeTimes->{3.580648242367312*^9}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computer Projects 21", "Subsection",
 CellTags->"SolutionsCP21"],

Cell["\<\
Generate a shared key using the Diffie-Hellman key exchange protocol.\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " Recall from Section 4.6 of the text the Diffie-Hellman key exchange \
protocol. "
}], "Text"],

Cell[TextData[{
 "(1) Alice and Bob agree on a prime number ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " and a primitive root ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " of ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 ". For this example, we\[CloseCurlyQuote]ll use a relatively small prime, \
one with between 6 and 8 digits."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dhPrime", "=", 
  RowBox[{"RandomPrime", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"10", "^", "6"}], ",", 
     RowBox[{"10", "^", "8"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["6397837"], "Output",
 CellChangeTimes->{3.5806481181301003`*^9, 3.580648242408729*^9}]
}, Open  ]],

Cell[TextData[{
 "For the primitive root, we\[CloseCurlyQuote]ll use ",
 StyleBox[ButtonBox["PrimitiveRoot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimitiveRoot"], "FunctionLink"],
 " to get the smallest primitive root of the prime."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dhRoot", "=", 
  RowBox[{"PrimitiveRoot", "[", "dhPrime", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["2"], "Output",
 CellChangeTimes->{3.5806481181638727`*^9, 3.5806482424400063`*^9}]
}, Open  ]],

Cell[TextData[{
 "(2) Alice chooses a secret integer ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "1"], TraditionalForm]]],
 ". Let's choose 421 since this is Computer Project 21 in Chapter 4. We need \
to compute  ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     SubscriptBox["k", "1"]], "(", 
    RowBox[{"mod", " ", "p"}], ")"}], TraditionalForm]]],
 " and send the resulting value to Bob. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"aliceSends", "=", 
  RowBox[{"PowerMod", "[", 
   RowBox[{"dhRoot", ",", "421", ",", "dhPrime"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["792900"], "Output",
 CellChangeTimes->{3.580648118197822*^9, 3.5806482424759283`*^9}]
}, Open  ]],

Cell[TextData[{
 "Note that we use ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " so that the exponentiation is computed efficiently."
}], "Text"],

Cell[TextData[{
 "(3) Bob also chooses a secret integer ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 ". From the perspective of Alice, we won't know what value of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 " that Bob chooses, only the value of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     SubscriptBox["k", "2"]], "(", 
    RowBox[{"mod", " ", "p"}], ")"}], TraditionalForm]]],
 ". So we'll have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " choose ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 " randomly in the computation. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"bobSends", "=", 
  RowBox[{"PowerMod", "[", 
   RowBox[{"dhRoot", ",", 
    RowBox[{"RandomInteger", "[", 
     RowBox[{"{", 
      RowBox[{"1", ",", "dhPrime"}], "}"}], "]"}], ",", "dhPrime"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["6220184"], "Output",
 CellChangeTimes->{3.580648118230483*^9, 3.580648242524972*^9}]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " function has similar syntax to ",
 StyleBox[ButtonBox["RandomPrime",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomPrime"], "FunctionLink"],
 ". Given a list of two integers as its argument, it returns a pseudorandom \
integer between the two integers. There are no necessary restrictions on the \
values of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "1"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 ", however, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{"k", "+", "p"}]], "\[Congruent]", 
    RowBox[{
     SuperscriptBox["a", "k"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 ", so it is no loss to assume the integers lie between 1 and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "(4) and (5) Alice computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      SuperscriptBox["a", 
       SubscriptBox["k", "2"]], ")"}], 
     SubscriptBox["k", "1"]], " ", 
    RowBox[{"(", 
     RowBox[{"mod", " ", "p"}], ")"}]}], TraditionalForm]]],
 " using the result that Bob transmitted and her ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "1"], TraditionalForm]]],
 ". Bob does the same using the value he got from Alice and his secret ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["k", "2"], TraditionalForm]]],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sharedKey", "=", 
  RowBox[{"PowerMod", "[", 
   RowBox[{"bobSends", ",", "421", ",", "dhPrime"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["2357094"], "Output",
 CellChangeTimes->{3.5806481182641697`*^9, 3.580648242559403*^9}]
}, Open  ]],

Cell["\<\
At the conclusion, both Alice and Bob know this shared key, but no one else \
does. \
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->"SolutionsCE1"],

Cell[TextData[{
 "Determine whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "p"], "-", "1"}], TraditionalForm]]],
 " is prime for each of the primes not exceeding 100."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem, we will write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " program that tests each prime ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " less than or equal to a given value to see whether ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "p"], "-", "1"}], TraditionalForm]]],
 " is a Mersenne prime. The function will output a list of those primes ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "p"], "-", "1"}], TraditionalForm]]],
 " is prime."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkMersenne", "[", "max_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", "=", "2"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"While", "[", 
       RowBox[{
        RowBox[{"p", "\[LessEqual]", "max"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"PrimeQ", "[", 
            RowBox[{
             RowBox[{"2", "^", "p"}], "-", "1"}], "]"}], ",", 
           RowBox[{"Sow", "[", "p", "]"}]}], "]"}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"p", "=", 
          RowBox[{"NextPrime", "[", "p", "]"}]}]}]}], "\[IndentingNewLine]", 
       "]"}], "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "The primes ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " less than 100 such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "p"], "-", "1"}], TraditionalForm]]],
 " is prime are:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkMersenne", "[", "100", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "3", ",", "5", ",", "7", ",", "13", ",", "17", ",", "19", ",", 
   "31", ",", "61", ",", "89"}], "}"}]], "Output",
 CellChangeTimes->{3.580648242622334*^9}]
}, Open  ]],

Cell[TextData[{
 "It is of note that there is a better test, called the Lucas-Lehmer test, \
that is more efficient than ",
 StyleBox[ButtonBox["PrimeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrimeQ"], "FunctionLink"],
 " for checking primality of numbers of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "p"], "-", "1"}], TraditionalForm]]],
 ", and can be implemented in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". For a complete description of that algorithm, consult Rosen's text on \
Number Theory. "
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 5", "Subsection",
 CellTags->"SolutionsCE5"],

Cell[TextData[{
 "Find as many primes of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a positive integer as you can. It is not known whether there are \
infinitely many such primes."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that, given a maximum ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", tests the integers of the given form."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"ce5", "[", "max_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "n", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Reap", "[", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"n", "=", "1"}], ",", 
        RowBox[{"n", "\[LessEqual]", "max"}], ",", 
        RowBox[{"n", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"PrimeQ", "[", 
           RowBox[{
            RowBox[{"n", "^", "2"}], "+", "1"}], "]"}], ",", 
          RowBox[{"Sow", "[", 
           RowBox[{
            RowBox[{"n", "^", "2"}], "+", "1"}], "]"}]}], "]"}]}], 
       "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", "]"}], "[", 
     RowBox[{"[", 
      RowBox[{"2", ",", "1"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "To save space, we\[CloseCurlyQuote]ll only compute up to a maximum of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "100"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ce5", "[", "100", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "2", ",", "5", ",", "17", ",", "37", ",", "101", ",", "197", ",", "257", 
   ",", "401", ",", "577", ",", "677", ",", "1297", ",", "1601", ",", "2917", 
   ",", "3137", ",", "4357", ",", "5477", ",", "7057", ",", "8101", ",", 
   "8837"}], "}"}]], "Output",
 CellChangeTimes->{3.580648242672176*^9}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate the list of the first 100 prime numbers larger than one \
million."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the one's complement of an arbitrary integer (see the prelude to \
Exercise 34 of Section 4.2)."
}], "Item1Exercise"],

Cell[TextData[{
 "For which odd prime moduli is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "1"}], TraditionalForm]]],
 " a square? That is, for which prime numbers ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " does there exist an integer ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "\[Congruent]", 
    RowBox[{
     RowBox[{"-", "1"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "p"}], ")"}]}]}], TraditionalForm]]],
 "?"
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine which numbers are perfect squares modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for various values of the modulus ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". For each perfect square ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 ", determine how many square roots ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " has. That is, for how many values of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "2"], "\[Congruent]", 
    RowBox[{"s", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "n"}], ")"}]}]}], TraditionalForm]]],
 ". What conjectures can you make about the number of different square roots \
an integer has modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "? (The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions ",
 StyleBox[ButtonBox["PowerMod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PowerMod"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Solve",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Solve"], "FunctionLink"],
 " may be of use.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the base 2 expansion of the 4th Fermat number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["F", "4"], "=", 
    RowBox[{
     SuperscriptBox["2", 
      SuperscriptBox["2", "4"]], "+", "1"}]}], TraditionalForm]]],
 ". Do the following for several large integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Compute the time required to calculate the remainder modulo ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " of various bases ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " raised to the power ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "4"], TraditionalForm]]],
 " (that is, to calculate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["b", 
     SubscriptBox["F", "4"]], "(", 
    RowBox[{"mod", " ", "n"}], ")"}], TraditionalForm]]],
 ") using two different methods. First, do the calculation by a \
straightforward exponentiation. Second, do it using the binary expansion of \
",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "4"], TraditionalForm]]],
 " with repeated squaring and multiplications. Why do you think ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["F", "4"], TraditionalForm]]],
 " is a good choice for the public exponent in the RSA encryption scheme? "
}], "Item1Exercise"],

Cell[TextData[{
 "Modify the function ",
 StyleBox["generateKeys", "Input"],
 " that we developed to produce the keys for the RSA system to incorporate \
the techniques for generating random large primes. Make your procedure take \
as an argument a \[OpenCurlyDoubleQuote]security\[CloseCurlyDoubleQuote] \
parameter which measures the number of digits in the primes."
}], "Item1Exercise"],

Cell[TextData[{
 "Write ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions to encode and decode English sentences into lists of integers, \
appropriate for encryption with RSA. You may ignore punctuation and insist \
that all letters are uppercase. Your functions should accept as input the \
block size."
}], "Item1Exercise"],

Cell[TextData[{
 "There are infinitely many primes of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "n"}], "+", "1"}], TraditionalForm]]],
 " and infinitely many of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "n"}], "+", "3"}], TraditionalForm]]],
 ". Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine for various values of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " whether there are more primes of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "n"}], "+", "1"}], TraditionalForm]]],
 " less than ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " than there are of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"4", "n"}], "+", "3"}], TraditionalForm]]],
 ". What conjectures can you make from this evidence?"
}], "Item1Exercise"],

Cell[TextData[{
 "Develop a function for determining whether Mersenne numbers are prime using \
the Lucas-Lehmer test as described in number theory books, such as ",
 StyleBox["Elementary Number Theory and its Applications",
  FontSlant->"Italic"],
 " by K. Rosen. How many Mersenne numbers can you test for primality using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "? "
}], "Item1Exercise"],

Cell[TextData[{
 StyleBox["Repunits",
  FontSlant->"Italic"],
 " are integers with decimal expansions consisting entirely of 1s (e.g., 11, \
111, 1111, etc.). Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to factor repunits. How many prime repunits can you find? Explore the same \
question for repunits in different base expansions."
}], "Item1Exercise"],

Cell[TextData[{
 "Compute the sequence of pseudorandom numbers generated by the linear \
congruential generator ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", 
     RowBox[{"n", "+", "1"}]], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", "\[CenterDot]", 
        SubscriptBox["x", "n"]}], "+", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "m"}], ")"}]}]}], TraditionalForm]]],
 " for various values of the multiplier ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 ", the increment ",
 Cell[BoxData[
  FormBox["c", TraditionalForm]]],
 ", and the modulus ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". For which values do you get a period of length ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " for the sequence that you generate? Formulate a conjecture."
}], "Item1Exercise"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["DivisorSigma",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DivisorSigma"], "FunctionLink"],
 " implements the function defined, for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", by: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "k"], "(", "n", ")"}], TraditionalForm]]],
 " is the sum of the ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "th powers of the positive divisors of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sigma]", "k"], "(", "n", ")"}], "=", 
    RowBox[{
     UnderscriptBox[
      StyleBox["\[Sum]",
       FontSize->16], 
      RowBox[{"d", "|", "n"}],
      LimitsPositioning->False], " ", 
     SuperscriptBox["d", "k"]}]}], TraditionalForm]]],
 ". For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], TraditionalForm]]],
 " is the number of positive divisors of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", which is sometimes also denoted ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Tau]", "(", "n", ")"}], TraditionalForm]]],
 ". Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to study the function ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Sigma]", "0"], TraditionalForm]]],
 ". What conjectures can you make about it? For example, when is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], TraditionalForm]]],
 " odd? Is there a formula for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], TraditionalForm]]],
 "? For which integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " does ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], "=", "m"}], 
   TraditionalForm]]],
 " have a solution for some integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "? Is there a formula for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", 
    RowBox[{"m", "\[CenterDot]", "n"}], ")"}], TraditionalForm]]],
 " in terms of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "m", ")"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], TraditionalForm]]],
 "? (Note: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Sigma]", "0"], "(", "n", ")"}], TraditionalForm]]],
 " is computed by ",
 StyleBox["DivisorSigma[0,n]", "Input"],
 ".)"
}], "Item1Exercise"],

Cell[TextData[{
 "A sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", 
    SubscriptBox["a", "3"], ",", "\[Ellipsis]"}], TraditionalForm]]],
 " is called ",
 StyleBox["periodic",
  FontSlant->"Italic"],
 " if there are positive integers ",
 Cell[BoxData[
  FormBox["N", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " for which ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "n"], "=", 
    SubscriptBox["a", 
     RowBox[{"n", "+", "p"}]]}], TraditionalForm]]],
 " for all ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[GreaterEqual]", "N"}], TraditionalForm]]],
 ". The least integer ",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 " for which this is true is called the ",
 StyleBox["period",
  FontSlant->"Italic"],
 " of the sequence. The sequence is said to be ",
 StyleBox["periodic modulo",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", for a positive integer ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", if the sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["a", "1"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "m"}], ")"}]}], ",", 
    RowBox[{
     SubscriptBox["a", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "m"}], ")"}]}], ",", 
    RowBox[{
     SubscriptBox["a", "3"], " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "m"}], ")"}], "\[Ellipsis]"}]}], 
   TraditionalForm]]],
 " is periodic. Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine whether the Fibonacci sequence is periodic modulo ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " for various integers ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and, if so, find the period. Can you, by examining enough different values \
of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", make any conjectures concerning the relationship between ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " and the period? Do the same thing for other sequences that you find \
interesting."
}], "Item1Exercise"],

Cell[TextData[{
 "(Class project) The Data Encryption Standard (DES) specifies a widely used \
algorithm for private key cryptography. Find a description of this algorithm \
(for example, in ",
 StyleBox["Cryptography, Theory and Practice",
  FontSlant->"Italic"],
 " by Douglas Stinson). Implement the DES in ",
 StyleBox["Mathematica.",
  FontSlant->"Italic"]
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{726, 844},
WindowMargins->{{35, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section2a",
                "Section2b", "Section2c", "Section3a", "Section3b", 
               "Section3c", "Section3d", "Section4a", "Section4b", 
               "Section4c", "Section4d", "Section5a", "Section5b", 
               "Section5c", "Section6a", "Section6b", "Section6c", 
               "SolutionsCE1", "SolutionsCE5", "SolutionsCP21", 
               "SolutionsCP3", "Section1", "Section2", "Section3", "Section4",
                "Section5", "Section6", "Solutions", "Exercises"}}]; 
           SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section2", "Section2a", "Section2b", "Section2c",
                "Section3", "Section3a", "Section3b", "Section3c", 
               "Section3d", "Section4", "Section4a", "Section4b", "Section4c",
                "Section4d", "Section5", "Section5a", "Section5b", 
               "Section5c", "Section6", "Section6a", "Section6b", "Section6c",
                "Solutions", "SolutionsCE1", "SolutionsCE5", "SolutionsCP21", 
               "SolutionsCP3"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1493, 50, 81, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[2243, 70, 83, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[10412, 386, 57, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[15327, 577, 129, 5, 27, "Subsection",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[20885, 781, 80, 1, 26, "Subsection",
   CellTags->"Section1d"]},
 "Section2"->{
  Cell[29946, 1087, 84, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[30673, 1107, 61, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[45437, 1629, 61, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section2c"->{
  Cell[52106, 1833, 67, 1, 26, "Subsection",
   CellTags->"Section2c"]},
 "Section3"->{
  Cell[57769, 2019, 81, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[58239, 2035, 52, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[63404, 2242, 65, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[68945, 2439, 72, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Section3d"->{
  Cell[85147, 2855, 95, 1, 26, "Subsection",
   CellTags->"Section3d"]},
 "Section4"->{
  Cell[93046, 3157, 65, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[93617, 3175, 62, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[105603, 3637, 75, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[124503, 4258, 58, 1, 26, "Subsection",
   CellTags->"Section4c"]},
 "Section4d"->{
  Cell[129820, 4409, 85, 1, 26, "Subsection",
   CellTags->"Section4d"]},
 "Section5"->{
  Cell[143960, 4921, 73, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[144488, 4937, 63, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[166828, 5665, 66, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Section5c"->{
  Cell[181593, 6132, 58, 1, 26, "Subsection",
   CellTags->"Section5c"]},
 "Section6"->{
  Cell[190093, 6424, 58, 1, 35, "Section",
   CellTags->"Section6"]},
 "Section6a"->{
  Cell[190511, 6438, 62, 1, 32, "Subsection",
   CellTags->"Section6a"]},
 "Section6b"->{
  Cell[205240, 6929, 93, 2, 26, "Subsection",
   PageBreakWithin->False,
   CellTags->"Section6b"]},
 "Section6c"->{
  Cell[212885, 7216, 60, 1, 26, "Subsection",
   CellTags->"Section6c"]},
 "Solutions"->{
  Cell[229346, 7774, 140, 4, 35, "Section",
   PageBreakWithin->False,
   CellTags->"Solutions"]},
 "SolutionsCP3"->{
  Cell[229511, 7782, 117, 3, 32, "Subsection",
   PageBreakAbove->False,
   PageBreakWithin->False,
   CellTags->"SolutionsCP3"]},
 "SolutionsCP21"->{
  Cell[235960, 8030, 70, 1, 26, "Subsection",
   CellTags->"SolutionsCP21"]},
 "SolutionsCE1"->{
  Cell[241655, 8248, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE1"]},
 "SolutionsCE5"->{
  Cell[244743, 8359, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE5"]},
 "Exercises"->{
  Cell[247026, 8441, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 277020, 9214},
 {"Section1a", 277102, 9217},
 {"Section1b", 277188, 9220},
 {"Section1c", 277276, 9223},
 {"Section1d", 277365, 9226},
 {"Section2", 277452, 9229},
 {"Section2a", 277537, 9232},
 {"Section2b", 277626, 9235},
 {"Section2c", 277715, 9238},
 {"Section3", 277803, 9241},
 {"Section3a", 277888, 9244},
 {"Section3b", 277977, 9247},
 {"Section3c", 278066, 9250},
 {"Section3d", 278155, 9253},
 {"Section4", 278243, 9256},
 {"Section4a", 278328, 9259},
 {"Section4b", 278417, 9262},
 {"Section4c", 278507, 9265},
 {"Section4d", 278597, 9268},
 {"Section5", 278686, 9271},
 {"Section5a", 278772, 9274},
 {"Section5b", 278862, 9277},
 {"Section5c", 278952, 9280},
 {"Section6", 279041, 9283},
 {"Section6a", 279127, 9286},
 {"Section6b", 279217, 9289},
 {"Section6c", 279334, 9293},
 {"Solutions", 279424, 9296},
 {"SolutionsCP3", 279542, 9300},
 {"SolutionsCP21", 279693, 9305},
 {"SolutionsCE1", 279790, 9308},
 {"SolutionsCE5", 279886, 9311},
 {"Exercises", 279979, 9314}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 103, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 4}}],
Cell[CellGroupData[{
Cell[707, 27, 31, 0, 55, "Section"],
Cell[741, 29, 715, 16, 80, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1493, 50, 81, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1577, 53, 641, 13, 65, "Text"],
Cell[CellGroupData[{
Cell[2243, 70, 83, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[2329, 73, 462, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[2816, 90, 86, 2, 28, "Input"],
Cell[2905, 94, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3012, 100, 81, 2, 28, "Input"],
Cell[3096, 104, 70, 1, 28, "Output"]
}, Open  ]],
Cell[3181, 108, 280, 9, 20, "Text"],
Cell[3464, 119, 642, 23, 36, "Text"],
Cell[4109, 144, 265, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[4399, 155, 95, 2, 28, "Input"],
Cell[4497, 159, 115, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4649, 167, 46, 0, 18, "Subsubsection"],
Cell[4698, 169, 261, 7, 37, "Text"],
Cell[CellGroupData[{
Cell[4984, 180, 81, 2, 28, "Input"],
Cell[5068, 184, 70, 1, 28, "Output"]
}, Open  ]],
Cell[5153, 188, 442, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[5620, 204, 87, 2, 28, "Input"],
Cell[5710, 208, 73, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5820, 214, 87, 2, 28, "Input"],
Cell[5910, 218, 76, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6035, 225, 31, 0, 18, "Subsubsection"],
Cell[6069, 227, 673, 22, 37, "Text"],
Cell[CellGroupData[{
Cell[6767, 253, 99, 3, 28, "Input"],
Cell[6869, 258, 70, 1, 28, "Output"]
}, Open  ]],
Cell[6954, 262, 375, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[7354, 275, 82, 2, 28, "Input"],
Cell[7439, 279, 71, 1, 28, "Output"]
}, Open  ]],
Cell[7525, 283, 503, 13, 50, "Text"],
Cell[8031, 298, 1187, 37, 67, "Text"],
Cell[CellGroupData[{
Cell[9243, 339, 91, 2, 28, "Input"],
Cell[9337, 343, 70, 1, 28, "Output"]
}, Open  ]],
Cell[9422, 347, 677, 20, 50, "Text"],
Cell[CellGroupData[{
Cell[10124, 371, 137, 4, 28, "Input"],
Cell[10264, 377, 87, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[10412, 386, 57, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[10472, 389, 407, 13, 23, "Text"],
Cell[CellGroupData[{
Cell[10904, 406, 154, 5, 28, "Input"],
Cell[11061, 413, 70, 1, 28, "Output"]
}, Open  ]],
Cell[11146, 417, 580, 19, 37, "Text"],
Cell[CellGroupData[{
Cell[11751, 440, 170, 5, 28, "Input"],
Cell[11924, 447, 73, 1, 28, "Output"]
}, Open  ]],
Cell[12012, 451, 333, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[12370, 467, 44, 0, 18, "Subsubsection"],
Cell[12417, 469, 1285, 39, 67, "Text"],
Cell[CellGroupData[{
Cell[13727, 512, 260, 8, 28, "Input"],
Cell[13990, 522, 202, 6, 28, "Output"]
}, Open  ]],
Cell[14207, 531, 552, 18, 36, "Text"],
Cell[14762, 551, 216, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[15003, 561, 173, 5, 28, "Input"],
Cell[15179, 568, 87, 2, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[15327, 577, 129, 5, 27, "Subsection",
 CellTags->"Section1c"],
Cell[15459, 584, 560, 20, 35, "Text"],
Cell[16022, 606, 417, 8, 50, "Text"],
Cell[16442, 616, 833, 23, 51, "Text"],
Cell[17278, 641, 212, 4, 20, "Text"],
Cell[17493, 647, 381, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[17899, 661, 92, 2, 28, "Input"],
Cell[17994, 665, 254, 9, 39, "Output"]
}, Open  ]],
Cell[18263, 677, 644, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[18932, 700, 129, 3, 28, "Input"],
Cell[19064, 705, 322, 11, 39, "Output"]
}, Open  ]],
Cell[19401, 719, 352, 8, 50, "Text"],
Cell[19756, 729, 220, 7, 29, "Input",
 InitializationCell->True],
Cell[19979, 738, 195, 6, 29, "Input",
 InitializationCell->True],
Cell[20177, 746, 328, 12, 20, "Text"],
Cell[CellGroupData[{
Cell[20530, 762, 61, 1, 28, "Input"],
Cell[20594, 765, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20701, 771, 62, 1, 28, "Input"],
Cell[20766, 774, 70, 1, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[20885, 781, 80, 1, 26, "Subsection",
 CellTags->"Section1d"],
Cell[20968, 784, 97, 2, 20, "Text"],
Cell[21068, 788, 758, 20, 53, "Text"],
Cell[21829, 810, 52, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[21906, 814, 334, 11, 28, "Input"],
Cell[22243, 827, 699, 20, 104, "Output"]
}, Open  ]],
Cell[22957, 850, 938, 24, 68, "Text"],
Cell[CellGroupData[{
Cell[23920, 878, 636, 19, 45, "Input"],
Cell[24559, 899, 1459, 50, 120, "Output"]
}, Open  ]],
Cell[26033, 952, 172, 3, 35, "Text"],
Cell[26208, 957, 1024, 31, 61, "Input",
 InitializationCell->True],
Cell[27235, 990, 1030, 31, 61, "Input",
 InitializationCell->True],
Cell[28268, 1023, 58, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[28351, 1027, 72, 1, 29, "Input"],
Cell[28426, 1030, 1459, 50, 120, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[29946, 1087, 84, 1, 35, "Section",
 CellTags->"Section2"],
Cell[30033, 1090, 615, 13, 65, "Text"],
Cell[CellGroupData[{
Cell[30673, 1107, 61, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[30737, 1110, 545, 17, 37, "Text"],
Cell[31285, 1129, 575, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[31885, 1146, 69, 1, 29, "Input"],
Cell[31957, 1149, 135, 3, 28, "Output"]
}, Open  ]],
Cell[32107, 1155, 244, 5, 35, "Text"],
Cell[32354, 1162, 516, 15, 36, "Text"],
Cell[CellGroupData[{
Cell[32895, 1181, 92, 2, 29, "Input"],
Cell[32990, 1185, 158, 4, 28, "Output"]
}, Open  ]],
Cell[33163, 1192, 784, 21, 52, "Text"],
Cell[CellGroupData[{
Cell[33972, 1217, 95, 2, 29, "Input"],
Cell[34070, 1221, 148, 3, 28, "Output"]
}, Open  ]],
Cell[34233, 1227, 527, 12, 51, "Text"],
Cell[CellGroupData[{
Cell[34785, 1243, 102, 2, 29, "Input"],
Cell[34890, 1247, 201, 5, 28, "Output"]
}, Open  ]],
Cell[35106, 1255, 693, 17, 52, "Text"],
Cell[CellGroupData[{
Cell[35824, 1276, 92, 2, 29, "Input"],
Cell[35919, 1280, 83, 1, 28, "Output"]
}, Open  ]],
Cell[36017, 1284, 124, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[36166, 1291, 57, 1, 28, "Input"],
Cell[36226, 1294, 75, 1, 29, "Output"]
}, Open  ]],
Cell[36316, 1298, 677, 16, 52, "Text"],
Cell[CellGroupData[{
Cell[37018, 1318, 95, 2, 29, "Input"],
Cell[37116, 1322, 82, 1, 28, "Output"]
}, Open  ]],
Cell[37213, 1326, 571, 16, 37, "Text"],
Cell[37787, 1344, 697, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[38509, 1366, 130, 3, 29, "Input"],
Cell[38642, 1371, 75, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38754, 1377, 74, 1, 29, "Input"],
Cell[38831, 1380, 75, 1, 28, "Output"]
}, Open  ]],
Cell[38921, 1384, 295, 8, 35, "Text"],
Cell[CellGroupData[{
Cell[39241, 1396, 179, 5, 29, "Input"],
Cell[39423, 1403, 73, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[39533, 1409, 99, 2, 29, "Input"],
Cell[39635, 1413, 75, 1, 28, "Output"]
}, Open  ]],
Cell[39725, 1417, 333, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[40083, 1428, 99, 2, 29, "Input"],
Cell[40185, 1432, 75, 1, 28, "Output"]
}, Open  ]],
Cell[40275, 1436, 513, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[40813, 1453, 104, 2, 29, "Input"],
Cell[40920, 1457, 85, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[41042, 1463, 111, 2, 29, "Input"],
Cell[41156, 1467, 73, 1, 28, "Output"]
}, Open  ]],
Cell[41244, 1471, 718, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[41987, 1494, 90, 2, 28, "Input"],
Cell[42080, 1498, 200, 7, 40, "Output"]
}, Open  ]],
Cell[42295, 1508, 394, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[42714, 1522, 32, 0, 28, "Input"],
Cell[42749, 1524, 72, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42858, 1530, 61, 0, 18, "Subsubsection"],
Cell[42922, 1532, 467, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[43414, 1548, 96, 2, 29, "Input"],
Cell[43513, 1552, 73, 1, 28, "Output"]
}, Open  ]],
Cell[43601, 1556, 185, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[43811, 1566, 90, 2, 29, "Input"],
Cell[43904, 1570, 81, 1, 28, "Output"]
}, Open  ]],
Cell[44000, 1574, 242, 10, 20, "Text"],
Cell[44245, 1586, 489, 12, 36, "Text"],
Cell[44737, 1600, 287, 8, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[45049, 1612, 109, 2, 29, "Input"],
Cell[45161, 1616, 81, 1, 28, "Output"]
}, Open  ]],
Cell[45257, 1620, 131, 3, 20, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[45437, 1629, 61, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[45501, 1632, 673, 12, 81, "Text"],
Cell[46177, 1646, 831, 20, 68, "Text"],
Cell[CellGroupData[{
Cell[47033, 1670, 142, 4, 28, "Input"],
Cell[47178, 1676, 168, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47383, 1685, 97, 2, 29, "Input"],
Cell[47483, 1689, 189, 4, 28, "Output"]
}, Open  ]],
Cell[47687, 1696, 1340, 42, 83, "Text"],
Cell[49030, 1740, 342, 8, 36, "Text"],
Cell[49375, 1750, 2241, 60, 234, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[51641, 1814, 245, 7, 28, "Input"],
Cell[51889, 1823, 168, 4, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[52106, 1833, 67, 1, 26, "Subsection",
 CellTags->"Section2c"],
Cell[52176, 1836, 300, 5, 35, "Text"],
Cell[52479, 1843, 764, 22, 53, "Text"],
Cell[CellGroupData[{
Cell[53268, 1869, 154, 4, 29, "Input",
 InitializationCell->True],
Cell[53425, 1875, 156, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53618, 1883, 157, 4, 29, "Input"],
Cell[53778, 1889, 192, 5, 28, "Output"]
}, Open  ]],
Cell[53985, 1897, 333, 10, 36, "Text"],
Cell[54321, 1909, 283, 9, 37, "Text"],
Cell[54607, 1920, 725, 21, 68, "Text"],
Cell[55335, 1943, 58, 0, 20, "Text"],
Cell[55396, 1945, 1857, 50, 236, "Input",
 InitializationCell->True],
Cell[57256, 1997, 71, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[57352, 2001, 208, 6, 29, "Input"],
Cell[57563, 2009, 145, 3, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[57769, 2019, 81, 1, 35, "Section",
 CellTags->"Section3"],
Cell[57853, 2022, 361, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[58239, 2035, 52, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[58294, 2038, 194, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[58513, 2048, 46, 0, 18, "Subsubsection"],
Cell[58562, 2050, 235, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[58822, 2061, 59, 1, 28, "Input"],
Cell[58884, 2064, 75, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[58996, 2070, 60, 1, 28, "Input"],
Cell[59059, 2073, 74, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[59170, 2079, 107, 3, 28, "Input"],
Cell[59280, 2084, 73, 1, 28, "Output"]
}, Open  ]],
Cell[59368, 2088, 864, 19, 68, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60269, 2112, 39, 0, 18, "Subsubsection"],
Cell[60311, 2114, 331, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[60667, 2130, 58, 1, 28, "Input"],
Cell[60728, 2133, 69, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[60834, 2139, 58, 1, 28, "Input"],
Cell[60895, 2142, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61002, 2148, 164, 5, 28, "Input"],
Cell[61169, 2155, 325, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61531, 2167, 63, 1, 28, "Input"],
Cell[61597, 2170, 76, 1, 28, "Output"]
}, Open  ]],
Cell[61688, 2174, 374, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[62087, 2188, 65, 1, 28, "Input"],
Cell[62155, 2191, 75, 1, 28, "Output"]
}, Open  ]],
Cell[62245, 2195, 688, 20, 52, "Text"],
Cell[CellGroupData[{
Cell[62958, 2219, 88, 2, 28, "Input"],
Cell[63049, 2223, 73, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[63159, 2229, 107, 3, 28, "Input"],
Cell[63269, 2234, 74, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[63404, 2242, 65, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[63472, 2245, 443, 13, 37, "Text"],
Cell[CellGroupData[{
Cell[63940, 2262, 68, 1, 29, "Input"],
Cell[64011, 2265, 213, 7, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64261, 2277, 74, 1, 29, "Input"],
Cell[64338, 2280, 278, 9, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64653, 2294, 92, 2, 29, "Input"],
Cell[64748, 2298, 358, 12, 28, "Output"]
}, Open  ]],
Cell[65121, 2313, 1401, 44, 73, "Text"],
Cell[66525, 2359, 609, 15, 52, "Text"],
Cell[CellGroupData[{
Cell[67159, 2378, 110, 2, 29, "Input"],
Cell[67272, 2382, 408, 13, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[67717, 2400, 133, 3, 29, "Input"],
Cell[67853, 2405, 315, 10, 28, "Output"]
}, Open  ]],
Cell[68183, 2418, 725, 16, 65, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[68945, 2439, 72, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[69020, 2442, 556, 17, 60, "Text"],
Cell[69579, 2461, 550, 17, 36, "Text"],
Cell[70132, 2480, 964, 28, 53, "Text"],
Cell[CellGroupData[{
Cell[71121, 2512, 63, 1, 28, "Input"],
Cell[71187, 2515, 72, 1, 28, "Output"]
}, Open  ]],
Cell[71274, 2519, 349, 12, 21, "Text"],
Cell[71626, 2533, 278, 9, 29, "Input"],
Cell[71907, 2544, 924, 26, 62, "Text"],
Cell[72834, 2572, 2002, 51, 115, "Text"],
Cell[74839, 2625, 181, 5, 29, "Input"],
Cell[75023, 2632, 309, 9, 29, "Input"],
Cell[CellGroupData[{
Cell[75357, 2645, 411, 13, 47, "Input"],
Cell[75771, 2660, 8935, 176, 240, "Output"]
}, Open  ]],
Cell[84721, 2839, 389, 11, 45, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[85147, 2855, 95, 1, 26, "Subsection",
 CellTags->"Section3d"],
Cell[85245, 2858, 596, 18, 37, "Text"],
Cell[CellGroupData[{
Cell[85866, 2880, 79, 2, 28, "Input"],
Cell[85948, 2884, 72, 1, 28, "Output"]
}, Open  ]],
Cell[86035, 2888, 271, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[86331, 2897, 90, 2, 28, "Input"],
Cell[86424, 2901, 72, 1, 28, "Output"]
}, Open  ]],
Cell[86511, 2905, 219, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[86755, 2916, 79, 2, 28, "Input"],
Cell[86837, 2920, 71, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86945, 2926, 92, 2, 28, "Input"],
Cell[87040, 2930, 72, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87149, 2936, 41, 0, 18, "Subsubsection"],
Cell[87193, 2938, 168, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[87386, 2945, 81, 2, 28, "Input"],
Cell[87470, 2949, 70, 1, 28, "Output"]
}, Open  ]],
Cell[87555, 2953, 202, 6, 21, "Text"],
Cell[87760, 2961, 300, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[88085, 2975, 84, 2, 29, "Input"],
Cell[88172, 2979, 74, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88283, 2985, 86, 2, 29, "Input"],
Cell[88372, 2989, 73, 1, 28, "Output"]
}, Open  ]],
Cell[88460, 2993, 848, 26, 42, "Text"],
Cell[89311, 3021, 74, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[89410, 3025, 97, 2, 29, "Input"],
Cell[89510, 3029, 73, 1, 28, "Output"]
}, Open  ]],
Cell[89598, 3033, 173, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[89796, 3040, 94, 2, 28, "Input"],
Cell[89893, 3044, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90000, 3050, 99, 2, 29, "Input"],
Cell[90102, 3054, 74, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[90225, 3061, 57, 0, 18, "Subsubsection"],
Cell[90285, 3063, 1320, 41, 81, "Text"],
Cell[91608, 3106, 618, 14, 66, "Text"],
Cell[CellGroupData[{
Cell[92251, 3124, 91, 2, 28, "Input"],
Cell[92345, 3128, 186, 6, 28, "Output"]
}, Open  ]],
Cell[92546, 3137, 439, 13, 35, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[93046, 3157, 65, 1, 35, "Section",
 CellTags->"Section4"],
Cell[93114, 3160, 478, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[93617, 3175, 62, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[93682, 3178, 420, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[94127, 3190, 97, 3, 18, "Subsubsection"],
Cell[94227, 3195, 614, 24, 35, "Text"],
Cell[94844, 3221, 1060, 32, 69, "Text"],
Cell[CellGroupData[{
Cell[95929, 3257, 259, 8, 29, "Input",
 InitializationCell->True],
Cell[96191, 3267, 210, 6, 28, "Output"]
}, Open  ]],
Cell[96416, 3276, 311, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[96752, 3289, 33, 0, 28, "Input"],
Cell[96788, 3291, 72, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[96897, 3297, 33, 0, 28, "Input"],
Cell[96933, 3299, 88, 2, 28, "Output"]
}, Open  ]],
Cell[97036, 3304, 614, 24, 35, "Text"],
Cell[CellGroupData[{
Cell[97675, 3332, 110, 3, 28, "Input"],
Cell[97788, 3337, 70, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[97907, 3344, 94, 3, 18, "Subsubsection"],
Cell[98004, 3349, 648, 22, 46, "Text"],
Cell[98655, 3373, 483, 15, 39, "Text"],
Cell[CellGroupData[{
Cell[99163, 3392, 94, 2, 28, "Input"],
Cell[99260, 3396, 70, 1, 28, "Output"]
}, Open  ]],
Cell[99345, 3400, 712, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[100082, 3422, 118, 3, 28, "Input"],
Cell[100203, 3427, 74, 1, 28, "Output"]
}, Open  ]],
Cell[100292, 3431, 137, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[100454, 3438, 114, 3, 28, "Input"],
Cell[100571, 3443, 475, 10, 24, "Message"],
Cell[101049, 3455, 157, 4, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[101255, 3465, 44, 0, 18, "Subsubsection"],
Cell[101302, 3467, 609, 19, 36, "Text"],
Cell[CellGroupData[{
Cell[101936, 3490, 174, 5, 28, "Input"],
Cell[102113, 3497, 145, 4, 28, "Output"]
}, Open  ]],
Cell[102273, 3504, 757, 23, 52, "Text"],
Cell[103033, 3529, 335, 11, 20, "Text"],
Cell[CellGroupData[{
Cell[103393, 3544, 169, 5, 28, "Input"],
Cell[103565, 3551, 87, 2, 28, "Output"]
}, Open  ]],
Cell[103667, 3556, 285, 10, 20, "Text"],
Cell[CellGroupData[{
Cell[103977, 3570, 174, 5, 28, "Input"],
Cell[104154, 3577, 228, 7, 28, "Output"]
}, Open  ]],
Cell[104397, 3587, 712, 25, 53, "Text"],
Cell[CellGroupData[{
Cell[105134, 3616, 260, 8, 28, "Input"],
Cell[105397, 3626, 145, 4, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[105603, 3637, 75, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[105681, 3640, 105, 3, 20, "Text"],
Cell[105789, 3645, 642, 19, 75, "Equation"],
Cell[106434, 3666, 933, 27, 51, "Text"],
Cell[107370, 3695, 455, 12, 72, "Equation"],
Cell[CellGroupData[{
Cell[107850, 3711, 232, 6, 28, "Input"],
Cell[108085, 3719, 75, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108197, 3725, 44, 0, 18, "Subsubsection"],
Cell[108244, 3727, 538, 12, 51, "Text"],
Cell[108785, 3741, 858, 20, 67, "Text"],
Cell[109646, 3763, 185, 5, 29, "Input",
 InitializationCell->True],
Cell[109834, 3770, 183, 5, 29, "Input",
 InitializationCell->True],
Cell[110020, 3777, 163, 4, 21, "Text"],
Cell[110186, 3783, 854, 24, 92, "Input",
 InitializationCell->True],
Cell[111043, 3809, 991, 28, 68, "Text"],
Cell[112037, 3839, 1583, 42, 85, "Text"],
Cell[113623, 3883, 436, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[114084, 3899, 205, 6, 29, "Input"],
Cell[114292, 3907, 73, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[114402, 3913, 194, 6, 29, "Input"],
Cell[114599, 3921, 237, 7, 24, "Message"],
Cell[114839, 3930, 236, 7, 24, "Message"],
Cell[115078, 3939, 74, 1, 28, "Output"]
}, Open  ]],
Cell[115167, 3943, 419, 13, 37, "Text"],
Cell[115589, 3958, 324, 10, 29, "myCode"],
Cell[115916, 3970, 1903, 57, 146, "Text"],
Cell[117822, 4029, 474, 11, 36, "Text"],
Cell[118299, 4042, 1238, 39, 68, "Text"],
Cell[119540, 4083, 2449, 91, 100, "Text"],
Cell[121992, 4176, 1900, 54, 173, "Input"],
Cell[123895, 4232, 222, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[124142, 4242, 225, 6, 28, "Input"],
Cell[124370, 4250, 72, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[124503, 4258, 58, 1, 26, "Subsection",
 CellTags->"Section4c"],
Cell[124564, 4261, 749, 24, 53, "Text"],
Cell[125316, 4287, 1219, 31, 84, "Text"],
Cell[126538, 4320, 1103, 29, 121, "Input",
 InitializationCell->True],
Cell[127644, 4351, 637, 18, 51, "Text"],
Cell[128284, 4371, 138, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[128447, 4380, 97, 2, 29, "Input"],
Cell[128547, 4384, 1224, 19, 119, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[129820, 4409, 85, 1, 26, "Subsection",
 CellTags->"Section4d"],
Cell[129908, 4412, 164, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[130097, 4421, 40, 0, 18, "Subsubsection"],
Cell[130140, 4423, 431, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[130596, 4437, 67, 1, 28, "Input"],
Cell[130666, 4440, 70, 1, 28, "Output"]
}, Open  ]],
Cell[130751, 4444, 867, 28, 51, "Text"],
Cell[131621, 4474, 1224, 38, 72, "Text"],
Cell[132848, 4514, 356, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[133229, 4526, 96, 2, 29, "Input"],
Cell[133328, 4530, 72, 1, 28, "Output"]
}, Open  ]],
Cell[133415, 4534, 1200, 47, 42, "Text"],
Cell[CellGroupData[{
Cell[134640, 4585, 202, 6, 28, "Input"],
Cell[134845, 4593, 136, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135018, 4601, 95, 2, 28, "Input"],
Cell[135116, 4605, 70, 1, 28, "Output"]
}, Open  ]],
Cell[135201, 4609, 1520, 56, 69, "Text"],
Cell[CellGroupData[{
Cell[136746, 4669, 96, 2, 29, "Input"],
Cell[136845, 4673, 70, 1, 28, "Output"]
}, Open  ]],
Cell[136930, 4677, 62, 0, 20, "Text"],
Cell[136995, 4679, 1091, 34, 66, "Text"],
Cell[138089, 4715, 453, 14, 45, "Input",
 InitializationCell->True],
Cell[138545, 4731, 592, 17, 36, "Text"],
Cell[139140, 4750, 1107, 30, 83, "Text"],
Cell[140250, 4782, 157, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[140432, 4790, 71, 1, 28, "Input"],
Cell[140506, 4793, 135, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[140690, 4802, 44, 0, 18, "Subsubsection"],
Cell[140737, 4804, 1472, 49, 66, "Text"],
Cell[142212, 4855, 1354, 44, 72, "Text"],
Cell[143569, 4901, 114, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[143708, 4908, 106, 2, 29, "Input"],
Cell[143817, 4912, 70, 1, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[143960, 4921, 73, 1, 35, "Section",
 CellTags->"Section5"],
Cell[144036, 4924, 427, 9, 50, "Text"],
Cell[CellGroupData[{
Cell[144488, 4937, 63, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[144554, 4940, 293, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[144872, 4949, 41, 0, 18, "Subsubsection"],
Cell[144916, 4951, 198, 4, 35, "Text"],
Cell[145117, 4957, 418, 13, 59, "Input",
 InitializationCell->True],
Cell[145538, 4972, 247, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[145810, 4980, 171, 5, 28, "Input",
 InitializationCell->True],
Cell[145984, 4987, 718, 11, 44, "Output"]
}, Open  ]],
Cell[146717, 5001, 811, 21, 66, "Text"],
Cell[147531, 5024, 153, 3, 20, "Text"],
Cell[147687, 5029, 207, 6, 29, "Input",
 InitializationCell->True],
Cell[147897, 5037, 28, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[147950, 5041, 125, 3, 28, "Input"],
Cell[148078, 5046, 71, 1, 28, "Output"]
}, Open  ]],
Cell[148164, 5050, 193, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[148382, 5059, 149, 4, 28, "Input",
 InitializationCell->True],
Cell[148534, 5065, 146, 3, 28, "Output"]
}, Open  ]],
Cell[148695, 5071, 122, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[148842, 5078, 91, 2, 28, "Input"],
Cell[148936, 5082, 124, 3, 28, "Output"]
}, Open  ]],
Cell[149075, 5088, 169, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[149269, 5095, 143, 4, 28, "Input"],
Cell[149415, 5101, 73, 1, 28, "Output"]
}, Open  ]],
Cell[149503, 5105, 290, 9, 21, "Text"],
Cell[CellGroupData[{
Cell[149818, 5118, 115, 3, 28, "Input"],
Cell[149936, 5123, 73, 1, 28, "Output"]
}, Open  ]],
Cell[150024, 5127, 129, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[150178, 5135, 125, 3, 28, "Input"],
Cell[150306, 5140, 70, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150413, 5146, 148, 4, 28, "Input",
 InitializationCell->True],
Cell[150564, 5152, 145, 3, 28, "Output"]
}, Open  ]],
Cell[150724, 5158, 145, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[150894, 5166, 125, 3, 28, "Input"],
Cell[151022, 5171, 71, 1, 28, "Output"]
}, Open  ]],
Cell[151108, 5175, 199, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[151332, 5185, 122, 3, 28, "Input"],
Cell[151457, 5190, 72, 1, 28, "Output"]
}, Open  ]],
Cell[151544, 5194, 198, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[151767, 5203, 149, 4, 28, "Input",
 InitializationCell->True],
Cell[151919, 5209, 142, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[152110, 5218, 41, 0, 18, "Subsubsection"],
Cell[152154, 5220, 137, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[152316, 5228, 40, 0, 28, "Input"],
Cell[152359, 5230, 864, 17, 59, "Output"]
}, Open  ]],
Cell[153238, 5250, 668, 18, 51, "Text"],
Cell[153909, 5270, 842, 23, 91, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[154776, 5297, 78, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[154879, 5302, 276, 7, 21, "Print"],
Cell[155158, 5311, 278, 7, 21, "Print"],
Cell[155439, 5320, 278, 7, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[155778, 5334, 59, 0, 18, "Subsubsection"],
Cell[155840, 5336, 364, 9, 37, "Text"],
Cell[156207, 5347, 1828, 46, 131, "Text"],
Cell[158038, 5395, 666, 12, 82, "Text"],
Cell[158707, 5409, 103, 4, 21, "Text"],
Cell[158813, 5415, 1261, 34, 151, "Input",
 InitializationCell->True],
Cell[160077, 5451, 41, 0, 20, "Text"],
Cell[160121, 5453, 120, 3, 28, "Input",
 InitializationCell->True],
Cell[160244, 5458, 120, 3, 28, "Input",
 InitializationCell->True],
Cell[160367, 5463, 120, 3, 28, "Input",
 InitializationCell->True],
Cell[160490, 5468, 89, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[160604, 5476, 78, 1, 29, "Input"],
Cell[CellGroupData[{
Cell[160707, 5481, 278, 7, 21, "Print"],
Cell[160988, 5490, 280, 7, 21, "Print"],
Cell[161271, 5499, 278, 7, 21, "Print"],
Cell[161552, 5508, 278, 7, 21, "Print"],
Cell[161833, 5517, 280, 7, 21, "Print"],
Cell[162116, 5526, 280, 7, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[162457, 5540, 43, 0, 18, "Subsubsection"],
Cell[162503, 5542, 482, 7, 65, "Text"],
Cell[162988, 5551, 1094, 27, 84, "Text"],
Cell[164085, 5580, 179, 5, 29, "Input",
 InitializationCell->True],
Cell[164267, 5587, 348, 6, 50, "Text"],
Cell[164618, 5595, 1695, 42, 227, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[166338, 5641, 70, 1, 28, "Input"],
Cell[166411, 5644, 122, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166570, 5652, 70, 1, 28, "Input"],
Cell[166643, 5655, 124, 3, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[166828, 5665, 66, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[166897, 5668, 765, 11, 95, "Text"],
Cell[167665, 5681, 1817, 57, 97, "Text"],
Cell[CellGroupData[{
Cell[169507, 5742, 118, 3, 28, "Input"],
Cell[169628, 5747, 74, 1, 28, "Output"]
}, Open  ]],
Cell[169717, 5751, 433, 9, 51, "Text"],
Cell[170153, 5762, 823, 18, 81, "Text"],
Cell[170979, 5782, 48, 0, 20, "Text"],
Cell[171030, 5784, 1276, 37, 137, "Input",
 InitializationCell->True],
Cell[172309, 5823, 75, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[172409, 5827, 119, 3, 28, "Input",
 InitializationCell->True],
Cell[172531, 5832, 256, 5, 28, "Output"]
}, Open  ]],
Cell[172802, 5840, 841, 28, 57, "Text"],
Cell[173646, 5870, 1611, 47, 152, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[175282, 5921, 121, 3, 28, "Input",
 InitializationCell->True],
Cell[175406, 5926, 281, 6, 28, "Output"]
}, Open  ]],
Cell[175702, 5935, 598, 17, 37, "Text"],
Cell[176303, 5954, 109, 4, 21, "Text"],
Cell[176415, 5960, 448, 13, 59, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[176888, 5977, 105, 2, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[177018, 5983, 276, 7, 21, "Print"],
Cell[177297, 5992, 276, 7, 21, "Print"],
Cell[177576, 6001, 280, 7, 21, "Print"],
Cell[177859, 6010, 278, 7, 21, "Print"],
Cell[178140, 6019, 278, 7, 21, "Print"],
Cell[178421, 6028, 276, 7, 21, "Print"],
Cell[178700, 6037, 278, 7, 21, "Print"],
Cell[178981, 6046, 278, 7, 21, "Print"],
Cell[179262, 6055, 274, 7, 21, "Print"],
Cell[179539, 6064, 276, 7, 21, "Print"],
Cell[179818, 6073, 307, 7, 21, "Print"],
Cell[180128, 6082, 276, 7, 21, "Print"],
Cell[180407, 6091, 278, 7, 21, "Print"],
Cell[180688, 6100, 280, 7, 21, "Print"],
Cell[180971, 6109, 280, 7, 21, "Print"],
Cell[181254, 6118, 278, 7, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[181593, 6132, 58, 1, 26, "Subsection",
 CellTags->"Section5c"],
Cell[181654, 6135, 200, 4, 35, "Text"],
Cell[181857, 6141, 588, 20, 52, "Equation"],
Cell[182448, 6163, 166, 7, 20, "Text"],
Cell[182617, 6172, 229, 6, 36, "Text"],
Cell[CellGroupData[{
Cell[182871, 6182, 103, 2, 29, "Input",
 InitializationCell->True],
Cell[182977, 6186, 111, 1, 28, "Output"]
}, Open  ]],
Cell[183103, 6190, 647, 16, 52, "Text"],
Cell[CellGroupData[{
Cell[183775, 6210, 120, 3, 29, "Input"],
Cell[183898, 6215, 80, 1, 28, "Output"]
}, Open  ]],
Cell[183993, 6219, 64, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[184082, 6223, 153, 4, 29, "Input"],
Cell[184238, 6229, 201, 7, 39, "Output"]
}, Open  ]],
Cell[184454, 6239, 706, 19, 52, "Text"],
Cell[CellGroupData[{
Cell[185185, 6262, 195, 5, 29, "Input"],
Cell[185383, 6269, 191, 7, 39, "Output"]
}, Open  ]],
Cell[185589, 6279, 315, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[185929, 6290, 92, 2, 29, "Input"],
Cell[186024, 6294, 199, 7, 39, "Output"]
}, Open  ]],
Cell[186238, 6304, 1489, 39, 97, "Text"],
Cell[187730, 6345, 1375, 36, 155, "Input",
 InitializationCell->True],
Cell[189108, 6383, 360, 8, 36, "Text"],
Cell[CellGroupData[{
Cell[189493, 6395, 72, 1, 29, "Input"],
Cell[189568, 6398, 75, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[189680, 6404, 79, 1, 28, "Input"],
Cell[189762, 6407, 76, 1, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[189875, 6413, 79, 1, 28, "Input"],
Cell[189957, 6416, 75, 1, 28, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[190093, 6424, 58, 1, 35, "Section",
 CellTags->"Section6"],
Cell[190154, 6427, 332, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[190511, 6438, 62, 1, 32, "Subsection",
 CellTags->"Section6a"],
Cell[190576, 6441, 413, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[191014, 6451, 64, 0, 18, "Subsubsection"],
Cell[191081, 6453, 498, 11, 51, "Text"],
Cell[191582, 6466, 494, 13, 36, "Text"],
Cell[CellGroupData[{
Cell[192101, 6483, 112, 2, 29, "Input"],
Cell[192216, 6487, 82, 1, 29, "Output"]
}, Open  ]],
Cell[192313, 6491, 248, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[192586, 6504, 131, 3, 29, "Input"],
Cell[192720, 6509, 82, 1, 28, "Output"]
}, Open  ]],
Cell[192817, 6513, 250, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[193092, 6521, 136, 3, 29, "Input"],
Cell[193231, 6526, 78, 1, 28, "Output"]
}, Open  ]],
Cell[193324, 6530, 344, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[193693, 6541, 161, 4, 29, "Input"],
Cell[193857, 6547, 81, 1, 29, "Output"]
}, Open  ]],
Cell[193953, 6551, 220, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[194198, 6561, 90, 1, 29, "Input"],
Cell[194291, 6564, 96, 1, 28, "Output"]
}, Open  ]],
Cell[194402, 6568, 374, 8, 36, "Text"],
Cell[194779, 6578, 544, 16, 36, "Text"],
Cell[CellGroupData[{
Cell[195348, 6598, 89, 1, 28, "Input"],
Cell[195440, 6601, 455, 7, 28, "Output"]
}, Open  ]],
Cell[195910, 6611, 253, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[196188, 6622, 139, 3, 29, "Input"],
Cell[196330, 6627, 96, 1, 29, "Output"]
}, Open  ]],
Cell[196441, 6631, 203, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[196669, 6642, 124, 2, 29, "Input"],
Cell[196796, 6646, 96, 1, 29, "Output"]
}, Open  ]],
Cell[196907, 6650, 150, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[197082, 6657, 456, 8, 45, "Input"],
Cell[197541, 6667, 98, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[197688, 6674, 55, 0, 18, "Subsubsection"],
Cell[197746, 6676, 470, 12, 53, "Text"],
Cell[198219, 6690, 1289, 29, 99, "Text"],
Cell[199511, 6721, 588, 20, 37, "Text"],
Cell[200102, 6743, 450, 15, 45, "Input",
 InitializationCell->True],
Cell[200555, 6760, 287, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[200867, 6772, 70, 1, 28, "Input"],
Cell[200940, 6775, 70, 1, 28, "Output"]
}, Open  ]],
Cell[201025, 6779, 132, 4, 21, "Text"],
Cell[201160, 6785, 456, 16, 45, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[201641, 6805, 62, 1, 28, "Input"],
Cell[201706, 6808, 78, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[201833, 6815, 85, 1, 18, "Subsubsection"],
Cell[201921, 6818, 150, 3, 20, "Text"],
Cell[202074, 6823, 968, 28, 67, "Text"],
Cell[203045, 6853, 508, 14, 77, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[203578, 6871, 91, 1, 29, "Input"],
Cell[203672, 6874, 305, 6, 28, "Output"]
}, Open  ]],
Cell[203992, 6883, 140, 4, 21, "Text"],
Cell[204135, 6889, 528, 15, 77, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[204688, 6908, 218, 5, 29, "Input"],
Cell[204909, 6915, 90, 1, 28, "Output"]
}, Open  ]],
Cell[205014, 6919, 177, 4, 35, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[205240, 6929, 93, 2, 26, "Subsection",
 PageBreakWithin->False,
 CellTags->"Section6b"],
Cell[205336, 6933, 240, 7, 20, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[205579, 6942, 290, 10, 26, "Equation"],
Cell[205872, 6954, 801, 24, 50, "Text"],
Cell[206676, 6980, 620, 18, 53, "Text"],
Cell[207299, 7000, 342, 11, 21, "Text"],
Cell[207644, 7013, 194, 6, 29, "Input",
 InitializationCell->True],
Cell[207841, 7021, 287, 6, 36, "Text"],
Cell[208131, 7029, 1054, 31, 109, "Input",
 InitializationCell->True],
Cell[209188, 7062, 339, 12, 21, "Text"],
Cell[209530, 7076, 251, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[209806, 7088, 124, 2, 29, "Input"],
Cell[209933, 7092, 95, 1, 28, "Output"]
}, Open  ]],
Cell[210043, 7096, 1542, 53, 74, "Text"],
Cell[211588, 7151, 141, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[211754, 7161, 114, 3, 28, "Input"],
Cell[211871, 7166, 73, 1, 28, "Output"]
}, Open  ]],
Cell[211959, 7170, 220, 9, 23, "Text"],
Cell[CellGroupData[{
Cell[212204, 7183, 125, 4, 28, "Input"],
Cell[212332, 7189, 71, 1, 28, "Output"]
}, Open  ]],
Cell[212418, 7193, 165, 7, 20, "Text"],
Cell[CellGroupData[{
Cell[212608, 7204, 129, 3, 29, "Input"],
Cell[212740, 7209, 96, 1, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[212885, 7216, 60, 1, 26, "Subsection",
 CellTags->"Section6c"],
Cell[212948, 7219, 243, 6, 35, "Text"],
Cell[213194, 7227, 415, 6, 50, "Text"],
Cell[213612, 7235, 332, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[213969, 7246, 39, 0, 18, "Subsubsection"],
Cell[214011, 7248, 1569, 58, 65, "Text"],
Cell[215583, 7308, 1362, 51, 60, "Text"],
Cell[216948, 7361, 1088, 33, 124, "Input",
 InitializationCell->True],
Cell[218039, 7396, 408, 11, 36, "Text"],
Cell[218450, 7409, 232, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[218707, 7422, 144, 4, 29, "Input",
 InitializationCell->True],
Cell[218854, 7428, 244, 7, 28, "Output"]
}, Open  ]],
Cell[219113, 7438, 48, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[219186, 7442, 139, 4, 29, "Input",
 InitializationCell->True],
Cell[219328, 7448, 141, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219506, 7456, 140, 4, 29, "Input",
 InitializationCell->True],
Cell[219649, 7462, 142, 3, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[219840, 7471, 33, 0, 18, "Subsubsection"],
Cell[219876, 7473, 630, 14, 65, "Text"],
Cell[220509, 7489, 858, 12, 65, "Text"],
Cell[221370, 7503, 271, 5, 36, "Text"],
Cell[CellGroupData[{
Cell[221666, 7512, 148, 3, 29, "Input",
 InitializationCell->True],
Cell[221817, 7517, 273, 6, 28, "Output"]
}, Open  ]],
Cell[222105, 7526, 314, 5, 50, "Text"],
Cell[CellGroupData[{
Cell[222444, 7535, 490, 15, 45, "Input",
 InitializationCell->True],
Cell[222937, 7552, 213, 5, 28, "Output"]
}, Open  ]],
Cell[223165, 7560, 194, 4, 21, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[223396, 7569, 35, 0, 18, "Subsubsection"],
Cell[223434, 7571, 552, 19, 35, "Text"],
Cell[223989, 7592, 585, 18, 61, "Input",
 InitializationCell->True],
Cell[224577, 7612, 312, 10, 37, "Text"],
Cell[224892, 7624, 107, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[225024, 7631, 157, 4, 29, "Input",
 InitializationCell->True],
Cell[225184, 7637, 213, 5, 28, "Output"]
}, Open  ]],
Cell[225412, 7645, 117, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[225554, 7652, 97, 2, 29, "Input"],
Cell[225654, 7656, 187, 5, 28, "Output"]
}, Open  ]],
Cell[225856, 7664, 212, 5, 21, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226105, 7674, 48, 0, 18, "Subsubsection"],
Cell[226156, 7676, 408, 12, 35, "Text"],
Cell[226567, 7690, 551, 15, 51, "Text"],
Cell[227121, 7707, 522, 10, 51, "Text"],
Cell[227646, 7719, 229, 6, 21, "Text"],
Cell[227878, 7727, 232, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[228135, 7737, 194, 6, 28, "Input"],
Cell[228332, 7745, 736, 12, 164, "Output"]
}, Open  ]],
Cell[229083, 7760, 202, 7, 21, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[229346, 7774, 140, 4, 35, "Section",
 PageBreakWithin->False,
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[229511, 7782, 117, 3, 32, "Subsection",
 PageBreakAbove->False,
 PageBreakWithin->False,
 CellTags->"SolutionsCP3"],
Cell[229631, 7787, 146, 3, 42, "Problem"],
Cell[229780, 7792, 282, 10, 20, "Text"],
Cell[230065, 7804, 481, 17, 26, "Equation"],
Cell[230549, 7823, 213, 8, 20, "Text"],
Cell[230765, 7833, 569, 20, 26, "Equation"],
Cell[231337, 7855, 549, 22, 29, "Text"],
Cell[231889, 7879, 932, 34, 45, "Equation"],
Cell[232824, 7915, 573, 23, 29, "Text"],
Cell[233400, 7940, 715, 28, 29, "Text"],
Cell[234118, 7970, 374, 11, 35, "Text"],
Cell[234495, 7983, 1173, 31, 170, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[235693, 8018, 70, 1, 29, "Input"],
Cell[235766, 8021, 145, 3, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[235960, 8030, 70, 1, 26, "Subsection",
 CellTags->"SolutionsCP21"],
Cell[236033, 8033, 96, 2, 42, "Problem"],
Cell[236132, 8037, 159, 5, 20, "Text"],
Cell[236294, 8044, 371, 12, 35, "Text"],
Cell[CellGroupData[{
Cell[236690, 8060, 223, 7, 28, "Input",
 InitializationCell->True],
Cell[236916, 8069, 100, 1, 28, "Output"]
}, Open  ]],
Cell[237031, 8073, 254, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[237310, 8083, 127, 3, 28, "Input",
 InitializationCell->True],
Cell[237440, 8088, 96, 1, 28, "Output"]
}, Open  ]],
Cell[237551, 8092, 433, 14, 38, "Text"],
Cell[CellGroupData[{
Cell[238009, 8110, 167, 4, 28, "Input",
 InitializationCell->True],
Cell[238179, 8116, 99, 1, 28, "Output"]
}, Open  ]],
Cell[238293, 8120, 213, 6, 21, "Text"],
Cell[238509, 8128, 671, 24, 38, "Text"],
Cell[CellGroupData[{
Cell[239205, 8156, 268, 8, 29, "Input",
 InitializationCell->True],
Cell[239476, 8166, 98, 1, 28, "Output"]
}, Open  ]],
Cell[239589, 8170, 1011, 33, 54, "Text"],
Cell[240603, 8205, 596, 21, 42, "Text"],
Cell[CellGroupData[{
Cell[241224, 8230, 168, 4, 29, "Input",
 InitializationCell->True],
Cell[241395, 8236, 100, 1, 28, "Output"]
}, Open  ]],
Cell[241510, 8240, 108, 3, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[241655, 8248, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE1"],
Cell[241738, 8251, 208, 7, 42, "Problem"],
Cell[241949, 8260, 666, 23, 35, "Text"],
Cell[242618, 8285, 959, 26, 139, "Input",
 InitializationCell->True],
Cell[243580, 8313, 241, 10, 20, "Text"],
Cell[CellGroupData[{
Cell[243846, 8327, 68, 1, 28, "Input"],
Cell[243917, 8330, 208, 5, 28, "Output"]
}, Open  ]],
Cell[244140, 8338, 566, 16, 51, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[244743, 8359, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE5"],
Cell[244826, 8362, 332, 11, 58, "Problem"],
Cell[245161, 8375, 270, 10, 20, "Text"],
Cell[245434, 8387, 918, 25, 121, "Input",
 InitializationCell->True],
Cell[246355, 8414, 183, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[246563, 8424, 58, 1, 28, "Input"],
Cell[246624, 8427, 341, 7, 44, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[247026, 8441, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[247103, 8446, 177, 6, 20, "Item1Exercise"],
Cell[247283, 8454, 202, 6, 20, "Item1Exercise"],
Cell[247488, 8462, 556, 21, 36, "Item1Exercise"],
Cell[248047, 8485, 1228, 43, 64, "Item1Exercise"],
Cell[249278, 8530, 1344, 43, 82, "Item1Exercise"],
Cell[250625, 8575, 389, 7, 47, "Item1Exercise"],
Cell[251017, 8584, 340, 8, 33, "Item1Exercise"],
Cell[251360, 8594, 853, 31, 46, "Item1Exercise"],
Cell[252216, 8627, 402, 9, 46, "Item1Exercise"],
Cell[252621, 8638, 368, 9, 33, "Item1Exercise"],
Cell[252992, 8649, 864, 28, 46, "Item1Exercise"],
Cell[253859, 8679, 2799, 105, 106, "Item1Exercise"],
Cell[256661, 8786, 2150, 78, 88, "Item1Exercise"],
Cell[258814, 8866, 382, 9, 46, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
